/*****************************************************************************\
* (c) Copyright 2000-2019 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
#ifndef FILTERTRUETRACKS_H
#define FILTERTRUETRACKS_H 1

// Include files
// from Gaudi
#include "GaudiAlg/GaudiAlgorithm.h"

#include "Event/MCParticle.h"
#include "Event/Track.h"
#include "Linker/LinkerTool.h"

typedef LinkerTool<LHCb::Track, LHCb::MCParticle>             Asct;
typedef Asct::DirectType                                      Table;
typedef Asct::Range                                           Range;
typedef Table::iterator                                       iter;
typedef std::vector<std::pair<const LHCb::MCParticle*, bool>> MCParts;
/** @class FilterTrueTracks FilterTrueTracks.h
 *
 *  Algorithm that keeps only true tracks associated
 *  to a given decay chain
 *
 *  @author Patrick Koppenburg
 *  @date   2007-08-17
 */
struct IMCDecayFinder;

#include "Event/Track.h"

// From LHCb
namespace LHCb {
  class IParticlePropertySvc;
}

class FilterTrueTracks : public GaudiAlgorithm {
public:
  /// Standard constructor
  FilterTrueTracks( const std::string& name, ISvcLocator* pSvcLocator );

  StatusCode initialize() override; ///< Algorithm initialization
  StatusCode execute() override;    ///< Algorithm execution

private:
  MCParts                  finalStates() const;
  bool                     isSignal( MCParts&, const Range& ) const;
  LHCb::Track::ConstVector signalTracks( MCParts& ) const;
  StatusCode               save( const LHCb::Track::ConstVector& ) const;

private:
  IMCDecayFinder*             m_mcDecFinder;    ///<  MCDecayFinder
  std::string                 m_mcParticlePath; ///< MC truth path
  std::vector<std::string>    m_tracksPath;     ///< Tracks input paths
  std::string                 m_outputPath;     ///< Tracks output paths
  bool                        m_filterOut;      ///< Revert logic and keep only non-signal?
  LHCb::IParticlePropertySvc* m_ppSvc;
};
#endif // FILTERTRUETRACKS_H
