/*****************************************************************************\
* (c) Copyright 2000-2019 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
// Include files
// from LHCb
#include "Event/MCParticle.h"
#include "MCInterfaces/IMCDecayFinder.h"
// local
#include "FilterEventByMCDecay.h"

//-----------------------------------------------------------------------------
// Implementation file for class : FilterEventByMCDecay
//
// 2009-04-28 : Juan PALACIOS
//-----------------------------------------------------------------------------

// Declaration of the Algorithm Factory
DECLARE_COMPONENT( FilterEventByMCDecay )

//=============================================================================
// Standard constructor, initializes variables
//=============================================================================
FilterEventByMCDecay::FilterEventByMCDecay( const std::string& name, ISvcLocator* pSvcLocator )
    : GaudiAlgorithm( name, pSvcLocator )
    , m_select( true )
    , m_decayFinder( 0 )
    , m_inputLocation( LHCb::MCParticleLocation::Default ) {
  declareProperty( "Select", m_select );
}

//=============================================================================
// Destructor
//=============================================================================
FilterEventByMCDecay::~FilterEventByMCDecay() {}

//=============================================================================
// Initialization
//=============================================================================
StatusCode FilterEventByMCDecay::initialize() {

  StatusCode sc = GaudiAlgorithm::initialize(); // must be executed first

  if ( sc.isFailure() ) return sc; // error printed already by GaudiAlgorithm

  if ( msgLevel( MSG::DEBUG ) ) debug() << "==> Initialize" << endmsg;

  m_decayFinder = tool<IMCDecayFinder>( "MCDecayFinder", this );

  return ( 0 != m_decayFinder ) ? StatusCode::SUCCESS : StatusCode::FAILURE;
}

//=============================================================================
// Main execution
//=============================================================================
StatusCode FilterEventByMCDecay::execute() {

  if ( msgLevel( MSG::DEBUG ) ) debug() << "==> Execute" << endmsg;

  LHCb::MCParticle::Container* mcParts_ = get<LHCb::MCParticle::Container>( m_inputLocation );

  const LHCb::MCParticle::ConstVector mcps( mcParts_->begin(), mcParts_->end() );

  const bool foundDecay = ( mcParts_ != 0 ) && ( m_decayFinder->hasDecay( mcps ) );

  const bool pass = ( m_select ) ? foundDecay : !foundDecay;

  setFilterPassed( pass );

  return StatusCode::SUCCESS;
}

//=============================================================================
