#ifndef USER_SingleBCandidateSelection_DStarMuNu_H 
#define USER_SingleBCandidateSelection_DStarMuNu_H 1

// from Gaudi
#include "Kernel/DaVinciAlgorithm.h"
#include "Kernel/IParticleDescendants.h"
#include "ITaggingUtilsChecker.h"

class IDistanceCalculator;
class DaVinciAlgorithm;

class IPVReFitter;

// from local

/** @class SingleBCandidateSelection_DStarMuNu SingleBCandidateSelection_DStarMuNu.h 
 *  
 *  Select events looking for the one candidate with best chi2
 *
 *  @author Marco Musy
 *  @date   02/11/2010
 */

class SingleBCandidateSelection_DStarMuNu : public DaVinciAlgorithm {

 public: 
  /// Standard constructor
  SingleBCandidateSelection_DStarMuNu( const std::string& name, ISvcLocator* pSvcLocator );

  virtual ~SingleBCandidateSelection_DStarMuNu( ); ///< Destructor

  virtual StatusCode initialize();    ///< Algorithm initialization
  virtual StatusCode execute   ();    ///< Algorithm execution
  virtual StatusCode finalize  ();    ///< Algorithm finalization

 private:
  const LHCb::Particle* findID(unsigned int id, LHCb::Particle::ConstVector& v, std::string s="");
  
  double get_MINIPCHI2(const LHCb::Particle* p);
  double get_BPVVDCHI2(const LHCb::Particle* B, const LHCb::Particle* P);
  double get_MIPDV(const LHCb::Particle* p);
  double get_IPOWNPV(const LHCb::Particle* p);
  

  double get_MINIPCHI2NEXTBEST(const LHCb::Particle* p);
  double get_MINIPNEXTBEST(const LHCb::Particle* p);

  std::string m_inputLocation;
  IParticleDescendants* m_descend;
  ITaggingUtilsChecker* m_util;

  IPVReFitter* m_pvReFitter;


};
//===========================================================================//
#endif // USER_SingleBCandidateSelection_DStarMuNu_H
