#define NEWCUTS
#ifndef GenericTool_H 
#define GenericTool_H 1

#include <Riostream.h>
#include <cmath>
#include "TString.h"

#include "taggingutils.h"
#include "globals.h"

#include "Particle.h"

#define endreq endl
#define endmsg endl


/** @class GenericTool
 *
 *  Generic tool containing common methods to tagger and combination tools
 *
 *  @author Marco Musy
 *  @date   2010-07-15
 */
class GenericTool {

public:
  /// Standard constructor
  GenericTool( ){

    MeV=0.001;
    GeV=1;
    TeV=1000;
    m_PIDe_cut_ele=m_PIDk_cut_kaon=m_PIDkp_cut_kaon=m_PIDm_cut_muon=0;
    m_PIDNoK_cut_pionS=m_PIDNoP_cut_pionS=0;
  }///< Constructor

  ~GenericTool( ){} ///< Destructor
 
  //=============================================
  double pol(double x, double a0, 
	     double a1=0, double a2=0, 
	     double a3=0, double a4=0) {
    double res = a0;
    if(a1) res += a1*x;
    if(a2) res += a2*x*x;
    if(a3) res += a3*x*x*x;
    if(a4) res += a4*x*x*x*x;
    return res;
  }///< return n-th degree polynomial by coefficients

  double func(double x, double a0=0,
              double a1=0, double a2=0,
              double a3=0){
   // TF1* arcTAN=new TF1("arcTAN","(atan((x-[3])*[0])+1.5)*[1]*0.17+[2]",0,1); 
  double arg = ((x-a3)*a0);
  double res = ((atan(arg))+1.5)*a1*0.17+a2;
  return res;  
  }


  /// check if particle properties satisfy a given PID hypothesis by looking at deltaLL of RICH, muon and calorimetric systems
  bool checkPIDhypo(Particle::PID s, Particle* a) {
    
    switch (s) {
    case Particle::muon :

      if(a->type()==3 
#ifdef NEWCUTS
         && a->isMuon()!=0         // needed for Reco14 sample
#endif
         && a->hasMuonFlag()!=0    // proto->info(ProtoParticle::MuonPIDStatus,0)
         && a->muonNShared()==0 
         && a->cloneDist()==-1 
         &&  a->PIDm() > m_PIDm_cut_muon 
         ) return true;
      return false;
    case Particle::electron :
      if( a->absIDtag()==321  )  return false;      
      
      if( a->inAccHcal() 
          && a->PIDe() > m_PIDe_cut_ele
          && a->type()==3 
          && a->cloneDist()==-1
          && a->isMuon()==0         // needed for Reco14 sample
          ) return true;      
      return false;
    case Particle::kaon_opposite :
      if(a->absIDtag()==13 )  return false;      
      if( a->PIDk()!=0 && a->PIDk()!=-1000.0  
          && a->type()==3 
          && a->PIDk() > m_PIDk_cut_kaon 
          && a->cloneDist()==-1 
          && a->PIDk()-a->PIDp() > m_PIDkp_cut_kaon
          && a->isMuon()==0         // needed for Reco14 sample

          )  return true;
      return false;

    case Particle::kaon_same :

      if( a->PIDk() 
          && a->PIDk()> m_PIDk_cut_kaonS 
          //&& a->cloneDist()==-1 
          //&& a->type()==3
          //&&( a->PIDk() - a->PIDp())> m_PIDkp_cut_kaonS 
          ) return true;
      return false;

    case Particle::pion :

      if( a->absIDtag()!=211)  return  false;
      if( a->absID()!=211)  return  false;
      if( a->type() !=3)    return false; // only long tracks
      //if (a->cloneDist()!=-1) return false;
      //if( a->hasMuonFlag() && a->muonNShared()==0 && a->PIDm() > m_PIDm_cut ) return false; 
      //if( a->inAccHcal() && a->PIDe() > m_PIDe_cut ) return false; 

      bool pidpass=false;
      if( a->PIDk()==0 ) pidpass=true;
      if( a->PIDk()!=0 ) {
        if(a->PIDk() < m_PIDNoK_cut_pionS &&
           a->PIDp() < m_PIDNoP_cut_pionS) pidpass=true;
      }
      return pidpass;
      
    }
    return false;
  }


  double MeV, GeV, TeV;

  //store the raw nnet output
  double m_rnet_mu, m_rnet_ele, m_rnet_k, m_rnet_kS, m_rnet_pS, m_rnet_vtx, m_rnet_fragOpp, m_rnet_combNN;

 protected:
  double m_PIDe_cut_ele,m_PIDk_cut_kaon,m_PIDkp_cut_kaon,
    m_PIDm_cut_muon,m_PIDNoK_cut_pionS, m_PIDNoP_cut_pionS,
    m_PIDNNm_cut_muon, m_PIDNNpi_cut_muon,m_PIDNNp_cut_muon, m_PIDNNk_cut_muon, m_PIDNNe_cut_muon, 
    m_ghostProb_muon,m_ghostProbNN_muon,

 m_ghostProb_ele,
    m_PIDNNe_cut_ele, m_PIDNNpi_cut_ele, m_PIDNNp_cut_ele,m_PIDNNk_cut_ele, m_PIDNNepi_cut_ele,
    m_PIDNNk_cut_kaon, m_PIDNNpi_cut_kaon, m_PIDNNp_cut_kaon,  m_PIDNNe_cut_kaon,  m_PIDNNm_cut_kaon,  m_PIDNNkp_cut_kaon,  m_PIDNNkpi_cut_kaon, 
m_ghostProb_kaon,
    m_PIDNNk_cut_pionS, m_PIDNNpi_cut_pionS, m_PIDNNp_cut_pionS, m_ghostProb_pionS;
  
  double m_PIDk_cut_kaonS, m_PIDkp_cut_kaonS;

  void print(std::vector<double>& par, double rnet, double pn) {
    if(DBGLEVEL>1) return;
    debug()<<" par =";
    for(int i=0; i<par.size(); ++i)debug()<<par.at(i)<<" ";
    debug()<<endmsg;
    debug()<<" rnet="<<rnet<<" pn="<<pn<<endmsg;
  }

};
#endif // GenericTool_H
