#include "CombineTaggersNN.h"
#include "taggingutils.h"
#include "globals.h"
#include "TMVA/Reader.h"
#include <list>

CombineTaggersNN::CombineTaggersNN() {

  declareProperty( "OmegaMaxBin_NN", m_omegamaxbin = 0.38 );
  declareProperty( "OmegaScale_NN",  m_omegascale  = 0.07 );
  declareProperty( "ProbMin_NN",  m_ProbMin     = 0.5 );
  //0.330113+/-0.00407095   p1=0.973957+/-0.0447826  p2=0.35649+/-0
  //0.392403+/-0.00267128   p1=1.01063+/-0.0298792  p2=0.382069+/-0
  declareProperty( "P0_NN",   m_P0_NN   = 0.4777);
  declareProperty( "P1_NN",   m_P1_NN   = 2.863);
  declareProperty( "P2_NN",   m_P2_NN   = 0.3896);

  theTag = new FlavourTag();
  NNetTool_MLP nnet;

}
CombineTaggersNN::~CombineTaggersNN() { 
  delete theTag;
}

/////////////////////////////////////////////////////////////
FlavourTag* CombineTaggersNN::combineTaggers( Taggers& vtg , Event& event ) {
  theTag->reset();

  theTag->setTaggers(vtg);

  if( vtg.empty() ) return theTag;

  int vtgsize = vtg.size();
  int abstagsum = 0;
  pmu=pe=pk=pss=pvtx=0.5; //NN variables (.h)
  int mutag, etag, ktag, sstag, vtxtag;
  mutag=etag=ktag=sstag=vtxtag=0;
  double pnsum=0.5;
  double probNN=0;

  //obtain pn(omega) and tagdec for each tagger
  for( int i = 0; i != vtgsize; i++ ) { //multiply all probabilities
    if (!vtg.at(i)->decision()) continue;
    int type = vtg.at(i)->type();
    if (type==2) {
      pmu = 1-(vtg.at(i))->omega(); //probability of 'right' mu
      mutag = vtg.at(i)->decision(); 
      if(msgLevel(MSG::DEBUG)) debug()<<"muon -> pmu: "<<pmu<<", mutag:"<<mutag<<endreq;
    }
    else if (type==3) {
      pe = 1-(vtg.at(i))->omega(); //probability of 'right' e
      etag = vtg.at(i)->decision(); 
      if(msgLevel(MSG::DEBUG)) debug()<<"ele -> pe: "<<pe<<", etag:"<<etag<<endreq;
    }
    else if (type==4) {
      pk = 1-(vtg.at(i))->omega(); //probability of 'right' k
      ktag = vtg.at(i)->decision(); 
      if(msgLevel(MSG::DEBUG)) debug()<<"kaon -> pk: "<<pk<<", ktag:"<<ktag<<endreq;
    }
    else if (type==5) {
      pss = 1-(vtg.at(i))->omega(); //probability of 'right' kaonSS
      sstag = vtg.at(i)->decision(); 
      if(msgLevel(MSG::DEBUG)) debug()<<"kaonSS -> pss: "<<pss<<", sstag:"<<sstag<<endreq;
    }
    else if (type==6){
      pss = 1-(vtg.at(i))->omega(); //probability of 'right' pionSS
      sstag = vtg.at(i)->decision(); 
      if(msgLevel(MSG::DEBUG)) debug()<<"pionSS -> pss: "<<pss<<", sstag:"<<sstag<<endreq;
    }
    else if (type==10){
      pvtx = 1-(vtg.at(i))->omega(); //probability of 'right' vtx
      vtxtag = vtg.at(i)->decision(); 
      if(msgLevel(MSG::DEBUG)) debug()<<"pvtx: "<<pvtx<<", vtxtag:"<<vtxtag<<endreq;
    }
    abstagsum = abs(mutag)+abs(etag)+abs(ktag)+abs(sstag)+abs(vtxtag);
  }

  double tagdecision=0;
  double pn=0.5;
  
  //only one tagger
  if (abstagsum==1){
     pn = pmu+pe+pk+pvtx - 1.5;
    tagdecision=mutag+etag+ktag+vtxtag;
    //pnsum=pmu+pe+pk+pss+pvtx-2;//sum all taggers minus 2(4 taggers inactive)
    //tagdecision=mutag+etag+ktag+sstag+vtxtag;
  }

  
  //more than one tagger
  if (abstagsum>1){
    //ch p[i] for tagdec=-1
    if (mutag<0) pmu=1-pmu;
    if (etag<0)  pe =1-pe;
    if (ktag<0)  pk =1-pk;
    if (sstag<0) pss=1-pss;
    if (vtxtag<0)pvtx=1-pvtx;

    probNN = 0.5;

    Particle* BSpart = event.signalB();

    list<std::pair<string, Float_t> > NNinputList;
    NNinputList.push_back(pair<string, Float_t>("om_muon", pmu));
    NNinputList.push_back(pair<string, Float_t>("om_ele",  pe));
    NNinputList.push_back(pair<string, Float_t>("om_kaon", pk));
    NNinputList.push_back(pair<string, Float_t>("om_vtx",  pvtx));

    pn = nnet.MLPCombNNTMVA( NNinputList);
    /*
    pn = 1. - pn;
    pn =  m_P0_NN + m_P1_NN*(pn - m_P2_NN );//tuning NN
    //pn =  0.3348 + 0.710*(pn - 0.3551 );//tuning NN
    //pn =  0.3658 + 0.995*(pn - 0.3820 );//tuning NN
    pn = 1. - pn;
    */
    
    //pn = 1. - pn;
    pn =  0.430 + 1.08*(pn - 0.378 );//tuning NN
    pn =  0.412 + 1.03*(pn - 0.387 );//tuning NN
    pn =  0.407 + 1.01*(pn - 0.391 );//tuning NN
    //pn = 1. - pn;
    
    if (pn>0.5) {tagdecision = +1;}
    if (pn<0.5) {tagdecision = -1; pn=1-pn;}
    
    //tagdecision = 1;
    
  }
  
  double omega = 1-pn;

  if ( omega < 0) omega = 0;
  if ( omega > 0.5 ) {
    tagdecision = -tagdecision;
    omega = 0.5 - omega;
  }
  pn = 1-omega;

  //sort decision into categories ------------------
  //cat=1 will be least reliable, cat=5 most reliable
  //ProbMin is a small offset to adjust for range of pnsum
  int category = 0;
  omega = fabs(1-pn);
  if(      omega > m_omegamaxbin                ) category=1;
  else if( omega > m_omegamaxbin-m_omegascale   ) category=2;
  else if( omega > m_omegamaxbin-m_omegascale*2 ) category=3;
  else if( omega > m_omegamaxbin-m_omegascale*3 ) category=4;
  else                                            category=5;
  if( !tagdecision ) category=0;

  //info
  if(msgLevel(MSG::DEBUG)) {
    debug() << "Final 1-w = " << pn <<endreq;
    debug() << "TagDecision = " << tagdecision <<endreq;
    debug() << "Category = " << category <<endreq;
    debug() << "Omega = " << omega <<endreq;
  }

  ///fill FlavourTag object
  if(      tagdecision ==  1 ) theTag->setDecisionOS( FlavourTag::bbar );
  else if( tagdecision == -1 ) theTag->setDecisionOS( FlavourTag::b );
  else theTag->setDecisionOS( FlavourTag::none );
  theTag->setCategoryOS( category );
  theTag->setOmegaOS( 1-pn );

  //fill in taggers info into FlavourTag object
  for(int j=0; j!=vtgsize; ++j) {
    int itag = vtg.at(j)->decision();
    if(itag) theTag->addTagger(vtg.at(j));
  }

  return theTag;
}
