#include "NNTuner.h"
#include "Variables.h"
NNTuner::NNTuner(TString& NNetTrain, int usesWeights , int useTMVA){

  name = NNetTrain;
  
  //usesWeights = 0;

  cout<<"\n --- INIT with sWeights: "<<usesWeights<<endl;
  name = NNetTrain;
  if(name=="none") return;

  //create tree of nnet variables:
  TString pref="output/nnet_",suff=".root";
  TString fname  = (pref+NNetTrain)+suff;
  fnnet = new TFile(fname, "RECREATE" );
  info()<<"Booking root file: "<< fname <<endmsg;
  
  hpidcat = new TH1F("hpidcat", "PID cat", 50, 0.0, 0.5);
  nntrain = new TTree("nntrain", fname);

  nntrain->Branch("iscorrect",  &iscorrect, "iscorrect/I");
  nntrain->Branch("tagger",     &tagger,  "tagger/I");
  nntrain->Branch("evtNum",     &evtNum,  "evtNum/I");
  nntrain->Branch("nnkrec",     &nnkrec,  "nnkrec/I");
  nntrain->Branch("mult",       &mult,    "mult/I");
  nntrain->Branch("ptB",        &ptB,     "ptB/F");
  nntrain->Branch("partP",      &partP,   "partP/F");
  nntrain->Branch("partPt",     &partPt,  "partPt/F");

  nntrain->Branch("IPPU",       &IPPU,    "IPPU/F");
  nntrain->Branch("PIDm",       &PIDm,    "PIDm/F");
  nntrain->Branch("PIDe",       &PIDe,    "PIDe/F");
  nntrain->Branch("PIDk",       &PIDk,    "PIDk/F");
  nntrain->Branch("PIDpk",      &PIDpk,   "PIDpk/F");
  nntrain->Branch("eOverP",     &eOverP,  "eOverP/F");
  nntrain->Branch("veloch",     &veloch,  "veloch/F");
  
  nntrain->Branch("PIDNNe",     &PIDNNe,  "PIDNNe/F");
  nntrain->Branch("PIDNNm",     &PIDNNm,  "PIDNNm/F");
  nntrain->Branch("PIDNNpi",    &PIDNNpi, "PIDNNpi/F");
  nntrain->Branch("PIDNNk",     &PIDNNk,  "PIDNNk/F");
  nntrain->Branch("PIDNNp",     &PIDNNp,  "PIDNNp/F");

  nntrain->Branch("IPs",        &IPs,      "IPs/F");
  nntrain->Branch("IPssign",    &IPssign,  "IPssign/F");
  nntrain->Branch("partlcs",    &partlcs,  "partlcs/F");
  nntrain->Branch("partTheta",  &partTheta,"partTheta/F");
  nntrain->Branch("partPhi",    &partPhi,  "partPhi/F");  
  nntrain->Branch("ghostProb",  &ghostProb,"ghostProb/F");

  nntrain->Branch("nndr",       &nndr,    "nndr/F");
  nntrain->Branch("nndeta",     &nndeta,  "nndeta/F");
  nntrain->Branch("nndphi",     &nndphi,  "nndphi/F");
  nntrain->Branch("nndq",       &nndq,    "nndq/F");
  nntrain->Branch("eta",        &eta,     "eta/F");

  
  nntrain->Branch("vflag",      &vflag,   "vflag/F");
  nntrain->Branch("ptmean",     &ptmean,  "ptmean/F");
  nntrain->Branch("ipsmean",    &ipsmean, "ipsmean/F");
  nntrain->Branch("ipsmean2",   &ipsmean2,"ipsmean2/F");
  nntrain->Branch("ptsum",      &ptsum,   "ptsum/F");
  nntrain->Branch("docamax",    &docamax, "docamax/F");
  nntrain->Branch("maxprobf",   &maxprobf,"maxprobf/F");
  nntrain->Branch("vratio",     &vratio,  "vratio/F");
  nntrain->Branch("vcharge",    &vcharge, "vcharge/F");
  nntrain->Branch("svtau",      &svtau,   "svtau/F");
  nntrain->Branch("svm",        &svm,     "svm/F");
  nntrain->Branch("BDphiDir",   &BDphiDir,"BDphiDir/F");
  nntrain->Branch("svp",        &svp,     "svp/F");

  
  nntrain->Branch("om_muon",    &om_muon, "om_muon/F");
  nntrain->Branch("om_ele",     &om_ele,  "om_ele/F");
  nntrain->Branch("om_kaon",    &om_kaon, "om_kaon/F");
  nntrain->Branch("om_same",    &om_same, "om_same/F");
  nntrain->Branch("om_vtx",     &om_vtx,  "om_vtx/F");
  nntrain->Branch("N_sig_sw",   &N_sig_sw,  "N_sig_sw/F");
  
  for (long i =0; i<111112;i++) {
    pidvec_right.push_back(0);
    pidvec_wrong.push_back(0);
    pidvec_omega.push_back(0);
    pidvec_sumomega.push_back(0);
    pidvec_count.push_back(0);
    pidvec_averageomega.push_back(0);
  }
  
}

/////////////////////////////////////////////////////////////////
void NNTuner::Fill(Event& event, FlavourTag* thetag, int useTMVA){

  //  cout<<" Fill NNTuner ======== "<<name<<endl;
  
  if(name=="none") return;

  Taggers taggers = thetag->taggers();
  Tagger* tag=NULL; 


  if(     name == "muon")  tag=taggers.at(0);
  else if(name == "ele" )  tag=taggers.at(1);
  else if(name == "kaon")  tag=taggers.at(2);
  else if(name == "kaonS") tag=taggers.at(3);
  else if(name == "pionS") tag=taggers.at(4);
  else if(name == "vtx")   tag=taggers.at(5);
  else if(name == "combNN"){

    Particle* BSpart = event.signalB();
    evtNum = (event.eventNr()%3);
    

    iscorrect = ((event.TrueTag()>0) ? 1 : 0);   // signal = B+, background = B-
    int sum = abs(taggers.at(0)->decision());
    sum    += abs(taggers.at(1)->decision());
    sum    += abs(taggers.at(2)->decision());
    //sum    += abs(taggers.at(3)->decision());//KaonSS
    //sum    += abs(taggers.at(4)->decision());//PionSS
    sum    += abs(taggers.at(5)->decision());
    if(sum<2) return;

    om_muon=0.;  om_ele=0.;  om_kaon = 0.;  om_vtx = 0.;
    
    if (taggers.at(0)->decision()!=0) om_muon= taggers.at(0)->omega();
    if (taggers.at(0)->decision()==1) om_muon = 1 - om_muon;
    if (taggers.at(1)->decision()!=0) om_ele = taggers.at(1)->omega();
    if (taggers.at(1)->decision()==1) om_ele = 1 - om_ele;
    if (taggers.at(2)->decision()!=0) om_kaon= taggers.at(2)->omega();
    if (taggers.at(2)->decision()==1) om_kaon = 1 - om_kaon;
    //om_same= taggers.at(3)->omega();
    //if (taggers.at(3)->decision()==1) om_same = 1 - om_same;
    if (taggers.at(5)->decision()!=0) om_vtx = taggers.at(5)->omega();
    if (taggers.at(5)->decision()==1) om_vtx = 1 - om_vtx;

    //cout<<endl<<"eventNum "<<event.eventNr()<<" om_muon "<<om_muon<<" om_ele "<<om_ele<<" om_kaon "<<om_kaon<<" om_vtx "<<om_vtx<<endl;

    mult    = event.multiplicity();
    ptB     = log(event.signalB()->pt());
    nnkrec  = event.pileup();
    eta = log(tan(BSpart->Theta()/2.));


    N_sig_sw  = event.N_sig_sw();
    nntrain->Fill();// <-- fills branch
    return;
  }
  else if(name =="combPID") {
    cout<<" Please verify the correctness of the code BRFORE RUNNING !!! Check if needs to be updated"<<endl;
    return;  // Stefania
    
    int tmuon_dec = taggers.at(0)->decision();
    int tele_dec  = taggers.at(1)->decision();
    int tkaon_dec = taggers.at(2)->decision();
    int tsame_dec = taggers.at(3)->decision();
    int tvtx_dec  = taggers.at(4)->decision();
    //build vector for combPID
    int tagsum=tmuon_dec+tele_dec+tkaon_dec+tsame_dec+tvtx_dec; //Sum of all possible taggers
    if (tagsum) {
      int piddecision = tagsum>0 ? 1:-1;
      int indexoftagger = abs(tmuon_dec)*10000+abs(tele_dec)*1000+abs(tkaon_dec)*100+abs(tsame_dec)*10+abs(tvtx_dec)*1;
      if (abs(tagsum)>1) indexoftagger += 100000;
      if(piddecision==event.TrueTag()) pidvec_right.at(indexoftagger)++;
      else pidvec_wrong.at(indexoftagger)++;
      //Omega from fit
      double ofm, ofe, ofk, ofss, ofvtx;
      ofm = ofe = ofk = ofss = ofvtx = 0.5;
      //mc
      if (taggers.at(0)->decision()!=0) ofm  = 0.31;
      if (taggers.at(1)->decision()!=0) ofe  = 0.36;
      if (taggers.at(2)->decision()!=0) ofk  = 0.38;
      if (taggers.at(3)->decision()!=0) ofss = 0.39;
      if (taggers.at(4)->decision()!=0) ofvtx= 0.46;
      if (taggers.at(0)->decision()==-1) ofm  =1-ofm;
      if (taggers.at(1)->decision()==-1) ofe  =1-ofe;
      if (taggers.at(2)->decision()==-1) ofk  =1-ofk;
      if (taggers.at(3)->decision()==-1) ofss =1-ofss;
      if (taggers.at(4)->decision()==-1) ofvtx=1-ofvtx;
      double pfplus=ofm*ofe*ofk*ofss*ofvtx;
      double pfmin=(1-ofm)*(1-ofe)*(1-ofk)*(1-ofss)*(1-ofvtx);
      double pftot=pfplus/(pfplus+pfmin);
      if (pftot<0.5) pftot = 1-pftot;
      pidvec_sumomega.at(indexoftagger)+=(1-pftot);
      pidvec_count.at(indexoftagger)++;
    }
  }
  else err()<<"in NNTuner: unknown tagger: "<<name<<endmsg;

  
  if(!tag) return;

  if(tag->decision()) {
    Particle* BSpart = event.signalB();
    Particle* tagpart= tag->taggerparts().at(0);
    iscorrect = (tag->decision() == event.TrueTag()) ? 1 : 0 ;

    evtNum = (event.eventNr()%3);
    
    //Get variables for NN training
    mult    = event.multiplicity();
    ptB     = log(event.signalB()->pt());
    partP   = log(tagpart->p());
    partPt  = log(tagpart->pt());
    //IPPV    = tagpart->IPs();
    IPPU    = log(tagpart->IPPU());
    //New
    PIDm    = tagpart->PIDm();
    PIDe    = tagpart->PIDe();
    PIDk    = tagpart->PIDk();
    PIDpk   = tagpart->PIDp() - tagpart->PIDk();
    eOverP  = tagpart->eOverP();
    veloch  = tagpart->VeloCharge();
    PIDNNe  = tagpart->PIDNNe();
    PIDNNm  = tagpart->PIDNNm();
    PIDNNpi = tagpart->PIDNNpi();
    PIDNNk  = tagpart->PIDNNk();
    PIDNNp  = tagpart->PIDNNp();

    IPs     = log(tagpart->IPs());//
    //IPssign = tagpart->IPssign();//
    double sign=1;
    if(tagpart->IPssign()<0) sign=-1.;
    IPssign = sign*sqrt(abs(tagpart->IPssign()));
    partlcs = log(tagpart->LCS());//
    partTheta = tagpart->Theta();//
    partPhi   = tagpart->Phi();//
    N_sig_sw  = event.N_sig_sw();
    ghostProb = -100;
    if(tagpart->ghostProbability() !=0)
      ghostProb = log(tagpart->ghostProbability());

    nndeta  = log(tan(BSpart->Theta()/2.))
             -log(tan(tagpart->Theta()/2.)); 
    nndphi  = min(fabs(tagpart->momentum().Phi()-BSpart->Phi()),
                  float(2.*PI)-fabs(tagpart->momentum().Phi()-BSpart->Phi()));
    nndr    = sqrt(nndeta*nndeta +nndphi*nndphi);
    nndq    = (BSpart->momentum() + tagpart->momentum()).M() 
             - BSpart->momentum().M();
    nnkrec  = event.pileup();

    eta = log(tan(BSpart->Theta()/2.));

    //Get variables for Vtx Charge
    Vertices mySVs = event.getSecondaryVertices();
    if(name == "vtx") {      
      if(mySVs.empty()) return;
      Vertex* mySV = mySVs.at(0);
      
      vflag    = mySV->getVflagged();
      ptmean   = log(mySV->getVptmean());
      ptsum    = log(mySV->getVptmean()*mySV->getVflagged());
      ipsmean  = log(mySV->getVipsmean());
      ipsmean2 = mySV->getVipsmean()*mySV->getVflagged();
      docamax  = mySV->getVdocamax();
      maxprobf = mySV->likelihood();
      vratio   = mySV->getVratio();
      vcharge  = fabs(mySV->getVCharge());
      svtau    = log(mySV->getSVtau());
      svm      = log(mySV->getSVm());
      BDphiDir = mySV->getBDphiDir();
      svp      = log(mySV->getSVP());
    } else {
      vflag    = 0.;
      ptmean   = 0.;
      ptsum    = 0.;
      ipsmean  = 0.;
      ipsmean2 = 0.;
      docamax  = 0.;
      maxprobf = 0.;
      vratio   = 0.;
      vcharge  = 0.;
      svtau    = 0.;
      svm      = 0.;
      BDphiDir = 0.;
      svp      = 0.;
    }
    
    nntrain->Fill();// <-- fills branch

  }

}

//////////////////////////////////////////////////////////////////////////////////

void NNTuner::TrainNNetTMVA(int usesWeights , TString& VarTransform) {
  //VarTransform = "D,N";  
  cout<<endl<<"Inside TrainTMVA "<<VarTransform;
  
  TString outfileName;  
  TString inputfileName;
  Variables InputList;
  
  if(name=="muon") {
    cout<<"\nWill train NNet using TMVA MLP for tagger: "<<name<<endl;
    
    // Default MVA methods to be trained + tested
    
    std::map<std::string,int> Use;
    
    Use["MLPBNN"]          = 1; // Recommended ANN with BFGS training method and bayesian regulator
    
    outfileName="output/nnet_"+name+"_TMVA.root";
    
    TFile* outputFile = TFile::Open( outfileName, "RECREATE" );
    
    TString TransfView="I";
    if (VarTransform!="")
      TransfView =  TransfView + ";D;N";
    //TransfView =  TransfView + ";" + VarTransform;
    
    //TransfView=TransfView+";N";
    
    TString Factory_def = "!V:!Silent:Color:Transformations="+TransfView+":DrawProgressBar:AnalysisType=Classification";

    TMVA::Factory *factory = new TMVA::Factory( "muon", outputFile,Factory_def );

    //Input: 
    inputfileName="output/nnet_"+name+".root";
    TChain *signal = new TChain("nntrain");
    signal->Add(inputfileName);    //signal

    TChain *background = new TChain("nntrain"); 
    background->Add(inputfileName);        //bkg
    
    Double_t signalWeight     = 1.0;
    Double_t backgroundWeight = 1.0;

    /* // Random Split
    factory->AddSignalTree    ( signal,     signalWeight );
    factory->AddBackgroundTree( background, backgroundWeight );
    */
    TTree *sig_train =signal->CopyTree("iscorrect==1 && evtNum%3==1");
    factory->AddTree(sig_train,"Signal",signalWeight,"","Train");
    TTree *sig_test =signal->CopyTree("iscorrect==1 && evtNum%3==0");
    factory->AddTree(sig_test,"Signal",signalWeight,"","Test");
    TTree *bkg_train =signal->CopyTree("iscorrect==0 && evtNum%3==1");
    factory->AddTree(bkg_train,"Background",signalWeight,"","Train");
    TTree *bkg_test =signal->CopyTree("iscorrect==0 && evtNum%3==0");
    factory->AddTree(bkg_test,"Background",signalWeight,"","Test");
    /*
    factory->AddTree(signal,"Signal",signalWeight,"iscorrect==1 && evtNum%2!=0","Train");
    factory->AddTree(signal,"Signal",signalWeight,"iscorrect==1 && evtNum%2==0","Test");
    factory->AddTree(signal,"Background",signalWeight,"iscorrect==0 && evtNum%2!=0","Train");
    factory->AddTree(signal,"Background",signalWeight,"iscorrect==0 && evtNum%2==0","Test");
    */
    vector<string>::const_iterator it;
    
    vector<string> muonlist = InputList.init_Muon(); 
#ifdef NNETcat
    TString variables_Cat1 = "";
    TString variables_Cat2 = "";
    for(it = muonlist.begin(); it != muonlist.end(); it++) {
      if(*it == "mult" || *it == "nnkrec") {
        factory->AddVariable(*it, *it, "I");
        variables_Cat1 += *it;
        variables_Cat1 += ":";
      } else {
        factory->AddVariable(*it, *it, "F");
        variables_Cat1 += *it;
        variables_Cat1 += ":";
      }
    }
   
    variables_Cat1 = variables_Cat1.Remove(variables_Cat1.Length() - 1);
    variables_Cat2 = variables_Cat1;
    cout << endl << "Variables_Cat1 "<<variables_Cat1<< "Variables_Cat2 "<<variables_Cat2;

#else    
    for(it = muonlist.begin(); it != muonlist.end(); it++) {
      if(*it == "nnkrec" || *it == "mult" ) factory->AddVariable(*it, *it, "I");
      else factory->AddVariable(*it, *it, "F");
      cout <<endl<< *it << " first variable for Muon (Inside Tuner)"<<endl;
    }
#endif
   
    TCut mycuts = "iscorrect==1";   
    TCut mycutb = "iscorrect==0";
    
    //should be in tag.opts
    
    TString Transformations = "";
    if (VarTransform!="") Transformations =  ":VarTransform="+ VarTransform;    
    cout<<endl<<"Transformations "<<Transformations;  
 
    //TString NeuronType("sigmoid");
    TString NeuronType("tanh");
    TString TrainingMethod("BFGS");

    //TString MLP_def = "H:!V:NeuronType="+NeuronType+Transformations+":NCycles=80:HiddenLayers=N+5:TestRate=5:TrainingMethod="+ TrainingMethod+":UseRegulator";
    TString MLP_def = "H:!V:NeuronType="+NeuronType+Transformations+":NCycles=180:HiddenLayers=N+5:TestRate=5:TrainingMethod="+ TrainingMethod+":UseRegulator:EstimatorType=CE";
   
    cout<<endl<<"MLP Definition "<<endl<<MLP_def<<endl;


    if (usesWeights == 1)factory->SetWeightExpression( "N_sig_sw" );

    int n_sig_train = sig_train->GetEntries();
    int n_sig_test  = sig_test->GetEntries();
    int n_bkg_train = bkg_train->GetEntries();
    int n_bkg_test  = bkg_test->GetEntries();
    
    string optstring = Form("nTrain_Signal=%d:nTest_Signal=%d:nTrain_Background=%d:nTest_Background=%d:SplitMode=Block:NormMode=NumEvents:!V",
                            n_sig_train, n_sig_test, n_bkg_train, n_bkg_test); 

    //factory->PrepareTrainingAndTestTree( mycuts, mycutb,"nTrain_Signal=0:nTrain_Background=0:NormMode=NumEvents:!V" );
    //factory->PrepareTrainingAndTestTree( mycuts, mycutb,"nTrain_Signal=0:nTrain_Background=0:SplitMode=Random:NormMode=NumEvents:!V" );

#ifdef NNETcat   
    TMVA::MethodCategory* mcat = 0;
 
    TMVA::MethodBase* cat = factory->BookMethod( TMVA::Types::kCategory, "_muonMLPBNN_Cat", "");
    mcat = dynamic_cast<TMVA::MethodCategory*>(cat);

    mcat->AddMethod("nnkrec==1", "mult:partP:partPt:ptB:IPs:partlcs:PIDm:PIDNNm:ghostProb", TMVA::Types::kMLP,"_muonMLPBNN_Cat_nnkrec_1", MLP_def);
    mcat->AddMethod("nnkrec>1", "mult:partP:partPt:nnkrec:ptB:IPs:partlcs:PIDm:PIDNNm:ghostProb:IPPU", TMVA::Types::kMLP,"_muonMLPBNN_Cat_nnkrec_greater_than_1", MLP_def);
    factory->PrepareTrainingAndTestTree( mycuts, mycutb,optstring.c_str() );
#else
    factory->PrepareTrainingAndTestTree( mycuts, mycutb,optstring.c_str() );
    factory->BookMethod( TMVA::Types::kMLP, "_muonMLPBNN",  MLP_def ); // BFGS training with bayesian regulators
#endif
    // Train MVAs using the set of training events
    factory->TrainAllMethods();
    
    // ---- Evaluate all MVAs using the set of test events
    factory->TestAllMethods();
    
    // ----- Evaluate and compare performance of all configured MVAs
    factory->EvaluateAllMethods();
    
    // --------------------------------------------------------------
    
    // Save the output
    outputFile->Close();
    
  }

  if(name=="ele") {
    cout<<"\nWill train NNet using TMVA MLP for tagger: "<<name<<endl;
    
    // Default MVA methods to be trained + tested
    
    std::map<std::string,int> Use;
    
    Use["MLPBNN"]          = 1; // Recommended ANN with BFGS training method and bayesian regulator
    
    outfileName="output/nnet_"+name+"_TMVA.root";
    
    TFile* outputFile = TFile::Open( outfileName, "RECREATE" );
    
    TString TransfView="I";
    if (VarTransform!="")
      TransfView =  TransfView + ";D;N";
    //TransfView =  TransfView + ";" + VarTransform;
    
    //TransfView=TransfView+";N";
    
    TString Factory_def = "!V:!Silent:Color:Transformations="+TransfView+":DrawProgressBar:AnalysisType=Classification"; 
    TMVA::Factory *factory = new TMVA::Factory( "ele", outputFile,Factory_def );
    
    //Input: 
    inputfileName="output/nnet_"+name+".root";
    TChain *signal = new TChain("nntrain");
    signal->Add(inputfileName);    //signal
    
    TChain *background = new TChain("nntrain"); 
    background->Add(inputfileName);        //bkg

    Double_t signalWeight     = 1.0;
    Double_t backgroundWeight = 1.0;

    /* // Random Split
    factory->AddSignalTree    ( signal,     signalWeight );
    factory->AddBackgroundTree( background, backgroundWeight );
    */
    TTree *sig_train =signal->CopyTree("iscorrect==1 && evtNum%3==1");
    factory->AddTree(sig_train,"Signal",signalWeight,"","Train");
    TTree *sig_test =signal->CopyTree("iscorrect==1 && evtNum%3==0");
    factory->AddTree(sig_test,"Signal",signalWeight,"","Test");
    TTree *bkg_train =signal->CopyTree("iscorrect==0 && evtNum%3==1");
    factory->AddTree(bkg_train,"Background",signalWeight,"","Train");
    TTree *bkg_test =signal->CopyTree("iscorrect==0 && evtNum%3==0");
    factory->AddTree(bkg_test,"Background",signalWeight,"","Test");
    /*
    factory->AddTree(signal,"Signal",signalWeight,"iscorrect==1 && evtNum%2!=0","Train");
    factory->AddTree(signal,"Signal",signalWeight,"iscorrect==1 && evtNum%2==0","Test");
    factory->AddTree(signal,"Background",signalWeight,"iscorrect==0 && evtNum%2!=0","Train");
    factory->AddTree(signal,"Background",signalWeight,"iscorrect==0 && evtNum%2==0","Test");

     */
    vector<string>::const_iterator it;

    vector<string> elelist = InputList.init_Ele();

#ifdef NNETcat
    TString variables_Cat1 = "";
    TString variables_Cat2 = "";
    for(it = elelist.begin(); it != elelist.end(); it++) {
      if(*it == "mult" || *it == "nnkrec") {
        factory->AddVariable(*it, *it, "I");
        variables_Cat1 += *it;
        variables_Cat1 += ":";
      } else {
        factory->AddVariable(*it, *it, "F");
        variables_Cat1 += *it;
        variables_Cat1 += ":";
      }
    }
   
    variables_Cat1 = variables_Cat1.Remove(variables_Cat1.Length() - 1);
    variables_Cat2 = variables_Cat1;
    cout << endl << "Variables_Cat1 "<<variables_Cat1<< "Variables_Cat2 "<<variables_Cat2;

#else
    for(it = elelist.begin(); it != elelist.end(); it++) {
      if(*it == "nnkrec" || *it == "mult" ) factory->AddVariable(*it, *it, "I");
      else factory->AddVariable(*it, *it, "F");
      cout <<endl<< *it << " first variable for Electron (Inside Tuner)"<<endl;
    }
#endif

    TCut mycuts = "iscorrect==1";   
    TCut mycutb = "iscorrect==0";
   
    TString Transformations = "";
    if (VarTransform!="")
      //Transformations =  VarTransform +","+ Transformations;
      Transformations =  ":VarTransform="+ VarTransform;

    cout<<endl<<"Transformations Applied "<<Transformations;
    
    //TString NeuronType("tanh");//sigmoid");
    TString NeuronType("sigmoid");
    TString TrainingMethod("BFGS");
   
    TString MLP_def = "H:!V:NeuronType="+NeuronType+Transformations+":NCycles=80:HiddenLayers=N+5:TestRate=5:TrainingMethod="+ TrainingMethod+":UseRegulator";
    //TString MLP_def = "H:!V:NeuronType="+NeuronType+Transformations+":NCycles=80:HiddenLayers=N+5:TestRate=5:TrainingMethod="+ TrainingMethod+":UseRegulator:EstimatorType=CE";
   

    if (usesWeights == 1){
      factory->SetWeightExpression( "N_sig_sw" );
    }
    int n_sig_train = sig_train->GetEntries();
    int n_sig_test  = sig_test->GetEntries();
    int n_bkg_train = bkg_train->GetEntries();
    int n_bkg_test  = bkg_test->GetEntries();
    
    string optstring = Form("nTrain_Signal=%d:nTest_Signal=%d:nTrain_Background=%d:nTest_Background=%d:SplitMode=Block:NormMode=NumEvents:!V", 
		    n_sig_train, n_sig_test, n_bkg_train, n_bkg_test); 


    //factory->PrepareTrainingAndTestTree( mycuts, mycutb,"nTrain_Signal=0:nTrain_Background=0:NormMode=NumEvents:!V" );
    //factory->PrepareTrainingAndTestTree( mycuts, mycutb,"nTrain_Signal=0:nTrain_Background=0:SplitMode=Random:NormMode=NumEvents:!V" );
#ifdef NNETcat
    TMVA::MethodCategory* mcat = 0;
 
    TMVA::MethodBase* cat = factory->BookMethod( TMVA::Types::kCategory, "_eleMLPBNN_Cat", "");
    mcat = dynamic_cast<TMVA::MethodCategory*>(cat);

    mcat->AddMethod("nnkrec==1", "mult:partP:partPt:ptB:IPs:partlcs:PIDe:PIDNNpi:PIDNNp:veloch", TMVA::Types::kMLP,"_eleMLPBNN_Cat_nnkrec_1", MLP_def);
    mcat->AddMethod("nnkrec>1", "mult:partP:partPt:nnkrec:ptB:IPs:partlcs:PIDe:PIDNNpi:PIDNNp:veloch:IPPU", TMVA::Types::kMLP,"_eleMLPBNN_Cat_nnkrec_greater_than_1", MLP_def);
    factory->PrepareTrainingAndTestTree( mycuts, mycutb,optstring.c_str() );
#else
    factory->PrepareTrainingAndTestTree( mycuts, mycutb,optstring.c_str() );
    factory->BookMethod( TMVA::Types::kMLP, "_eleMLPBNN",  MLP_def ); // BFGS training with bayesian regulators
#endif
    // Train MVAs using the set of training events
    factory->TrainAllMethods();

    // ---- Evaluate all MVAs using the set of test events
    factory->TestAllMethods();

    // ----- Evaluate and compare performance of all configured MVAs
    factory->EvaluateAllMethods();

    // --------------------------------------------------------------

    // Save the output
    outputFile->Close();

  }
  if(name=="kaon") {
    cout<<"\nWill train NNet using TMVA MLP for tagger: "<<name<<endl;

    // Default MVA methods to be trained + tested

    std::map<std::string,int> Use;

    Use["MLPBNN"]          = 1; // Recommended ANN with BFGS training method and bayesian regulator

    outfileName="output/nnet_"+name+"_TMVA.root";

    TFile* outputFile = TFile::Open( outfileName, "RECREATE" );

    TString TransfView="I";
    if (VarTransform!="")
      TransfView =  TransfView + ";D;N";
    //TransfView =  TransfView + ";" + VarTransform;

    //TransfView=TransfView+";N";
 
    TString Factory_def = "!V:!Silent:Color:Transformations="+TransfView+":DrawProgressBar:AnalysisType=Classification"; 
    TMVA::Factory *factory = new TMVA::Factory( "kaon", outputFile,Factory_def );

    //Input: 
    inputfileName="output/nnet_"+name+".root";
    TChain *signal = new TChain("nntrain");
    signal->Add(inputfileName);    //signal

    TChain *background = new TChain("nntrain"); 
    background->Add(inputfileName);        //bkg

    Double_t signalWeight     = 1.0;
    Double_t backgroundWeight = 1.0;


    /* // Random Split
    factory->AddSignalTree    ( signal,     signalWeight );
    factory->AddBackgroundTree( background, backgroundWeight );
    */
    TTree *sig_train =signal->CopyTree("iscorrect==1 && evtNum%3==1");
    factory->AddTree(sig_train,"Signal",signalWeight,"","Train");
    TTree *sig_test =signal->CopyTree("iscorrect==1 && evtNum%3==0");
    factory->AddTree(sig_test,"Signal",signalWeight,"","Test");
    TTree *bkg_train =signal->CopyTree("iscorrect==0 && evtNum%3==1");
    factory->AddTree(bkg_train,"Background",signalWeight,"","Train");
    TTree *bkg_test =signal->CopyTree("iscorrect==0 && evtNum%3==0");
    factory->AddTree(bkg_test,"Background",signalWeight,"","Test");
    /*
    factory->AddTree(signal,"Signal",signalWeight,"iscorrect==1 && evtNum%3==1","Train");
    factory->AddTree(signal,"Signal",signalWeight,"iscorrect==1 && evtNum%3==0","Test");
    factory->AddTree(signal,"Background",signalWeight,"iscorrect==0 && evtNum%3==1","Train");
    factory->AddTree(signal,"Background",signalWeight,"iscorrect==0 && evtNum%3==0","Test");
    */
   
    vector<string>::const_iterator it;

    vector<string> kaonlist = InputList.init_Kaon();


#ifdef NNETcat
    TString variables_Cat1 = "";
    TString variables_Cat2 = "";
    for(it = kaonlist.begin(); it != kaonlist.end(); it++) {
      if(*it == "mult" || *it == "nnkrec") {
        factory->AddVariable(*it, *it, "I");
        variables_Cat1 += *it;
        variables_Cat1 += ":";
      } else {
        factory->AddVariable(*it, *it, "F");
        variables_Cat1 += *it;
        variables_Cat1 += ":";
      }
    }
   
    variables_Cat1 = variables_Cat1.Remove(variables_Cat1.Length() - 1);
    variables_Cat2 = variables_Cat1;
    cout << endl << "Variables_Cat1 "<<variables_Cat1<< "Variables_Cat2 "<<variables_Cat2;

#else    
    for(it = kaonlist.begin(); it != kaonlist.end(); it++) {
      if(*it == "nnkrec" || *it == "mult" ) factory->AddVariable(*it, *it, "I");
      else factory->AddVariable(*it, *it, "F");

      //factory->AddVariable(*it, *it, "F");
      cout <<endl<< *it << " first variable for Kaon (Inside Tuner)"<<endl;
    }
#endif
    TCut mycuts = "iscorrect==1";   
    TCut mycutb = "iscorrect==0";
   

    TString Transformations = "";
    if (VarTransform!="")
      //Transformations =  VarTransform +","+ Transformations;
      Transformations =  ":VarTransform="+ VarTransform;


    cout<<endl<<"Transformations Applied "<<Transformations;
    
    TString NeuronType("tanh");//sigmoid");
    TString TrainingMethod("BFGS");
   
    //TString MLP_def = "H:!V:NeuronType="+NeuronType+Transformations+":NCycles=100:HiddenLayers=N+5:TestRate=5:TrainingMethod="+ TrainingMethod+":UseRegulator";
    TString MLP_def = "H:!V:NeuronType="+NeuronType+Transformations+":NCycles=180:HiddenLayers=N+5:TestRate=5:TrainingMethod="+ TrainingMethod+":UseRegulator:EstimatorType=CE";
   

    if (usesWeights == 1){
      factory->SetWeightExpression( "N_sig_sw" );
    }
    int n_sig_train = sig_train->GetEntries();
    int n_sig_test  = sig_test->GetEntries();
    int n_bkg_train = bkg_train->GetEntries();
    int n_bkg_test  = bkg_test->GetEntries();
    
    string optstring = Form("nTrain_Signal=%d:nTest_Signal=%d:nTrain_Background=%d:nTest_Background=%d:SplitMode=Block:NormMode=NumEvents:!V", 
		    n_sig_train, n_sig_test, n_bkg_train, n_bkg_test); 

    //factory->PrepareTrainingAndTestTree( mycuts, mycutb,"nTrain_Signal=0:nTrain_Background=0:SplitMode=Block:NormMode=NumEvents:!V" );
    //factory->PrepareTrainingAndTestTree( mycuts, mycutb,"nTrain_Signal=0:nTrain_Background=0:SplitMode=Random:NormMode=NumEvents:!V" );
#ifdef NNETcat
    TMVA::MethodCategory* mcat = 0;
 
    TMVA::MethodBase* cat = factory->BookMethod( TMVA::Types::kCategory, "_kaonMLPBNN_Cat", "");
    mcat = dynamic_cast<TMVA::MethodCategory*>(cat);

    mcat->AddMethod("nnkrec==1", "mult:partP:partPt:ptB:IPs:partlcs:PIDk:PIDpk:PIDNNk:PIDNNpi:PIDNNp:ghostProb", TMVA::Types::kMLP,"_kaonMLPBNN_Cat_nnkrec_1", MLP_def);
    mcat->AddMethod("nnkrec>1", "mult:partP:partPt:nnkrec:ptB:IPs:partlcs:PIDk:PIDpk:PIDNNk:PIDNNpi:PIDNNp:ghostProb:IPPU", TMVA::Types::kMLP,"_kaonMLPBNN_Cat_nnkrec_greater_than_1", MLP_def);
    factory->PrepareTrainingAndTestTree( mycuts, mycutb,optstring.c_str() );
#else
    factory->PrepareTrainingAndTestTree( mycuts, mycutb,optstring.c_str() );
    factory->BookMethod( TMVA::Types::kMLP, "_kaonMLPBNN",  MLP_def ); // BFGS training with bayesian regulators
#endif
    // Train MVAs using the set of training events
    factory->TrainAllMethods();

    // ---- Evaluate all MVAs using the set of test events
    factory->TestAllMethods();

    // ----- Evaluate and compare performance of all configured MVAs
    factory->EvaluateAllMethods();

    // --------------------------------------------------------------

    // Save the output
    outputFile->Close();

  }

  if(name=="pionS") {
    cout<<"\nWill train NNet using TMVA MLP for tagger: "<<name<<endl;

    // Default MVA methods to be trained + tested

    std::map<std::string,int> Use;

    Use["MLPBNN"]          = 1; // Recommended ANN with BFGS training method and bayesian regulator

    outfileName="output/nnet_"+name+"_TMVA.root";

    TFile* outputFile = TFile::Open( outfileName, "RECREATE" );

    TString TransfView="I";
    if (VarTransform!="")
      TransfView =  TransfView + ";D;N";
    //TransfView =  TransfView + ";" + VarTransform;
      //TransfView =  TransfView +","+ VarTransform;

    //TransfView=TransfView+";N";
 
    TString Factory_def = "!V:!Silent:Color:Transformations="+TransfView+":DrawProgressBar:AnalysisType=Classification"; 
    TMVA::Factory *factory = new TMVA::Factory( "pionS", outputFile,Factory_def );

    //Input: 
    inputfileName="output/nnet_"+name+".root";
    TChain *signal = new TChain("nntrain");
    signal->Add(inputfileName);    //signal

    TChain *background = new TChain("nntrain"); 
    background->Add(inputfileName);        //bkg

    Double_t signalWeight     = 1.0;
    Double_t backgroundWeight = 1.0;

    /* // Random Split
    factory->AddSignalTree    ( signal,     signalWeight );
    factory->AddBackgroundTree( background, backgroundWeight );
    */
    TTree *sig_train =signal->CopyTree("iscorrect==1 && evtNum%3==1");
    factory->AddTree(sig_train,"Signal",signalWeight,"","Train");
    TTree *sig_test =signal->CopyTree("iscorrect==1 && evtNum%3==0");
    factory->AddTree(sig_test,"Signal",signalWeight,"","Test");
    TTree *bkg_train =signal->CopyTree("iscorrect==0 && evtNum%3==1");
    factory->AddTree(bkg_train,"Background",signalWeight,"","Train");
    TTree *bkg_test =signal->CopyTree("iscorrect==0 && evtNum%3==0");
    factory->AddTree(bkg_test,"Background",signalWeight,"","Test");

    vector<string>::const_iterator it;

    vector<string> pionSlist = InputList.init_pionS();

#ifdef NNETcat
    TString variables_Cat1 = "";
    TString variables_Cat2 = "";
    for(it = pionSlist.begin(); it != pionSlist.end(); it++) {
      if(*it == "mult" || *it == "nnkrec") {
        factory->AddVariable(*it, *it, "I");
        variables_Cat1 += *it;
        variables_Cat1 += ":";
      } else {
        factory->AddVariable(*it, *it, "F");
        variables_Cat1 += *it;
        variables_Cat1 += ":";
      }
    }
   
    variables_Cat1 = variables_Cat1.Remove(variables_Cat1.Length() - 1);
    variables_Cat2 = variables_Cat1;
    cout << endl << "Variables_Cat1 "<<variables_Cat1<< "Variables_Cat2 "<<variables_Cat2;

#else
    for(it = pionSlist.begin(); it != pionSlist.end(); it++) {

      if(*it == "nnkrec" || *it == "mult" ) factory->AddVariable(*it, *it, "I");
      else factory->AddVariable(*it, *it, "F");
      //      factory->AddVariable(*it, *it, "F");
      cout <<endl<< *it << " first variable for pionS (Inside Tuner)"<<endl;
    }
#endif

    TCut mycuts = "iscorrect==1";   
    TCut mycutb = "iscorrect==0";
   
    TString Transformations = "";
    if (VarTransform!="")
      Transformations =  ":VarTransform="+ VarTransform;

    cout<<endl<<"Transformations Applied "<<Transformations;
    
    TString NeuronType("tanh");//sigmoid");
    TString TrainingMethod("BFGS");
   
    //    TString MLP_def = "H:!V:NeuronType="+NeuronType+Transformations+":NCycles=60:HiddenLayers=N+5:TestRate=5:TrainingMethod="+ TrainingMethod+":UseRegulator";
    TString MLP_def = "H:!V:NeuronType="+NeuronType+Transformations+":NCycles=180:HiddenLayers=N+5:TestRate=5:TrainingMethod="+ TrainingMethod+":UseRegulator:EstimatorType=CE";
   

    if (usesWeights == 1){
      factory->SetWeightExpression( "N_sig_sw" );
    }

    int n_sig_train = sig_train->GetEntries();
    int n_sig_test  = sig_test->GetEntries();
    int n_bkg_train = bkg_train->GetEntries();
    int n_bkg_test  = bkg_test->GetEntries();
    
    string optstring = Form("nTrain_Signal=%d:nTest_Signal=%d:nTrain_Background=%d:nTest_Background=%d:SplitMode=Block:NormMode=NumEvents:!V", 
		    n_sig_train, n_sig_test, n_bkg_train, n_bkg_test); 

    factory->PrepareTrainingAndTestTree( mycuts, mycutb,optstring.c_str() );
    //factory->PrepareTrainingAndTestTree( mycuts, mycutb,"nTrain_Signal=0:nTrain_Background=0:NormMode=NumEvents:!V" );
    //factory->PrepareTrainingAndTestTree( mycuts, mycutb,"nTrain_Signal=0:nTrain_Background=0:SplitMode=Random:NormMode=NumEvents:!V" );
    factory->BookMethod( TMVA::Types::kMLP, "_pionSMLPBNN",  MLP_def ); // BFGS training with bayesian regulators
    // Train MVAs using the set of training events
    factory->TrainAllMethods();

    // ---- Evaluate all MVAs using the set of test events
    factory->TestAllMethods();

    // ----- Evaluate and compare performance of all configured MVAs
    factory->EvaluateAllMethods();

    // --------------------------------------------------------------

    // Save the output
    outputFile->Close();

  }

  if(name=="vtx") {
    cout<<"\nWill train NNet using TMVA MLP for tagger: "<<name<<endl;

    // Default MVA methods to be trained + tested

    std::map<std::string,int> Use;

    Use["MLPBNN"]          = 1; // Recommended ANN with BFGS training method and bayesian regulator

    outfileName="output/nnet_"+name+"_TMVA.root";

    TFile* outputFile = TFile::Open( outfileName, "RECREATE" );

    TString TransfView="I";
    if (VarTransform!="")
      TransfView =  TransfView + ";D;N";
    //TransfView =  TransfView + ";" + VarTransform;

    //TransfView=TransfView+";N";
 
    TString Factory_def = "!V:!Silent:Color:Transformations="+TransfView+":DrawProgressBar:AnalysisType=Classification"; 
    TMVA::Factory *factory = new TMVA::Factory( "vtx", outputFile,Factory_def );

    //Input: 
    inputfileName="output/nnet_"+name+".root";
    TChain *signal = new TChain("nntrain");
    signal->Add(inputfileName);    //signal

    TChain *background = new TChain("nntrain"); 
    background->Add(inputfileName);        //bkg

    Double_t signalWeight     = 1.0;
    Double_t backgroundWeight = 1.0;

    /* // Random Split
    factory->AddSignalTree    ( signal,     signalWeight );
    factory->AddBackgroundTree( background, backgroundWeight );
    */
    TTree *sig_train =signal->CopyTree("iscorrect==1 && evtNum%3==1");
    factory->AddTree(sig_train,"Signal",signalWeight,"","Train");
    TTree *sig_test =signal->CopyTree("iscorrect==1 && evtNum%3==0");
    factory->AddTree(sig_test,"Signal",signalWeight,"","Test");
    TTree *bkg_train =signal->CopyTree("iscorrect==0 && evtNum%3==1");
    factory->AddTree(bkg_train,"Background",signalWeight,"","Train");
    TTree *bkg_test =signal->CopyTree("iscorrect==0 && evtNum%3==0");
    factory->AddTree(bkg_test,"Background",signalWeight,"","Test");
    /*
    factory->AddTree(signal,"Signal",signalWeight,"iscorrect==1 && evtNum%2!=0","Train");
    factory->AddTree(signal,"Signal",signalWeight,"iscorrect==1 && evtNum%2==0","Test");
    factory->AddTree(signal,"Background",signalWeight,"iscorrect==0 && evtNum%2!=0","Train");
    factory->AddTree(signal,"Background",signalWeight,"iscorrect==0 && evtNum%2==0","Test");
    */
    vector<string>::const_iterator it;

    vector<string> vtxlist = InputList.init_vtx();

#ifdef NNETcat
    TString variables_Cat1 = "";
    TString variables_Cat2 = "";
    for(it = vtxlist.begin(); it != vtxlist.end(); it++) {
      if(*it == "mult" || *it == "nnkrec") {
        factory->AddVariable(*it, *it, "I");
        variables_Cat1 += *it;
        variables_Cat1 += ":";
      } else {
        factory->AddVariable(*it, *it, "F");
        variables_Cat1 += *it;
        variables_Cat1 += ":";
      }
    }
   
    variables_Cat1 = variables_Cat1.Remove(variables_Cat1.Length() - 1);
    variables_Cat2 = variables_Cat1;
    cout << endl << "Variables_Cat1 "<<variables_Cat1<< "Variables_Cat2 "<<variables_Cat2;

#else
    for(it = vtxlist.begin(); it != vtxlist.end(); it++) {

      if(*it == "nnkrec" || *it == "mult" ) factory->AddVariable(*it, *it, "I");
      else factory->AddVariable(*it, *it, "F");
      //      factory->AddVariable(*it, *it, "F");
      cout <<endl<< *it << " first variable for Vertex (Inside Tuner)"<<endl;
    }
#endif

    TCut mycuts = "iscorrect==1";   
    TCut mycutb = "iscorrect==0";
   
    TString Transformations = "";
    if (VarTransform!="")
      //Transformations =  VarTransform +","+ Transformations;
      Transformations =  ":VarTransform="+ VarTransform;

    cout<<endl<<"Transformations Applied "<<Transformations;
    
    TString NeuronType("tanh");//sigmoid");
    TString TrainingMethod("BFGS");
   
    //    TString MLP_def = "H:!V:NeuronType="+NeuronType+Transformations+":NCycles=60:HiddenLayers=N+5:TestRate=5:TrainingMethod="+ TrainingMethod+":UseRegulator";
    TString MLP_def = "H:!V:NeuronType="+NeuronType+Transformations+":NCycles=180:HiddenLayers=N+5:TestRate=5:TrainingMethod="+ TrainingMethod+":UseRegulator:EstimatorType=CE";
   

    if (usesWeights == 1){
      factory->SetWeightExpression( "N_sig_sw" );
    }
    int n_sig_train = sig_train->GetEntries();
    int n_sig_test  = sig_test->GetEntries();
    int n_bkg_train = bkg_train->GetEntries();
    int n_bkg_test  = bkg_test->GetEntries();
    
    string optstring = Form("nTrain_Signal=%d:nTest_Signal=%d:nTrain_Background=%d:nTest_Background=%d:SplitMode=Block:NormMode=NumEvents:!V", 
		    n_sig_train, n_sig_test, n_bkg_train, n_bkg_test); 

    factory->PrepareTrainingAndTestTree( mycuts, mycutb,optstring.c_str() );
    //factory->PrepareTrainingAndTestTree( mycuts, mycutb,"nTrain_Signal=0:nTrain_Background=0:SplitMode=Random:NormMode=NumEvents:!V" );
    //factory->PrepareTrainingAndTestTree( mycuts, mycutb,"nTrain_Signal=0:nTrain_Background=0:NormMode=NumEvents:!V" );

#ifdef NNETcat
    // not needed to split according the n. of cat (here there is no IPPU)
#endif
    factory->BookMethod( TMVA::Types::kMLP, "_vtxMLPBNN",  MLP_def ); // BFGS training with bayesian regulators
    
    // Train MVAs using the set of training events
    factory->TrainAllMethods();

    // ---- Evaluate all MVAs using the set of test events
    factory->TestAllMethods();

    // ----- Evaluate and compare performance of all configured MVAs
    factory->EvaluateAllMethods();

    // --------------------------------------------------------------

    // Save the output
    outputFile->Close();

  }
  if(name=="kaonS") {
    cout<<"\nWill train NNet using TMVA MLP for tagger: "<<name<<endl;

    // Default MVA methods to be trained + tested

    std::map<std::string,int> Use;

    Use["MLPBNN"]          = 1; // Recommended ANN with BFGS training method and bayesian regulator

    outfileName="output/nnet_"+name+"_TMVA.root";

    TFile* outputFile = TFile::Open( outfileName, "RECREATE" );

    TString TransfView="I";
    if (VarTransform!="")
      TransfView =  TransfView + ";D;N";
    //TransfView =  TransfView + ";" + VarTransform;
      //TransfView =  TransfView +","+ VarTransform;

    //TransfView=TransfView+";N";
 
    TString Factory_def = "!V:!Silent:Color:Transformations="+TransfView+":DrawProgressBar:AnalysisType=Classification"; 
    TMVA::Factory *factory = new TMVA::Factory( "kaonS", outputFile,Factory_def );

    //Input: 
    inputfileName="output/nnet_"+name+".root";
    TChain *signal = new TChain("nntrain");
    signal->Add(inputfileName);    //signal

    TChain *background = new TChain("nntrain"); 
    background->Add(inputfileName);        //bkg

    Double_t signalWeight     = 1.0;
    Double_t backgroundWeight = 1.0;

    /* // Random Split
    factory->AddSignalTree    ( signal,     signalWeight );
    factory->AddBackgroundTree( background, backgroundWeight );
    */
    TTree *sig_train =signal->CopyTree("iscorrect==1 && evtNum%3==1");
    factory->AddTree(sig_train,"Signal",signalWeight,"","Train");
    TTree *sig_test =signal->CopyTree("iscorrect==1 && evtNum%3==0");
    factory->AddTree(sig_test,"Signal",signalWeight,"","Test");
    TTree *bkg_train =signal->CopyTree("iscorrect==0 && evtNum%3==1");
    factory->AddTree(bkg_train,"Background",signalWeight,"","Train");
    TTree *bkg_test =signal->CopyTree("iscorrect==0 && evtNum%3==0");
    factory->AddTree(bkg_test,"Background",signalWeight,"","Test");
    /*
    factory->AddTree(signal,"Signal",signalWeight,"iscorrect==1 && evtNum%2!=0","Train");
    factory->AddTree(signal,"Signal",signalWeight,"iscorrect==1 && evtNum%2==0","Test");
    factory->AddTree(signal,"Background",signalWeight,"iscorrect==0 && evtNum%2!=0","Train");
    factory->AddTree(signal,"Background",signalWeight,"iscorrect==0 && evtNum%2==0","Test");
    */

    vector<string>::const_iterator it;

    vector<string> kaonSlist = InputList.init_kaonS();

#ifdef NNETcat
    TString variables_Cat1 = "";
    TString variables_Cat2 = "";
    for(it = kaonSlist.begin(); it != kaonSlist.end(); it++) {
      if(*it == "mult" || *it == "nnkrec") {
        factory->AddVariable(*it, *it, "I");
        variables_Cat1 += *it;
        variables_Cat1 += ":";
      } else {
        factory->AddVariable(*it, *it, "F");
        variables_Cat1 += *it;
        variables_Cat1 += ":";
      }
    }
   
    variables_Cat1 = variables_Cat1.Remove(variables_Cat1.Length() - 1);
    variables_Cat2 = variables_Cat1;
    cout << endl << "Variables_Cat1 "<<variables_Cat1<< "Variables_Cat2 "<<variables_Cat2;

#else    
    for(it = kaonSlist.begin(); it != kaonSlist.end(); it++) {
      if(*it == "nnkrec" || *it == "mult" ) factory->AddVariable(*it, *it, "I");
      else factory->AddVariable(*it, *it, "F");
      //      factory->AddVariable(*it, *it, "F");
      cout <<endl<< *it << " first variable for kaonS (Inside Tuner)"<<endl;
    }
#endif

    TCut mycuts = "iscorrect==1";   
    TCut mycutb = "iscorrect==0";
   
    TString Transformations = "";
    if (VarTransform!="")
      //Transformations =  VarTransform +","+ Transformations;
      Transformations =  ":VarTransform="+ VarTransform;

    cout<<endl<<"Transformations Applied "<<Transformations;
    
    TString NeuronType("tanh");//sigmoid");
    TString TrainingMethod("BFGS");
   
    //    TString MLP_def = "H:!V:NeuronType="+NeuronType+Transformations+":NCycles=60:HiddenLayers=N+5:TestRate=5:TrainingMethod="+ TrainingMethod+":UseRegulator";
    TString MLP_def = "H:!V:NeuronType="+NeuronType+Transformations+":NCycles=180:HiddenLayers=N+5:TestRate=5:TrainingMethod="+ TrainingMethod+":UseRegulator:EstimatorType=CE";
   

    if (usesWeights == 1){
      factory->SetWeightExpression( "N_sig_sw" );
    }
    int n_sig_train = sig_train->GetEntries();
    int n_sig_test  = sig_test->GetEntries();
    int n_bkg_train = bkg_train->GetEntries();
    int n_bkg_test  = bkg_test->GetEntries();
    
    string optstring = Form("nTrain_Signal=%d:nTest_Signal=%d:nTrain_Background=%d:nTest_Background=%d:SplitMode=Block:NormMode=NumEvents:!V", 
		    n_sig_train, n_sig_test, n_bkg_train, n_bkg_test); 

    factory->PrepareTrainingAndTestTree( mycuts, mycutb,optstring.c_str() );
    //factory->PrepareTrainingAndTestTree( mycuts, mycutb,"nTrain_Signal=0:nTrain_Background=0:NormMode=NumEvents:!V" );
    //factory->PrepareTrainingAndTestTree( mycuts, mycutb,"nTrain_Signal=0:nTrain_Background=0:SplitMode=Random:NormMode=NumEvents:!V" );
    factory->BookMethod( TMVA::Types::kMLP, "_kaonSMLPBNN",  MLP_def ); // BFGS training with bayesian regulators
    // Train MVAs using the set of training events
    factory->TrainAllMethods();

    // ---- Evaluate all MVAs using the set of test events
    factory->TestAllMethods();

    // ----- Evaluate and compare performance of all configured MVAs
    factory->EvaluateAllMethods();

    // --------------------------------------------------------------

    // Save the output
    outputFile->Close();
    
  }
  //Combination
  if(name=="combNN") {
    cout<<"\nWill train NNet using TMVA MLP for tagger: "<<name<<endl;

    // Default MVA methods to be trained + tested

    std::map<std::string,int> Use;

    Use["MLPBNN"]          = 1; // Recommended ANN with BFGS training method and bayesian regulator

    outfileName="output/nnet_"+name+"_TMVA.root";

    TFile* outputFile = TFile::Open( outfileName, "RECREATE" );

    TString TransfView="I";
    if (VarTransform!="")
      TransfView =  TransfView + ";D;N";
    //TransfView =  TransfView + ";" + VarTransform;

    //TransfView=TransfView+";N";

    TString Factory_def = "!V:!Silent:Color:Transformations="+TransfView+":DrawProgressBar:AnalysisType=Classification";

    TMVA::Factory *factory = new TMVA::Factory( "combNN", outputFile,Factory_def );

    //Input:
    inputfileName="output/nnet_"+name+".root";
    TChain *signal = new TChain("nntrain");
    signal->Add(inputfileName);    //signal

    TChain *background = new TChain("nntrain");
    background->Add(inputfileName);        //bkg

    Double_t signalWeight     = 1.0;
    Double_t backgroundWeight = 1.0;

    /* // Random Split
    factory->AddSignalTree    ( signal,     signalWeight );
    factory->AddBackgroundTree( background, backgroundWeight );
    */
    //Right tagged
    TTree *sig_train =signal->CopyTree("iscorrect==1 && evtNum%3==1");
    //factory->AddTree(sig_train,"Signal",signalWeight,"","Train");
    TTree *sig_test =signal->CopyTree("iscorrect==1 && evtNum%3==0");
    //factory->AddTree(sig_test,"Signal",signalWeight,"","Test");

    //Wrong tagged
    TTree *bkg_train =signal->CopyTree("iscorrect==0 && evtNum%3==1");
    //factory->AddTree(bkg_train,"Background",signalWeight,"","Train");
    TTree *bkg_test =signal->CopyTree("iscorrect==0 && evtNum%3==0");
    //factory->AddTree(bkg_test,"Background",signalWeight,"","Test");

    factory->AddTree(signal,"Signal",signalWeight,"iscorrect==1 && evtNum%3==1","Train");
    factory->AddTree(signal,"Signal",signalWeight,"iscorrect==1 && evtNum%3==0","Test");
    factory->AddTree(signal,"Background",signalWeight,"iscorrect==0 && evtNum%3==1","Train");
    factory->AddTree(signal,"Background",signalWeight,"iscorrect==0 && evtNum%3==0","Test");

    vector<string>::const_iterator it;

    vector<string> combNNlist = InputList.init_combNN();



    for(it = combNNlist.begin(); it != combNNlist.end(); it++) {
      factory->AddVariable(*it, *it, "F");
      cout <<endl<< *it << " first variable for combNN (Inside Tuner)"<<endl;
    }

    TCut mycuts = "iscorrect==1";
    TCut mycutb = "iscorrect==0";

    //should be in tag.opts

    TString Transformations = "";
    if (VarTransform!="") Transformations =  ":VarTransform="+ VarTransform;
    cout<<endl<<"Transformations "<<Transformations;

    TString NeuronType("sigmoid");
    TString TrainingMethod("BFGS");

    TString MLP_def = "H:!V:NeuronType="+NeuronType+Transformations+":NCycles=180:HiddenLayers=N+5:TestRate=5:TrainingMethod="+ TrainingMethod+":UseRegulator";
    //TString MLP_def = "H:!V:NeuronType="+NeuronType+Transformations+":NCycles=180:HiddenLayers=N+5:TestRate=5:TrainingMethod="+ TrainingMethod+":UseRegulator:EstimatorType=CE";

    cout<<endl<<"MLP Definition "<<endl<<MLP_def<<endl;


    if (usesWeights == 1)factory->SetWeightExpression( "N_sig_sw" );

    int n_sig_train = sig_train->GetEntries();
    int n_sig_test  = sig_test->GetEntries();
    int n_bkg_train = bkg_train->GetEntries();
    int n_bkg_test  = bkg_test->GetEntries();

    string optstring = Form("nTrain_Signal=%d:nTest_Signal=%d:nTrain_Background=%d:nTest_Background=%d:SplitMode=Block:NormMode=NumEvents:!V",
		    n_sig_train, n_sig_test, n_bkg_train, n_bkg_test);

    cout << endl << "optstring "<<optstring;

    factory->PrepareTrainingAndTestTree( mycuts, mycutb,optstring.c_str() );
    //factory->PrepareTrainingAndTestTree( mycuts, mycutb,"nTrain_Signal=0:nTrain_Background=0:NormMode=NumEvents:!V" );
    //factory->PrepareTrainingAndTestTree( mycuts, mycutb,"nTrain_Signal=0:nTrain_Background=0:SplitMode=Random:NormMode=NumEvents:!V" );

#ifdef NNETcat
    TMVA::MethodCategory* mcat = 0;

    TMVA::MethodBase* cat = factory->BookMethod( TMVA::Types::kCategory, "_combNNMLPBNN_Cat", "");
    mcat = dynamic_cast<TMVA::MethodCategory*>(cat);

    mcat->AddMethod("nnkrec==1", "mult:partP:partPt:ptB:IPs:partlcs:PIDm:PIDNNm:ghostProb", TMVA::Types::kMLP,"_combNNMLPBNN_Cat_nnkrec_1", MLP_def);
    mcat->AddMethod("nnkrec>1", "mult:partP:partPt:nnkrec:ptB:IPs:partlcs:PIDm:PIDNNm:ghostProb:IPPU", TMVA::Types::kMLP,"_combNNMLPBNN_Cat_nnkrec_greater_than_1", MLP_def);
#else
    factory->BookMethod( TMVA::Types::kMLP, "_combNNMLPBNN",  MLP_def ); // BFGS training with bayesian regulators
#endif
    // Train MVAs using the set of training events
    factory->TrainAllMethods();

    // ---- Evaluate all MVAs using the set of test events
    factory->TestAllMethods();

    // ----- Evaluate and compare performance of all configured MVAs
    factory->EvaluateAllMethods();

    // --------------------------------------------------------------

    // Save the output
    outputFile->Close();

  }



  exittuner_TMVA();
}




//////////////////////////////////////////////////////////////////////////////////
void NNTuner::exittuner_TMVA() {
  //  pSnnet->Write(); pSnnet->Close();
  fnnet->Close();
  if(name=="combPID") {
    info()<<"Copy the vector in CombPID, and choose the optimal cuts to spread into 5 cat"<<endl;
  }
  else if(name!="") {
    cout<<system("rm -v output/tag.root"); //to avoid confusion

    info()<<BOLD<<SUBAMARILLO<<"\n    INSTRUCTIONS on how to proceed:";
    info()<<ROJO<<"\n 1. Quit root, set NNetTrain=none in tag.opts "
          <<" \nissue \ncp weights/"<<name<<"__"<<name<<"MLPBNN.class.C "
          <<"nnet/"<<name<<"_MLPBNN.class.C\n"<<" and rerun. \n     Ignore any error message."<<endl;
//    info()<<VIOLETA<<" 2. Update the fit values of the polynomials p0, p1"
//          <<" into src/NNetTool_MLP.cxx"<<endl;
    info()<<AZUL<<" 3. Finally rerun and look at the new combined performance."
          <<ENDC<<endl<<endl;
    return;
  }
}



