#include "configuration.h"
/*
This file contains normalization functions and functions to calculate NN output for each tagger.
*/
#include <algorithm>
#include "NNetTool_MLP.h"
//=============================================================================
NNetTool_MLP::NNetTool_MLP( ) {

  // cuts_Moriond2013 trained on CloneKO, lowCorrelation, newNNPID D,N, Split i%3=0,1  
  /*
    tag.BTaggingTool.CombineTaggersProbability.ProbMin  = 0.
    tag.BTaggingTool.CombineTaggersProbability.ProbMin_OS  = 0.5
    tag.BTaggingTool.CombineTaggersProbability.P0_Cal_OS  = 0.390
    tag.BTaggingTool.CombineTaggersProbability.P1_Cal_OS  = 0.899
    tag.BTaggingTool.CombineTaggersProbability.Eta_Cal_OS = 0.365
  

  //mu scaleX=-2.16396 scaleY=2.57925 offsetY=-0.352321 pivotX=0.648114
  declareProperty( "P0_mu_scale", m_P0mu =  -2.16396);
  declareProperty( "P1_mu_scale", m_P1mu =  2.57925);
  declareProperty( "P2_mu_scale", m_P2mu =  -0.352321);
  declareProperty( "P3_mu_scale", m_P3mu =  0.648114);
  //e scaleX=-3.29727 scaleY=1.59377 offsetY=-0.111448 pivotX=0.654335
  declareProperty( "P0_e_scale", m_P0e =  -3.29727);
  declareProperty( "P1_e_scale", m_P1e =  1.59377);
  declareProperty( "P2_e_scale", m_P2e =  -0.111448);
  declareProperty( "P3_e_scale", m_P3e =  0.654335);
  //k scaleX=-3.50981 scaleY=1.77512 offsetY=-0.0539218 pivotX=0.575734
  declareProperty( "P0_k_scale", m_P0k =  -3.50981);
  declareProperty( "P1_k_scale", m_P1k =  1.77512);
  declareProperty( "P2_k_scale", m_P2k =  -0.0539218);
  declareProperty( "P3_k_scale", m_P3k =  0.575734);
  //vtx scaleX=-6.4019 scaleY=1.09482 offsetY=0.119658 pivotX=0.575981
  declareProperty( "P0_vtx_scale", m_P0vtx =  -6.4019);
  declareProperty( "P1_vtx_scale", m_P1vtx =  1.09482);
  declareProperty( "P2_vtx_scale", m_P2vtx =  0.119658);
  declareProperty( "P3_vtx_scale", m_P3vtx =  0.575981);
*/
  // cuts_Moriond2013 trained on CloneKO, lowCorrelation, newNNPID D,N, CE tanh, Split i%3=0,1  
  //mu scaleX=-5.47039 scaleY=1.23885 offsetY=-0.00793716 pivotX=0.647253
declareProperty( "P0_mu_scale", m_P0mu =  -5.47039);
declareProperty( "P1_mu_scale", m_P1mu =  1.23885);
declareProperty( "P2_mu_scale", m_P2mu =  -0.00793716);
declareProperty( "P3_mu_scale", m_P3mu =  0.647253);

//e scaleX=-3.04032 scaleY=1.7055 offsetY=-0.136785 pivotX=0.646733
declareProperty( "P0_e_scale", m_P0e =  -3.04032);
declareProperty( "P1_e_scale", m_P1e =  1.7055);
declareProperty( "P2_e_scale", m_P2e =  -0.136785);
declareProperty( "P3_e_scale", m_P3e =  0.646733);
//k scaleX=-5.12122 scaleY=1.21427 offsetY=0.088854 pivotX=0.573943
declareProperty( "P0_k_scale", m_P0k =  -5.12122);
declareProperty( "P1_k_scale", m_P1k =  1.21427);
declareProperty( "P2_k_scale", m_P2k =  0.088854);
declareProperty( "P3_k_scale", m_P3k =  0.573943);
//vtx scaleX=-5.77134 scaleY=1.11591 offsetY=0.113682 pivotX=0.570684
declareProperty( "P0_vtx_scale", m_P0vtx =  -5.77134);
declareProperty( "P1_vtx_scale", m_P1vtx =  1.11591);
declareProperty( "P2_vtx_scale", m_P2vtx =  0.113682);
declareProperty( "P3_vtx_scale", m_P3vtx =  0.570684);
  // 
  declareProperty( "P0_ks_scale", m_P0ks =  1.22418); //dec2011_v2
  declareProperty( "P1_ks_scale", m_P1ks = -1.63297);  
  declareProperty( "P2_ks_scale", m_P2ks =  0.401361);
  
  //scaleX=-0.143512 saleY=36.8308 offsetY=-8.96514 pivotX=0.571475
  declareProperty( "P0_ps_scale", m_P0ps = -0.143512);
  declareProperty( "P1_ps_scale", m_P1ps =  36.8308);
  declareProperty( "P2_ps_scale", m_P2ps = -8.96514);
  declareProperty( "P3_ps_scale", m_P3ps =  0.571475);


  declareProperty( "P0_combNN_scale", m_P0combNN = 0);
  declareProperty( "P1_combNN_scale", m_P1combNN =  7.92316e+01);
  declareProperty( "P2_combNN_scale", m_P2combNN = -1.97776e+01);
  declareProperty( "P3_combNN_scale", m_P3combNN =  0.6);


 
}

class MyReader {
public:
  typedef std::list<std::pair<string, Float_t> > ParListType;
  MyReader(const TString& method, const TString& weightfile, const ParListType& par):
    m_method(method),
    m_reader(new TMVA::Reader("!Color:Silent")),
    m_vars(new Float_t[par.size()])
  {
    Float_t *var = m_vars;
    for(ParListType::const_iterator p = par.begin(); p != par.end(); p++) {
       m_reader->AddVariable(p->first, var);
       ++var;
    }
    m_reader->BookMVA(method, weightfile);
  }
  ~MyReader() {
    delete[] m_vars;
  }

  
  inline void set(const ParListType& par) {
    std::transform(par.begin(), par.end(), m_vars, i_getSecond);
  }

  inline Double_t eval() {
    return m_reader->EvaluateMVA(m_method);
  }

  inline Double_t eval(const ParListType& par) {
    set(par);
    return eval();
  }

private:

  TString m_method;
  std::auto_ptr<TMVA::Reader> m_reader;
  Float_t *m_vars;

  static inline Float_t i_getSecond(const std::pair<string, Float_t> &p) {
    return p.second;
  }

};

//=============================================================================
double NNetTool_MLP::MLPmTMVA(std::list<std::pair<string, Float_t> >& par) { 

  bool debug = false; 
  double rnet = 0.5;  
  double pn = 0.5;  
  TString dir = "weights/";
  TString prefix = "muon__muonMLPBNN";
#ifdef NNETcat
  prefix = "muon__muonMLPBNN_Cat";
#endif
  TString methodName = TString("MLPBNN")+TString("method");
  TString weightfile = dir + prefix + TString(".weights.xml");



#ifndef TRAINING  
  static MyReader reader(methodName, weightfile, par);  
  rnet = reader.eval(par);
#endif
  
  if (rnet>=0 && rnet<=1) {
    pn = 1.0 -func(rnet, m_P0mu, m_P1mu, m_P2mu, m_P3mu);
    
    if (debug==true)  {
      cout<<endl<<"Using MLPBNN for Muon "<<endl;
      cout<<endl<<"rnet "<<rnet;
      cout<<endl<<"pn "<<pn;
    }
  } else {
    cout<<endl<<"**********************BAD TRAINING muon"<<rnet<<endl;
    rnet = -1.;
  }
  
  
  m_rnet_mu = rnet;

  return pn;
}; 


//=============================================================================
double NNetTool_MLP::MLPeTMVA(std::list<std::pair<string, Float_t> >& par) {

  bool debug = false; 
  double rnet = 0.5;
  double pn = 0.5;
  
  TString dir = "weights/";
  TString prefix = "ele__eleMLPBNN";
#ifdef NNETcat
  prefix = "ele__eleMLPBNN_Cat";
#endif
  
  TString methodName = TString("MLPBNN")+TString("method");
  TString weightfile = dir + prefix + TString(".weights.xml");

#ifndef TRAINING  
  static MyReader reader(methodName, weightfile, par);
  rnet = reader.eval(par);
#endif
  
  if (rnet>=0 && rnet<=1) {
    
    pn = 1.0 -func(rnet, m_P0e, m_P1e, m_P2e, m_P3e);
    
    if (debug==true)  {
      cout<<endl<<"Using MLPBNN for Electron "<<endl;
      cout<<endl<<"rnet "<<rnet;
      cout<<endl<<"pn "<<pn;
    }
  } else {
     cout<<endl<<"**********************BAD TRAINING ele"<<rnet<<endl;
     rnet = -1.;
  }
  
  m_rnet_ele = rnet;

  return pn;
};

//=============================================================================
double NNetTool_MLP::MLPkaonTMVA(std::list<std::pair<string, Float_t> >& par) {

  bool debug = false; 
  double rnet = 0.5;  
  double pn = 0.5;  
  TString dir = "weights/";
  TString prefix = "kaon__kaonMLPBNN";
#ifdef NNETcat
  prefix = "kaon__kaonMLPBNN_Cat";
#endif


  TString methodName = TString("MLPBNN")+TString("method");
  TString weightfile = dir + prefix + TString(".weights.xml");

#ifndef TRAINING  
  static MyReader reader(methodName, weightfile, par);
  rnet = reader.eval(par);
#endif
  

  if (rnet>=0 && rnet <=1){
    pn = 1.0 -func(rnet, m_P0k, m_P1k, m_P2k, m_P3k);
    
    if (debug==true)  {
      cout<<endl<<"Using MLPBNN for kaon "<<endl;
      cout<<endl<<"rnet "<<rnet;
      cout<<endl<<"pn "<<pn;
    }
  }
  else{    
    cout<<endl<<"**********************BAD TRAINING kaon"<<rnet<<endl;
    rnet = -1.;
  }
  

  m_rnet_k = rnet;

  return pn;

};

//=============================================================================
double NNetTool_MLP::MLPpionSTMVA(std::list<std::pair<string, Float_t> >& par) {

  bool debug = false;
  
  double rnet = 0.5; 

  double pn = 1.0 -func(rnet, m_P0ps, m_P1ps, m_P2ps, m_P3ps);
  
  if (debug==true){
    cout<<endl<<"Using MLPBNN for pionS "<<endl;
    cout<<endl<<"rnet "<<rnet;
    cout<<endl<<"pn "<<pn;
  } 
  //print(values, rnet, pn);
  
  m_rnet_pS = rnet;
  
  return pn;
};

//=============================================================================
double NNetTool_MLP::MLPvtxTMVA(std::list<std::pair<string, Float_t> >& par) {

  bool debug = false; 
  double rnet = 0.5;  
  double pn = 0.5;  
  TString dir = "weights/";
  TString prefix = "vtx__vtxMLPBNN";

  TString methodName = TString("MLPBNN")+TString("method");
  TString weightfile = dir + prefix + TString(".weights.xml");


#ifndef TRAINING  
  static MyReader reader(methodName, weightfile, par);
  rnet = reader.eval(par);
#endif
  
  if (rnet>=0 && rnet<=1) {
    
    pn = 1.0 -func(rnet, m_P0vtx, m_P1vtx, m_P2vtx, m_P3vtx);
    
    if (debug==true)  {
      cout<<endl<<"Using MLPBNN for vtx "<<endl;
      cout<<endl<<"rnet "<<rnet;
      cout<<endl<<"pn "<<pn;
    }
  } else {
    cout<<endl<<"**********************BAD TRAINING vtx"<<rnet<<endl;
    rnet = -1;
  }

  m_rnet_vtx = rnet;

  return pn;

};
//=============================================================================
double NNetTool_MLP::MLPkaonSTMVA(std::list<std::pair<string, Float_t > >& par) {

  bool debug = false; 
  double rnet = 0.5;  
  double pn = 0.5;  
  TString dir = "weights/";
  TString prefix = "ks__ksMLPBNN";

  TString methodName = TString("MLPBNN")+TString("method");
  TString weightfile = dir + prefix + TString(".weights.xml");


#ifndef TRAINING  
  static MyReader reader(methodName, weightfile, par);
  rnet = reader.eval(par);
#endif
  
  if (rnet>=0 && rnet<=1) {
    
    pn = 1.0 -func(rnet, m_P0ks, m_P1ks, m_P2ks, m_P3ks);
    
    if (debug==true)  {
      cout<<endl<<"Using MLPBNN for kaonS "<<endl;
      cout<<endl<<"rnet "<<rnet;
      cout<<endl<<"pn "<<pn;
    }
  }
  else{    
    cout<<endl<<"**********************BAD TRAINING KaonSS"<<rnet<<endl;
    rnet = -1.;
  }
  
  
  m_rnet_kS = rnet;

  return pn;
};
//=============================================================================
double NNetTool_MLP::MLPfragOppTMVA(std::list<std::pair<string, Float_t> >& par) {

  bool debug = false; 
  double rnet = 0.5;  
  double pn = 0.5;  
  TString dir = "weights/";
  TString prefix = "fo__foMLPBNN";

  TString methodName = TString("MLPBNN")+TString("method");
  TString weightfile = dir + prefix + TString(".weights.xml");

#ifndef TRAINING
  static MyReader reader(methodName, weightfile, par);
  rnet = reader.eval(par);
#endif
  
  if (rnet>=0 && rnet<=1){

    pn = 1.0;// -func(rnet, m_P0fo, m_P1fo, m_P2fo, m_P3fo);  // scommentare Stefania

    if (debug==true)  {
      cout<<endl<<"Using MLPBNN for vtx "<<endl;
      cout<<endl<<"rnet "<<rnet;
      cout<<endl<<"pn "<<pn;
    }
  } else {    
    cout<<endl<<"**********************BAD TRAINING fragOpp"<<rnet<<endl;
    rnet = -1.;
    
  }
  
  
  m_rnet_fragOpp = rnet;

  return pn;
};

//=============================================================================
double NNetTool_MLP::MLPCombNNTMVA(std::list<std::pair<string, Float_t > >& par) {

  bool debug = false;
  double rnet = 0.5;
  double pn = 0.5;
  TString dir = "weights/";
  TString prefix = "combNN__combNNMLPBNN";

  TString methodName = TString("MLPBNN")+TString("method");
  TString weightfile = dir + prefix + TString(".weights.xml");


#ifndef TRAININGCOMB
  static MyReader reader(methodName, weightfile, par);
  rnet = reader.eval(par);
#endif

  if (rnet>=0 && rnet<=1) {

     //pn = 1.0 -pol(rnet, m_P0combNN, m_P1combNN, m_P2combNN, m_P3combNN);
    //pn = m_P0combNN + m_P1combNN* (rnet - m_P2combNN);
    pn =rnet;

    if (debug==true)  {
      cout<<endl<<"Using MLPBNN for combNN "<<endl;
      cout<<endl<<"rnet "<<rnet;
      cout<<endl<<"pn "<<pn;
    }
  }
  else{
    cout<<endl<<"**********************BAD TRAINING combNN"<<rnet<<endl;
    rnet = -1.;
  }


  m_rnet_combNN = rnet;

  return pn;
};
