#include "configuration.h"
#define NtpTag_cxx

#include <TLorentzVector.h>
#include <TVector3.h>
#include <iostream>
#include <Riostream.h>

#include "globals.h"
#include "NtpTag.h"

extern double  m_IPPU_cut, m_thetaMin, m_distphi_cut, m_Pmaxtot_cut, m_Ptmaxtot_cut;
extern bool IsControlChannel;

using namespace std;

//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
bool NtpTag::execute(Long64_t ientry, Event& evt) {

  if (fChain == 0) return false;
  Long64_t nentries = fChain->GetEntriesFast();
  if(ientry>=nentries) return false;
  int i = LoadTree(ientry);
  if (i < 0) return false;
  fChain->GetEntry(ientry); 

  //build Event object//////////////////////////////////
  evt.setRunNr(long(Run));
  evt.setEventNr(long(eventNr));
  evt.setMultiplicity(N);
  evt.setStdOmega(Omega);
  evt.setStdTag(Tag);
  evt.setStdTagCat(TagCat);
  evt.seteventType(evType);

  int tr1,tr2,tr3;
  decode(trig, tr3,tr2,tr1);
  evt.setL0  ( tr1 ); 
  evt.setHlt1( tr2 ); 
  evt.setHlt2( tr3 ); 

  evt.setL0TisTos(L0TisTos); 
  evt.setHltTisTos(HltTisTos);
  evt.setBmass(Bmass)  ;
  evt.setBtau(tau)  ;
  evt.setBtauErr(tauErr);
  evt.setTauChi2(ctChi2);
  
  evt.setpileup(krec);

  evt.sethasOscillated(BSosc);
  evt.setbackgroundCat(bkgCat);
  evt.setProcessNr(kType);

  evt.setN_sig_sw(N_sig_sw)  ;

  double BOMas= 5.279; if(abs(BOID)==531) BOMas = 5.367;
  TLorentzVector OppoB(0,0,0,0);
  if(enableMC) OppoB = build4V( BOP, BOP*sin(BOthe), BOphi, BOMas );

  Particle* partOB = new Particle(OppoB, BOID);
  if(enableMC) partOB->setEndVertexPosition(TVector3(BOx,BOy,BOz));
  if(enableMC) partOB->setHasOscillated(BOosc);
  evt.setOppositeB(partOB);

  //fill signal
  if(enableMC) evt.setTrueTag(TrueTag);
  evt.signalParts().clear();

  for (int ii=0; ii<M; ii++){

    int bid = abs(int(sID[ii]));
    TLorentzVector p = build4V( sP[ii], sPt[ii], sPhi[ii], sMass[ii] );

    double BMas= 5.279; if(abs(sMCID[ii])==531) BMas = 5.367;
    TLorentzVector mcp(0,0,0,0);
    if(enableMC) mcp = build4V( sMCP[ii], sMCPt[ii], sMCPhi[ii], BMas );

    Particle* signal = new Particle();
    signal->setMomentum( p );
    signal->setID( int(sID[ii]) );

    if(abs(sID[ii])<100) signal->setCharge( -1*int(sID[ii]) ); // leptons
    else signal->setCharge( int(sID[ii]) ); // hadrons  WARNING!!! not correct for neutrals!! 
    signal->setfromB(-1);
  
    if(enableMC) {
      signal->setMCMomentum( mcp );
      signal->setMCID( int(sMCID[ii]));
      //cout<<bid <<" " <<sP[ii]<<" "<<sMCP[ii]<<" "<<int(sMCID[ii])<<endl;

    }
    
    evt.addToSignalParts(signal);

    //Set Mu and Dstar
    if (bid==13) evt.setMu(signal);
    if (bid==413) evt.setDstar(signal);
    if (bid==421) evt.setD0(signal);
    if (bid==411) evt.setDminus(signal);
    
    if(bid==531 || bid==511 || bid==521) {
      //Redefine TrueTag as the flavour of the reconstructed final state
      //as it is provided by signal selection in TES:
      if( IsControlChannel ) TrueTag = sID[ii]>0? 1 : -1 ;
      evt.setSignalB(signal);
      evt.setBSVertex(TVector3(sVx[ii],sVy[ii],sVz[ii]));
      evt.setTrueTag(TrueTag);
    }
  }

  evt.setRecVertex( TVector3(RVx,RVy,RVz) );
  if(enableMC) evt.setMCVertex ( TVector3(kx, ky, kz) );

  /////////////////////////////////////////////////////////////////
  //build Particle object
  Particles parts = evt.particles(); parts.clear();
 
  int nIPPU_KO=0;
  
  //- loop over N ---------------------------->
  for (int i=0; i<N; ++i) {

    // CUTS APPLIED TO ALL CANDIDATES:
    //if(ipPU[i] == 10000 ) {ipPU[i] = 20.; nIPPU_KO ++; }
    
    
    if(ipPU[i]    < m_IPPU_cut)    continue;//all taggers far from PU vertices
    if(distPhi[i] < m_distphi_cut) continue;
    if(acos(Pt[i]/P[i])<m_thetaMin)continue;
    //    if(P[i] < 2000) continue;//
    //talls absurds
    //    if(P[i] > m_Pmaxtot_cut)continue;//no preselection
    //    if(Pt[i] > m_Ptmaxtot_cut)continue;
    //    if(P[i] > 200000)continue;//no preselection
    //    if(Pt[i] > 10000)continue;
    //    if(P[i] > 20000000)continue;//no preselection
    //    if(Pt[i] > 1000000)continue;
    //altres talls
    //if(ip[i]      > m_IP_cut)      continue;
    //if(cloneDist[i]!=-1) continue; //no clones
    
    /*
    //cuts for PV>1
    if (krec>1) {
      if ((pow(ip[i]-ipPU[i],2)/pow(ipPU[i],2))<0.865) continue;
      if (ip[i]-nippu[i]>0.5) continue;
      if (ip[i]-ipmean[i]>0.2) continue;
    }
    */
       
    ////////////////////////////////
    Particle* apart = new Particle();
    ////////////////////////////////

    int ismu, muNSH, inHcal, iflag_e, iflag_m, iinfo_m, iflag_k;
    decode( int(PIDfl[i]), ismu, muNSH, inHcal, iflag_e, iflag_m, iflag_k );

    TLorentzVector p   = build4V( P[i], Pt[i], phi[i], ID[i] );
    
    TLorentzVector mcp(0,0,0,0);
    if(enableMC) mcp = build4V( MCP[i], MCPt[i], MCphi[i], MCID[i] );
 
    apart->setMomentum( p );
    
    apart->setIDtag( 211*int(ID[i])/abs(int(ID[i])) );
    apart->setID( int(ID[i]) );
    apart->setPIDe( PIDe[i] );
    apart->setPIDk( PIDk[i] );
    apart->setPIDm( PIDm[i] );

    apart->setHasMuonFlag( iflag_m );
    apart->setisMuon( ismu );
    apart->setInAccHcal( inHcal );
    apart->setEOverP( EOverP[i] );
    apart->setVeloCharge( veloch[i] );
  
    apart->setDOCA( DOCA[i] );
    apart->setIPSV( IPSV[i] );
 
    apart->setDistPhi( distPhi[i] );

    apart->setLikelihood( tsal[i] );
   
    apart->setMuonNShared(muNSH);
    apart->setPIDp( PIDp[i] );
    apart->setIPs( ip[i]/iperr[i] );
    apart->setIP( ip[i] );
    apart->setIPsign( ipsign[i] );
    apart->setIPssign( ipsign[i]/iperr[i] );
    apart->setIPerr( iperr[i] );
    apart->setIPPU( ipPU[i] );
#ifdef ANN
    apart->setGhostProbability( gprob[i] ); //ghostProbability
    apart->setGhostProbabilityNN( gprobNN[i] ); //ghostProbabilityNN
    apart->setPIDNNe( PIDNNe[i] );
    apart->setPIDNNm( PIDNNm[i] );
    apart->setPIDNNk( PIDNNk[i] );
    apart->setPIDNNp( PIDNNp[i] );
    apart->setPIDNNpi( PIDNNpi[i] );
#else
    apart->setGhostProbability( 1. ); //ghostProbability
    apart->setGhostProbabilityNN( 1. ); //ghostProbability
    apart->setPIDNNe( 0. );
    apart->setPIDNNm( 0. );
    apart->setPIDNNk( 0. );
    apart->setPIDNNp( 0. );
    apart->setPIDNNpi( 0.);
#endif
    //extrainfo for PV
    apart->setipmean( ipmean[i] );
    apart->setnippu( nippu[i] );
    apart->setnippuerr( nippuerr[i] );
    apart->settracks( tracks[i] );
    apart->settrackzfirst( trackzfirst[i] );
    apart->settrackp( trackp[i] );
    apart->setippubs( ippubs[i] );
    apart->setippuchi2bs( ippuchi2bs[i] );
    apart->setzpos( zpos[i] );
    apart->setzerrpos( zerrpos[i] );

    apart->setcloneDist( cloneDist[i] );
    apart->setLCS( lcs[i] );
    apart->setCharge( int(ch[i]) );
    apart->setType( int(trtyp[i]) );
    apart->setComesFromSVtx( vFlag[i] );

    //MC INFO
    if(enableMC) {
      apart->setMotherID( int(mothID[i]) );
      apart->setAncestorID( int(ancID[i]) );
      apart->setfromB( int(bFlag[i]) );
      apart->setxFlag( int(xFlag[i]) );
      apart->setMCMomentum(mcp);
      apart->setMCID( int(MCID[i]) );
    }
    //if( apart->ghostProbability()<0.5)    parts.push_back(apart);
    if( apart->ghostProbability()>0.5) cout<<" Particle with ghostPtob="<<apart->ghostProbability()<<endl;
    
    parts.push_back(apart);
  }
  
#ifdef CLEANCLONES
  
  //Double_t pi_mass = 0.139570;
  Double_t pi_mass =  0.105658;
  int itag(0);  
  for (Particles::iterator ipart=parts.begin(); ipart!=parts.end(); ++ipart) {          
    TLorentzVector p1 = (*ipart)->momentum();
    p1.SetE(sqrt(pow(p1.P(),2)+pow(pi_mass,2)));
    for (Particles::iterator jpart=ipart+1; jpart!=parts.end(); ++jpart) {
      TLorentzVector p2 = (*jpart)->momentum();
      p2.SetE(sqrt(pow(p2.P(),2)+pow(pi_mass,2)));
      TLorentzVector p12 = p1+p2;
      Double_t angle = p1.Angle(p2.Vect());
      
      double tx_1 = p1.Px()/p1.Pz();
      double tx_2 = p2.Px()/p2.Pz();
      double ty_1 = p1.Py()/p1.Pz();
      double ty_2 = p2.Py()/p2.Pz();
      double qOp_1 = (*ipart)->charge()/p1.P();
      double qOp_2 = (*jpart)->charge()/p2.P();
      double m12 = ((p1+p2).M2() - 2.*p1.M2() - 2.*p2.M2())/(p1+p2).M2();            

      
      if( (fabs(tx_1 - tx_2)<0.0004 && fabs(ty_1 - ty_2)<0.0002)  || 
          (fabs(tx_1 - tx_2)<0.005 && fabs(ty_1 - ty_2)<0.005 && fabs(qOp_1-qOp_2)<1e-3 ) ||
          ( m12<0.02 ) ||
          ( m12<0.05 && angle < 0.001 ) /*||
          ( m12<0.15 && angle < 0.012 && fabs(qOp_1- qOp_2)<5e-3 && p1.DeltaR(p2)<0.4)*/ ){
        if((*jpart)->ghostProbability()> (*ipart)->ghostProbability()) (*jpart)->setLCS(999);
        else  (*jpart)->setLCS(999);
        itag++;
        
      }
      /*
      int ich = 0;              
      if((*ipart)->charge()*(*jpart)->charge()<0) ich=1;
      
      if(angle<0.008 && (p1-p2).P()/(p1+p2).P()<0.1 && (p1-p2).Pt()/(p1+p2).Pt()<0.15 && ich==0 ) {
      //if(angle<0.0005 && ich==0 ) {
        if((*jpart)->ghostProbability()> (*ipart)->ghostProbability()){ (*jpart)->setLCS(999);            itag++;
        }
        
        //if((*jpart)->LCS()> (*ipart)->LCS()) (*jpart)->setLCS(999);            
        else (*ipart)->setLCS(999);            
      }
      

      if(angle<0.008 && (p1-p2).P()/(p1+p2).P()<0.1 && (p1-p2).Pt()/(p1+p2).Pt()<0.15 && ich==0 && fabs((p1+p2).M()-2.*pi_mass)<1) 
      {
        cout <<" MCID="<<(*ipart)->MCID()<<"/"<<(*jpart)->MCID()<<" mother="<<(*ipart)->motherID()<<"/"<<(*jpart)->motherID()<<
          " MCp="<<(*ipart)->momentumMC().P()<<"/"<<(*jpart)->momentumMC().P()<<" fromB"<<(*ipart)->fromB()<<"/"<<(*jpart)->fromB()<<
          " fromBx="<<(*ipart)->xFlag()<<"/"<<(*jpart)->xFlag()<<endl;
      }
      */      
      
    }    
  }
  
  int isig(0);
  
  for (int i=0; i<evt.signalParts().size(); ++i){
    Particle *ipart = evt.signalParts().at(i);
    if(!(ipart))continue;
    if(abs((ipart)->ID())<350 && (ipart)->momentum().M()!=0) {
      TLorentzVector p1 = (ipart)->momentum();
      p1.SetE(sqrt(pow(p1.P(),2)+pow(pi_mass,2)));    
      for (Particles::iterator jpart=parts.begin(); jpart!=parts.end(); ++jpart) {      
        TLorentzVector p2 = (*jpart)->momentum();
        p2.SetE(sqrt(pow(p2.P(),2)+pow(pi_mass,2)));
        TLorentzVector p12 = p1+p2;
        Double_t angle = p1.Angle(p2.Vect());

        double tx_1 = p1.Px()/p1.Pz();
        double tx_2 = p2.Px()/p2.Pz();
        double ty_1 = p1.Py()/p1.Pz();
        double ty_2 = p2.Py()/p2.Pz();
        double qOp_1 = (ipart)->charge()/p1.P();
        double qOp_2 = (*jpart)->charge()/p2.P();
        double m12 = ((p1+p2).M2() - 2.*p1.M2() - 2.*p2.M2())/(p1+p2).M2();            
        if(  (*jpart)->LCS()<998 ) {
          if(fabs(tx_1 - tx_2)<0.0004 && fabs(ty_1 - ty_2)<0.0002) (*jpart)->setLCS(999);
          if(fabs(tx_1 - tx_2)<0.005 && fabs(ty_1 - ty_2)<0.005 && fabs(qOp_1-qOp_2)<1e-3 ) (*jpart)->setLCS(999);        
          if( m12<0.02 )  (*jpart)->setLCS(999);
          if( m12<0.05 && angle < 0.001 )  (*jpart)->setLCS(999);
          //if( m12<0.15 && angle < 0.012 && fabs(qOp_1- qOp_2)<5e-3 && p1.DeltaR(p2)<0.4) (*jpart)->setLCS(999);
          
          if( (*jpart)->LCS()>998) isig++;          
        }
        
        /*        
        int ich = 0;              
        if(ipart->charge()*(*jpart)->charge()<0) ich=1;
        //if(angle<0.0005 && ich==0 )   (*jpart)->setLCS(999);              
        if(angle<0.008 && (p1-p2).P()/(p1+p2).P()<0.1 && (p1-p2).Pt()/(p1+p2).Pt()<0.15 && ich==0 )  {
          (*jpart)->setLCS(999);              
          isig++;          
        }
        */
             
      }              
    }      
  }
  
  //  if(parts.size()>1) cout <<" Clone_sig "<<isig<<" Clone_tag "<<float(itag)<<" part "<<parts.size()<<endl;
  
  

  
  for (Particles::iterator ipart=parts.begin(); ipart!=parts.end();  ) {          
    if((*ipart)->LCS()>998) {
      
      //cout<<" clone mu "<< 
      //  ( (*ipart)->isMuon()!=0 && (*ipart)->hasMuonFlag()!=0 && (*ipart)->muonNShared()==0 && (*ipart)->cloneDist()==-1 
      //     &&(*ipart)->PIDNNm()<0.52 ) <<
      //  " e "<< ( (*ipart)->inAccHcal() && (*ipart)->PIDe() > -10  && (*ipart)->cloneDist()==-1 &&  
      //            (*ipart)->PIDNNpi()<0.4 &&  (*ipart)->PIDNNp()<0.2 )  <<
      //  " KaonOppo "<< 
      //  ( (*ipart)->PIDk()!=0 && (*ipart)->PIDk()!=-1000 && (*ipart)->type()==3 &&
      //    (*ipart)->PIDk() > -100 && (*ipart)->cloneDist()==-1 && (*ipart)->PIDk()-(*ipart)->PIDp()> -300 && 
      //    (*ipart)->PIDNNpi()<0.4 &&  (*ipart)->PIDNNp()<0.5 ) <<endl;

      parts.erase(ipart);            
    } else ++ipart;
  }
  
  
#endif

  evt.setParticles(parts);

  /////////////////////////////////////////////////////////////////
  //build Secondary Vertex object (ONLY THE SEED)

  Vertices  allSeeds;
  Particles parts_in_Seed; parts_in_Seed.clear();

  for ( int iv=0; iv<V; iv++ ) {
    int i_seed1=-1, i_seed2=-1; 
    for ( int i=0; i< (int)parts.size(); i++ ) {
      if(!vFlag[i]) continue;
      if(SecVtx_pt1[iv]==Pt[i]) i_seed1=i; //N-block index 
      if(SecVtx_pt2[iv]==Pt[i]) i_seed2=i;
    }
    if( i_seed1==-1 || i_seed2==-1 )  {
      if(msgLevel(MSG::DEBUG)) debug()<<"could not find 2 seeds "<<endmsg;
      continue;//info missing in N-block
    }

    parts_in_Seed.push_back( parts.at(i_seed1) );
    parts_in_Seed.push_back( parts.at(i_seed2) );
    
    TVector3 SVpoint(SecVtx_x[iv]-RVx, SecVtx_y[iv]-RVy, SecVtx_z[iv]-RVz);

    Vertex* Seed = new Vertex();

    Seed->setPosition( SVpoint );
    Seed->setChi2PerDoF( SecVtx_chi2[iv] );
    Seed->setType( Vertex::Seed );
    Seed->setOutgoingParticles( parts_in_Seed );

    allSeeds.push_back( Seed );

    break; //TAKE JUST ONE SEED
  }

  evt.setSeeds( allSeeds );

  //Fill tag info direct from DaVinci run algorithm
  //build a bit flag
  int dectagDV=0, tagDV=0;
  if(property["checkDV"]==true) {
    for(int i=0; i<T; ++i) {
      if(TaggerDecision[i]==0) continue;
      int adec = int(TaggerDecision[i])+2;
      
      int aType= int(TaggerType[i]);
      //cout<<" TaggerType "<<TaggerType[i]<<" "<<aType<<endl;
      if     (aType==2) { tagDV += 100000; dectagDV += adec*100000; }
      else if(aType==3) { tagDV +=  10000; dectagDV += adec* 10000; }
      else if(aType==4) { tagDV +=   1000; dectagDV += adec*  1000; }
      else if(aType==5) { tagDV +=    100; dectagDV += adec*   100; }//sameS
      else if(aType==6) { tagDV +=     10; dectagDV += adec*    10; }//sameS
      else if(aType==10) { tagDV +=      1; dectagDV += adec*    1; }//Vertex
      // cout<< aType<<" "<<tagDV<<" "<<dectagDV<<endl;
    }
  } 
  evt.setTaggersFromDV(tagDV);
  evt.setTaggersDecisionFromDV(dectagDV);

  return true;
}

