#include "configuration.h"
#include "TaggerElectronTool.h"

TaggerElectronTool::TaggerElectronTool() {

  declareProperty( "Ele_Pt_cut",        m_Pt_cut_ele    = 1000. );
  declareProperty( "Ele_P_cut",         m_P_cut_ele     = 0.0 );
  declareProperty( "Ele_lcs_cut",       m_lcs_cut_ele   = 3.85 );
  declareProperty( "Ele_IPs_cut",       m_IPs_cut_ele   = 1. );
  declareProperty( "Ele_IPssign_cut",   m_IPssign_cut_ele   = -9999. );
  declareProperty( "Ele_IPssignPlus_cut", m_IPssignPlus_cut_ele   = -1. );
  declareProperty( "Ele_IPssignMinus_cut",m_IPssignMinus_cut_ele   = 1. );
  declareProperty( "Ele_ghost_cut",     m_ghost_cut_ele = -999 );
  declareProperty( "Ele_Mult_cut",      m_mult_cut_ele  = 500 );
  declareProperty( "Ele_VeloChargeMin", m_VeloChMin     = 0.0 );
  declareProperty( "Ele_VeloChargeMax", m_VeloChMax     = 1.6 );
  declareProperty( "Ele_EoverP",        m_EoverP        = 0.75 );
  declareProperty( "Ele_EoverPmax",     m_EoverPmax        = 6. );
  declareProperty( "Ele_PIDe_cut",      m_PIDe_cut_ele      = 4. );
  declareProperty( "Ele_ipPU_cut",      m_ipPU_cut_ele      = 5.0 );
  declareProperty( "Ele_distPhi_cut",   m_distPhi_cut_ele= 0.005 );
  declareProperty( "Ele_cand",          m_cand_ele   = 10. ); 
  declareProperty( "Ele_ProbMin",       m_ProbMin_ele   = 0. ); //no cut
  declareProperty( "Ele_PIDNNp_cut",    m_PIDNNp_cut_ele     = 1. );
  declareProperty( "Ele_PIDNNpi_cut",   m_PIDNNpi_cut_ele    = 1. );
  declareProperty( "Ele_PIDNNe_cut",    m_PIDNNe_cut_ele     = -1. );
  declareProperty( "Ele_PIDNNk_cut",    m_PIDNNk_cut_ele     = 1. );
  declareProperty( "Ele_PIDNNepi_cut",  m_PIDNNepi_cut_ele     = -1. );
  declareProperty( "Ele_ghostProb_cut", m_ghostProb_ele= 10. );
  declareProperty( "Ele_P0_Cal",        m_P0_Cal_ele   = 0.306 ); 
  declareProperty( "Ele_P1_Cal",        m_P1_Cal_ele   = 0.974 ); 
  declareProperty( "Ele_Eta_Cal",       m_Eta_Cal_ele  = 0.346 ); 

  NNetTool_MLP nnet;
  tele = new Tagger();
    
  hcut_ele_ippu   = new TH1F("hcut_ele_ippu","hcut_ele_ippu",100, 0, 20);
  hcut_ele_distphi= new TH1F("hcut_ele_distphi","hcut_ele_distphi",100, 0, 0.09);
  hcut_ele_N      = new TH1F("hcut_ele_N",   "hcut_ele_N",100, 0, 100);
  hcut_ele_ips    = new TH1F("hcut_ele_ips", "hcut_ele_ips",100, 0, 20);
  hcut_ele_ipssign= new TH1F("hcut_ele_ipssign", "hcut_ele_ipssign",100, -20, 20);
  hcut_ele_pidk   = new TH1F("hcut_ele_pidk","hcut_ele_pidk",100, -10, 50);
  hcut_ele_pt     = new TH1F("hcut_ele_pt",  "hcut_ele_pt",  100, 0, 5);
  hcut_ele_p      = new TH1F("hcut_ele_p",   "hcut_ele_p",   100, 0, 100);
  hcut_ele_lcs    = new TH1F("hcut_ele_lcs", "hcut_ele_lcs", 100, 0, 5);
  hcut_ele_gho    = new TH1F("hcut_ele_gho", "hcut_ele_gho", 100, -40, 0);
  hcut_ele_EoP    = new TH1F("hcut_ele_EoP", "hcut_ele_EoP", 100,  0, 2);
  hcut_ele_velo   = new TH1F("hcut_ele_velo","hcut_ele_velo",100, 0, 2);
  hcut_ele_cand   = new TH1F("hcut_ele_cand","hcut_ele_cand", 5, 0, 5);


}

////////////////////////////////////////////////
Tagger* TaggerElectronTool::tag(Event& event) {
  tele->reset();

  if(msgLevel(MSG::VERBOSE)) verbose()<<"--Electron Tagger --"<<endreq;

  Particle* iele=NULL;
  double ptmaxe=-999, ncand=0;
  Particles::iterator ipart;
  Particles vtags = event.particles();

  
  for (ipart=vtags.begin(); ipart!=vtags.end(); ++ipart) {


    Particle* axp = (*ipart);

    if(!checkPIDhypo(Particle::electron, axp)) continue; // PID and acceptance cuts.
    if(msgLevel(MSG::VERBOSE)) verbose() << " Ele PIDe="<< axp->PIDe() <<endmsg;

#ifdef NEWCUTS
    if(axp->ghostProbability() >  m_ghostProb_ele ) continue;    
    
    if(axp->PIDNNk() > m_PIDNNk_cut_ele  ) continue;    
    if(axp->PIDNNpi() > m_PIDNNpi_cut_ele  ) continue;   
    if(axp->PIDNNp() >  m_PIDNNp_cut_ele ) continue;     
    if(axp->PIDNNe() <  m_PIDNNe_cut_ele ) continue;    
    if(axp->PIDNNe() - axp->PIDNNpi() < m_PIDNNepi_cut_ele  ) continue;
    
#endif    
    
    double Pt = axp->pt();
    if( Pt < m_Pt_cut_ele*0.001 )  continue;//to have GeV

    double P = axp->p();
    if( P < m_P_cut_ele*0.001 )  continue;//to have GeV
    if(msgLevel(MSG::VERBOSE)) verbose() << " Ele P="<< P <<" Pt="<<Pt<<endmsg;
    if( axp->LCS() > m_lcs_cut_ele ) continue;

    if( axp->type() != Particle::Long ) continue;
    //old FTC long and matched are trtyop=7
    //    && axp->type() != Particle::Matched
    //    && axp->type() != Particle::Upstream ) continue;

    double tsa = axp->likelihood();
    if(tsa < m_ghost_cut_ele) continue;

    if(msgLevel(MSG::VERBOSE)) verbose() << " Ele lcs="<< axp->LCS() <<" tsa="<<tsa<<endmsg;
    //calculate signed IP wrt RecVert
    double IPsig = axp->IPs();
    if(IPsig < m_IPs_cut_ele ) continue;
    double IPssig = axp->IPssign();
    if(IPssig < m_IPssign_cut_ele ) continue;
    if ( IPssig>0 && IPssig<m_IPssignPlus_cut_ele ) continue;
    if ( IPssig<0 && IPssig>m_IPssignMinus_cut_ele ) continue;

    //ip and phi wrt PU
    double ipPU  = axp->IPPU();
    if(ipPU < m_ipPU_cut_ele ) continue;
    double distPhi = axp->distPhi();
    if (distPhi < m_distPhi_cut_ele) continue;

    double eOverP = axp->eOverP();
    if(eOverP > m_EoverPmax ) continue;
    if(eOverP > m_EoverP ) {// || eOverP<-100) {
      if(msgLevel(MSG::VERBOSE)) verbose() << " Elec E/P=" << eOverP <<endreq;
      
      double veloch = axp->VeloCharge();
      if( veloch > m_VeloChMin && veloch < m_VeloChMax ) {
        if(msgLevel(MSG::VERBOSE)) verbose() << " Elec veloch=" << veloch << endreq;
        
        ncand++;

        hcut_ele_pidk   ->Fill(axp->PIDe());
        hcut_ele_pt     ->Fill(Pt);
        hcut_ele_p      ->Fill(P);
        hcut_ele_lcs    ->Fill(axp->LCS());
        hcut_ele_gho    ->Fill(tsa);
        hcut_ele_EoP    ->Fill(eOverP);
        hcut_ele_velo   ->Fill(veloch);
        hcut_ele_ippu   ->Fill(axp->IPPU());
        hcut_ele_distphi->Fill(distPhi);
        hcut_ele_N      ->Fill(event.multiplicity());
        hcut_ele_ips    ->Fill(IPsig);
        hcut_ele_ipssign->Fill(IPssig);

        (*ipart)->setIDtag(-11*(*ipart)->charge());
        if( Pt > ptmaxe ) { 
          iele = (*ipart);
          ptmaxe = Pt;
        }
      }
    }
  }
  hcut_ele_cand   ->Fill(ncand);
  if(!iele) return tele;
  if(ncand>m_cand_ele) return tele;
  if(event.multiplicity()>m_mult_cut_ele ) return tele; //tagging part mult
  
  //calculate omega
  std::vector<double> NNinputs(10);
  list<std::pair<string, Float_t> > NNinputList;

  NNinputList.push_back(pair<string, Float_t>("mult",   event.multiplicity()));
  NNinputList.push_back(pair<string, Float_t>("partP",  log(iele->p())));
  NNinputList.push_back(pair<string, Float_t>("partPt", log(iele->pt())));
  //======  NNinputList.push_back(pair<string, Float_t>("nnkrec", event.pileup()));
  NNinputList.push_back(pair<string, Float_t>("ptB",log(event.signalB()->pt())));
  //double ssign=1;
  //if(iele->IPssign()<0) ssign=-1;  
  //double myIPs = ssign*sqrt(abs(iele->IPssign()));  
  //NNinputList.push_back(pair<string, double>("IPssign",   myIPs));
  NNinputList.push_back(pair<string, Float_t>("IPs",    log(iele->IPs())));
  NNinputList.push_back(pair<string, Float_t>("partlcs",log(iele->LCS())));
  //====  NNinputList.push_back(pair<string, Float_t>("PIDe",   iele->PIDe()));
  //====  NNinputList.push_back(pair<string, Float_t>("PIDNNe", iele->PIDNNe()));
  //====  NNinputList.push_back(pair<string, Float_t>("PIDNNpi",iele->PIDNNpi()));
  //====  NNinputList.push_back(pair<string, Float_t>("PIDNNp", iele->PIDNNp()));
  //====  NNinputList.push_back(pair<string, Float_t>("PIDNNk", iele->PIDNNk()));
  NNinputList.push_back(pair<string, Float_t>("eOverP", iele->eOverP()));
  //====  NNinputList.push_back(pair<string, Float_t>("veloch", iele->VeloCharge()));
  NNinputList.push_back(pair<string, Float_t>("ghostProb", log(iele->ghostProbability()))); 
  NNinputList.push_back(pair<string, Float_t>("IPPU",   log(iele->IPPU())));

  double pn = nnet.MLPeTMVA( NNinputList ); 
  if(msgLevel(MSG::VERBOSE)) verbose() << " Elec pn=" << pn << endreq;

  //Calibration (w=1-pn) w' = p0 + p1(w-eta)
  pn = 1 - m_P0_Cal_ele - m_P1_Cal_ele * ( (1-pn)-m_Eta_Cal_ele);
  //debug() << " Elec pn=" << pn <<" w="<<1-pn<< endreq;
  if( pn < 0 || pn > 1 )  cout<<endl<<"Something wrong with ELECTRON training pn "<<pn;/*return tele*/;
  if( nnet.m_rnet_ele < -0.5 ) {
    cout<<endl<<"===> Something wrong with ELECTRON nnet "<<nnet.m_rnet_ele ;
    return tele;
  }
  if( pn < m_ProbMin_ele ) return tele;

  iele->setIDtag(-11*iele->charge());
  iele->setID(-11*iele->charge());
  float sign=1;
#ifndef TRAINING
  if(pn<0.5){
    pn = 1-pn;
    sign=-1;
  }
#endif
  if(property["checkDV"]==true) {
    std::cout<<" TaggerElectron: "<<sign*iele->charge()<<" omega="<<1-pn;
    
    typedef std::list<std::pair<std::string, Float_t> > ParListType;
    for(ParListType::const_iterator p = NNinputList.begin(); p != NNinputList.end(); p++) {
      std::cout << p->second<<" ";
    }
    std::cout<<std::endl;
  }
  
  tele->setOmega( 1-pn );
  tele->setRawNNetOutput( nnet.m_rnet_ele );
  tele->setDecision(sign*(iele->charge())>0 ? -1: 1);
  tele->addToTaggerParts(iele);
  tele->setType( Tagger::OS_Electron );

  return tele;
}
