#include "configuration.h"
#include "TaggerKaonOppositeTool.h"

TaggerKaonOppositeTool::TaggerKaonOppositeTool() {

  declareProperty( "Kaon_Pt_cut",           m_Pt_cut_kaon   = 700  );
  declareProperty( "Kaon_P_cut",            m_P_cut_kaon    = 5000 );
  declareProperty( "Kaon_IPs_cut",          m_IPs_cut_kaon  = 4.3 );
  declareProperty( "Kaon_IPssign_cut",      m_IPssign_cut_kaon  = -9999.0 );
  declareProperty( "Kaon_IPssignPlus_cut",  m_IPssignPlus_cut_kaon  = -1. );
  declareProperty( "Kaon_IPssignMinus_cut", m_IPssignMinus_cut_kaon  = 1. );
  declareProperty( "Kaon_IP_cut",           m_IP_cut_kaon   = 1.45 );
  declareProperty( "Kaon_lcs_cut",          m_lcs_cut_kaon      = 2.45 );
  declareProperty( "Kaon_PIDk",             m_PIDk_cut_kaon     =  0.75);
  declareProperty( "Kaon_PIDkp",            m_PIDkp_cut_kaon     = -3. );
  declareProperty( "Kaon_ghost_cut",        m_ghost_cut_kaon     = -999.0 );
  declareProperty( "Kaon_ipPU_cut",         m_ipPU_cut_kaon      = 7.5 );
  declareProperty( "Kaon_distPhi_cut",      m_distPhi_cut_kaon= 0.005 );
  declareProperty( "Kaon_cand",             m_cand_kaon  = 10. );
  declareProperty( "Kaon_ProbMin",          m_ProbMin_kaon  = 0.54 );

  declareProperty( "Kaon_PIDNNmu_cut",      m_PIDNNm_cut_kaon     = 1. );
  declareProperty( "Kaon_PIDNNk_cut",       m_PIDNNk_cut_kaon      = 0. );
  declareProperty( "Kaon_PIDNNpi_cut",      m_PIDNNpi_cut_kaon     = 1. );
  declareProperty( "Kaon_PIDNNp_cut",       m_PIDNNp_cut_kaon     = 1. );
  declareProperty( "Kaon_PIDNNe_cut",       m_PIDNNe_cut_kaon     = 1. );

  declareProperty( "Kaon_PIDNNkp_cut",       m_PIDNNkp_cut_kaon    = -1. );
  declareProperty( "Kaon_PIDNNkpi_cut",       m_PIDNNkpi_cut_kaon  = -1. );

  declareProperty( "Kaon_ghostProb_cut",    m_ghostProb_kaon= -1. );

  declareProperty( "Kaon_P0_Cal",           m_P0_Cal_kaon   = 0.393 ); 
  declareProperty( "Kaon_P1_Cal",           m_P1_Cal_kaon   = 0.706 ); 
  declareProperty( "Kaon_Eta_Cal",          m_Eta_Cal_kaon  = 0.354 ); 


  

  NNetTool_MLP nnet;
  tkaon = new Tagger();
    
  hcut_ko_ippu   = new TH1F("hcut_ko_ippu","hcut_ko_ippu",100, 0, 20);
  hcut_ko_distphi= new TH1F("hcut_ko_distphi","hcut_ko_distphi",100, 0, 0.09);
  hcut_ko_N      = new TH1F("hcut_ko_N",   "hcut_ko_N"   ,100, 0, 100);
  hcut_ko_pidk   = new TH1F("hcut_ko_pidk","hcut_ko_pidk",100, -10, 90);
  hcut_ko_pidp   = new TH1F("hcut_ko_pidp","hcut_ko_pidp",100, -10, 90);
  hcut_ko_pt     = new TH1F("hcut_ko_pt", "hcut_ko_pt",  100, 0, 8);
  hcut_ko_p      = new TH1F("hcut_ko_p",  "hcut_ko_p",   100, 0, 100);
  hcut_ko_lcs    = new TH1F("hcut_ko_lcs","hcut_ko_lcs", 100, 0, 5);
  hcut_ko_gho    = new TH1F("hcut_ko_gho","hcut_ko_gho", 100, -40, 0);
  hcut_ko_IPs    = new TH1F("hcut_ko_IPs","hcut_ko_IPs", 100, 0, 20);
  hcut_ko_IPssign= new TH1F("hcut_ko_IPssign","hcut_ko_IPssign", 100, -20, 20);
  hcut_ko_IP     = new TH1F("hcut_ko_IP ","hcut_ko_IP",  100, 0, 5);
  hcut_ko_cand   = new TH1F("hcut_ko_cand ","hcut_ko_cand",  5, 0, 5);
  
}

///////////////////////////////////////////////////
Tagger* TaggerKaonOppositeTool::tag(Event& event) {
  tkaon->reset();
  //std::cout<<" KaonProbMin = "<<m_ProbMin_kaon<<std::endl;
  if(msgLevel(MSG::VERBOSE)) verbose() << "--Kaon Oppsite Tagger--"<<endmsg;

  Particle* ikaon=NULL;
  double ptmaxk=-999, ncand=0;
  Particles::iterator ipart;
  Particles vtags = event.particles();
  for (ipart=vtags.begin(); ipart!=vtags.end(); ++ipart) {

    Particle* axp = (*ipart);
    double pidk= axp->PIDk();
    double pidproton = axp->PIDp();

    /*
    if ( pidk==0 or pidk==-1000.0 or axp->cloneDist()!=-1) continue;
    if ( pidk < m_PIDk_cut_kaon) continue;
    if ( (pidk-pidproton) < m_PIDkp_cut_kaon) continue;
    */
    if(!checkPIDhypo(Particle::kaon_opposite, axp)) continue;
    if(msgLevel(MSG::VERBOSE)) verbose() << " Kaon PID pass"<<endreq;

    double Pt = axp->pt();
    if( Pt < m_Pt_cut_kaon*0.001 )  continue;//GeV
 
    double P = axp->p();
    if( P < m_P_cut_kaon*0.001 )  continue;//GeV
 
    if(msgLevel(MSG::VERBOSE)) verbose() << " Kaon P="<< P <<" Pt="<<Pt<<endmsg;
    
#ifdef NEWCUTS
    if(axp->ghostProbability() >  m_ghostProb_kaon ) continue;    
    if(axp->PIDNNk()  <  m_PIDNNk_cut_kaon ) continue;    
    if(axp->PIDNNpi() >  m_PIDNNpi_cut_kaon ) continue;    
    if(axp->PIDNNp()  >  m_PIDNNp_cut_kaon ) continue;    
    if(axp->PIDNNe()  >  m_PIDNNe_cut_kaon ) continue;    
    if(axp->PIDNNm()  >  m_PIDNNm_cut_kaon ) continue;    
    if(axp->PIDNNk()-axp->PIDNNp()  < m_PIDNNkp_cut_kaon ) continue;    
    if(axp->PIDNNk()-axp->PIDNNpi()  < m_PIDNNkpi_cut_kaon ) continue;    

    
#endif
    
    double lcs = axp->LCS();
    if(lcs > m_lcs_cut_kaon) continue;
 
    double tsa = axp->likelihood();
    if( tsa < m_ghost_cut_kaon ) continue;
    if(msgLevel(MSG::VERBOSE)) verbose() << " Kaon lcs="<< lcs <<" tsa="<<tsa<<endmsg;
    
    //calculate signed IP wrt RecVert
    double IPsig = axp->IPs();
    double IPssig = axp->IPssign();
    double IP    = axp->IP();
    double ipPU  = axp->IPPU();
    if(msgLevel(MSG::VERBOSE)) verbose() << " Kaon IPs="<< IPsig <<" IP="<<fabs(IP)<<" ipPU="<<ipPU<<endmsg;
    if(IPsig < m_IPs_cut_kaon ) continue;
    if(IPssig < m_IPssign_cut_kaon ) continue;
    if(fabs(IP) > m_IP_cut_kaon ) continue;
    if(ipPU < m_ipPU_cut_kaon ) continue;
    if ( IPssig>0 && IPssig<m_IPssignPlus_cut_kaon ) continue;
    if ( IPssig<0 && IPssig>m_IPssignMinus_cut_kaon ) continue;

    double distPhi = axp->distPhi();
    if(msgLevel(MSG::VERBOSE)) verbose() << " Kaon distPhi="<< distPhi<<endmsg;
    if (distPhi < m_distPhi_cut_kaon) continue;

    ncand++;
    
    hcut_ko_ippu   ->Fill(ipPU);
    hcut_ko_N      ->Fill(event.multiplicity());
    hcut_ko_pidk   ->Fill(pidk);
    hcut_ko_pidp   ->Fill(pidproton);
    hcut_ko_pt     ->Fill(Pt);
    hcut_ko_p      ->Fill(P);
    hcut_ko_lcs    ->Fill(lcs);
    hcut_ko_gho    ->Fill(tsa);
    hcut_ko_IPs    ->Fill(IPsig);
    hcut_ko_IPssign->Fill(IPssig);
    hcut_ko_IP     ->Fill(IP);
    hcut_ko_distphi->Fill(distPhi);

    (*ipart)->setIDtag(321*(*ipart)->charge()); // overlap ?
    
    if( Pt > ptmaxk ) { 
      ikaon = (*ipart);
      ptmaxk = Pt;
    }
  }  
  
  hcut_ko_cand   ->Fill(ncand);
  if( ! ikaon ) return tkaon;
  if (ncand>m_cand_kaon) return tkaon;

  //calculate omega
  list<std::pair<string, Float_t> > NNinputList;

  NNinputList.push_back(pair<string, Float_t>("mult",     event.multiplicity()));
  NNinputList.push_back(pair<string, Float_t>("partP",    log(ikaon->p())));
  NNinputList.push_back(pair<string, Float_t>("partPt",   log(ikaon->pt())));
  NNinputList.push_back(pair<string, Float_t>("nnkrec",   event.pileup()));
  NNinputList.push_back(pair<string, Float_t>("ptB",      log(event.signalB()->pt())));

  //double ssign=1;
  //if(ikaon->IPssign()<0) ssign=-1;  
  //double myIPs = ssign*sqrt(abs(ikaon->IPssign()));  
  //NNinputList.push_back(pair<string, Float_t>("IPssign",   myIPs));
  //NNinputList.push_back(pair<string, Float_t>("IPssign",       ikaon->IPssign()));
  NNinputList.push_back(pair<string, Float_t>("IPs",           log(ikaon->IPs())));
  NNinputList.push_back(pair<string, Float_t>("partlcs",       log(ikaon->LCS())));
  //  NNinputList.push_back(pair<string, Float_t>("PIDk",          ikaon->PIDk()));
  //  NNinputList.push_back(pair<string, Float_t>("PIDpk",         ikaon->PIDp()-ikaon->PIDk()));
  NNinputList.push_back(pair<string, Float_t>("PIDNNk",        ikaon->PIDNNk()));
  NNinputList.push_back(pair<string, Float_t>("PIDNNpi",       ikaon->PIDNNpi()));
  NNinputList.push_back(pair<string, Float_t>("PIDNNp",        ikaon->PIDNNp()));
  NNinputList.push_back(pair<string, Float_t>("ghostProb",     log(ikaon->ghostProbability())));
  NNinputList.push_back(pair<string, Float_t>("IPPU",          log(ikaon->IPPU())));

  double pn = nnet.MLPkaonTMVA( NNinputList);

  if(msgLevel(MSG::DEBUG)) debug() << " Kaon pn="<< pn<<endmsg;

  //Calibration (w=1-pn) w' = p0 + p1(w-eta)
  pn = 1 - m_P0_Cal_kaon - m_P1_Cal_kaon * ( (1-pn)-m_Eta_Cal_kaon);

  if( pn < 0 || pn > 1 )  cout<<endl<<"===> Something wrong with  KAON Training "<<pn;/*return tkaon;*/
  if(nnet.m_rnet_k < -0.5 ) {
    cout<<endl<<"===> Something wrong with KAON nnet "<<nnet.m_rnet_k ;
    return tkaon;
  }

  float sign=1;
#ifndef TRAINING  
  if(pn<0.5){
    pn = 1-pn;
    sign=-1;
   }
#endif
  if( pn < m_ProbMin_kaon ) return tkaon;
  if(property["checkDV"]==true) {
    std::cout<<" TaggerKaon: "<<sign*ikaon->charge()<<" omega="<<1-pn<<" NNinputs:";
    typedef std::list<std::pair<std::string, Float_t> > ParListType;
    for(ParListType::const_iterator p = NNinputList.begin(); p != NNinputList.end(); p++) {
      std::cout << p->second<<" ";
    }
    std::cout<<std::endl;
  }
  
  ikaon->setID(321*ikaon->charge());
  tkaon->setOmega( 1-pn );
  tkaon->setRawNNetOutput( nnet.m_rnet_k );
  tkaon->setDecision(sign*(ikaon->charge())>0 ? -1: 1);
  tkaon->setType( Tagger::OS_Kaon ); 
  tkaon->addToTaggerParts(ikaon);

  return tkaon;
}
