#include "configuration.h"
#include "TaggerMuonTool.h"

TaggerMuonTool::TaggerMuonTool( ) {

  declareProperty( "Muon_Pt_cut",      m_Pt_cut_muon  = 1200 );
  declareProperty( "Muon_P_cut",       m_P_cut_muon   = 0.0 );
  declareProperty( "Muon_lcs_cut",     m_lcs_cut_muon = 3.2 );
  declareProperty( "Muon_IPs_cut",     m_IPs_cut_muon = 0.0 );
  declareProperty( "Muon_IPssign_cut", m_IPssign_cut_muon = -9999.0 );
  declareProperty( "Muon_PIDm_cut",    m_PIDm_cut_muon     = 2.5 );
  declareProperty( "Muon_ipPU_cut",    m_ipPU_cut_muon      = 3.0 );
  declareProperty( "Muon_distPhi_cut", m_distPhi_cut_muon= 0.005 );
  declareProperty( "Muon_cand",        m_cand_muon = 10. );
  declareProperty( "Muon_ProbMin",     m_ProbMin_muon = 0. ); //no cut

  declareProperty( "Muon_PIDNNm_cut",    m_PIDNNm_cut_muon     = -1. );
  declareProperty( "Muon_PIDNNpi_cut",   m_PIDNNpi_cut_muon    = 1. );
  declareProperty( "Muon_PIDNNk_cut",    m_PIDNNk_cut_muon     = 1. );
  declareProperty( "Muon_PIDNNe_cut",    m_PIDNNe_cut_muon     = 1. );
  declareProperty( "Muon_PIDNNp_cut",    m_PIDNNp_cut_muon     = 1. );
  declareProperty( "Muon_ghostProb_cut", m_ghostProb_muon= 1. );
  declareProperty( "Muon_ghostProbNN_cut", m_ghostProbNN_muon= 1. );

  declareProperty( "Muon_P0_Cal",      m_P0_Cal_muon   = 0.309 ); 
  declareProperty( "Muon_P1_Cal",      m_P1_Cal_muon   = 1.1953 ); 
  declareProperty( "Muon_Eta_Cal",     m_Eta_Cal_muon  = 0.304 ); 

  NNetTool_MLP nnet;
  tmu = new Tagger();
  
  hcut_mu_ippu   = new TH1F("hcut_mu_ippu","hcut_mu_ippu",100, 0, 20);
  hcut_mu_distphi= new TH1F("hcut_mu_distphi","hcut_mu_distphi",100, 0, 0.09);
  hcut_mu_N      = new TH1F("hcut_mu_N",   "hcut_mu_N",   100, 0, 100);
  hcut_mu_ips    = new TH1F("hcut_mu_ips","hcut_mu_ips", 100, 0, 10);
  hcut_mu_ipssign= new TH1F("hcut_mu_ipssign","hcut_mu_ipssign", 100, -10, 10);
  hcut_mu_pid    = new TH1F("hcut_mu_pid","hcut_mu_pid", 100, -10, 40);
  hcut_mu_pt     = new TH1F("hcut_mu_pt", "hcut_mu_pt",  100, 0, 10);
  hcut_mu_p      = new TH1F("hcut_mu_p",  "hcut_mu_p",   100, 0, 100);
  hcut_mu_lcs    = new TH1F("hcut_mu_lcs","hcut_mu_lcs", 100, 0, 10);
  hcut_mu_cand   = new TH1F("hcut_mu_cand","hcut_mu_cand", 5, 0, 5);

  
  

}

/////////////////////////////////////////////
Tagger* TaggerMuonTool::tag(Event& event) {
  tmu->reset();

  if(msgLevel(MSG::VERBOSE)) verbose()<<"--Muon Tagger--"<<endreq;

  Particle* imuon=NULL;
  double ptmaxm=-999, ncand=0;
  Particles::iterator ipart;
  Particles vtags = event.particles();
  
  for (ipart=vtags.begin(); ipart!=vtags.end(); ++ipart) {

    Particle* axp = (*ipart);
    if(msgLevel(MSG::VERBOSE)) verbose() << " Muon PIDm"<< axp->PIDm()<<endreq;
    if(!checkPIDhypo(Particle::muon, axp)) continue;
    if(msgLevel(MSG::VERBOSE)) verbose() << " Muon PIDm"<< axp->PIDm()<<endreq;

#ifdef NEWCUTS 
    if(axp->ghostProbability() >  m_ghostProb_muon ) continue; 
    if(axp->ghostProbabilityNN() >  m_ghostProbNN_muon ) continue; 
    if(axp->PIDNNm() <  m_PIDNNm_cut_muon ) continue;    
    if(axp->PIDNNpi()>  m_PIDNNpi_cut_muon ) continue;    
    if(axp->PIDNNp() >  m_PIDNNp_cut_muon ) continue;    
    if(axp->PIDNNe() >  m_PIDNNe_cut_muon ) continue;    
    if(axp->PIDNNk() >  m_PIDNNk_cut_muon ) continue;    
#endif

    double Pt = axp->pt();
    if( Pt < m_Pt_cut_muon*0.001 ) continue;//to have GeV

    double P = axp->p();
    if( P  < m_P_cut_muon*0.001 ) continue;//to have GeV
    if(msgLevel(MSG::VERBOSE)) verbose() << " Muon P="<< P <<" Pt="<< Pt <<endreq;

    double lcs = axp->LCS();
    if( lcs > m_lcs_cut_muon) continue;
    if(msgLevel(MSG::VERBOSE)) verbose() << " Muon lcs="<< lcs <<endreq;

    //calculate signed IP wrt RecVert
    double IPs = axp->IPs();         // is fabs
    if(IPs < m_IPs_cut_muon ) continue;
    if(msgLevel(MSG::VERBOSE)) verbose() << " IPs="<< IPs <<endreq;
    double sIPs = axp->IPssign();
    if(sIPs < m_IPssign_cut_muon ) continue;
    if(msgLevel(MSG::VERBOSE)) verbose() << " sIPs="<< sIPs <<endreq;

    //ip and phi wrt PU
    double ipPU  = axp->IPPU();
    if(msgLevel(MSG::VERBOSE)) verbose() << " ipPU="<< ipPU <<endreq;
    if(ipPU < m_ipPU_cut_muon ) continue;
    double distPhi = axp->distPhi();
    if (distPhi < m_distPhi_cut_muon) continue;
    if(msgLevel(MSG::VERBOSE)) verbose() << " distPhi="<< distPhi <<endreq;

    ncand++;

    hcut_mu_ippu   ->Fill(axp->IPPU());
    hcut_mu_distphi->Fill(distPhi);
    hcut_mu_N      ->Fill(event.multiplicity());
    hcut_mu_ips    ->Fill(axp->IPs());
    hcut_mu_ipssign->Fill(axp->IPssign());
    hcut_mu_pid    ->Fill(axp->PIDm());
    hcut_mu_pt     ->Fill(Pt);
    hcut_mu_p      ->Fill(P);
    hcut_mu_lcs    ->Fill(axp->LCS());
    (*ipart)->setIDtag(-13*(*ipart)->charge());
    if( Pt > ptmaxm ) { //Pt ordering
      imuon = (*ipart);
      ptmaxm = Pt;
    }
    
  }  

  hcut_mu_cand   ->Fill(ncand);
  if(!imuon) return tmu;
  if(ncand>m_cand_muon) return tmu;
 
  
  //calculate omega
  list<std::pair<string, Float_t> > NNinputList;

  NNinputList.push_back(pair<string, Float_t>("mult",   event.multiplicity()));
  NNinputList.push_back(pair<string, Float_t>("partP",  log(imuon->p())));
  NNinputList.push_back(pair<string, Float_t>("partPt", log(imuon->pt())));
  //===== NNinputList.push_back(pair<string, Float_t>("nnkrec", event.pileup()));
  NNinputList.push_back(pair<string, Float_t>("ptB",    log(event.signalB()->pt())));
  //double ssign=1;
  //if(imuon->IPssign()<0) ssign=-1;  
  //double myIPs = ssign*sqrt(abs(imuon->IPssign()));  
  //NNinputList.push_back(pair<string, double>("IPssign",   myIPs));
  //NNinputList.push_back(pair<string, double>("IPssign",     imuon->IPssign()));
  NNinputList.push_back(pair<string, Float_t>("IPs",    log(imuon->IPs())));
  NNinputList.push_back(pair<string, Float_t>("partlcs",log(imuon->LCS())));
  //=======  NNinputList.push_back(pair<string, Float_t>("PIDm",   imuon->PIDm()));
  NNinputList.push_back(pair<string, Float_t>("PIDNNm", imuon->PIDNNm()));
  //=======  NNinputList.push_back(pair<string, Float_t>("PIDNNpi", imuon->PIDNNpi()));
  NNinputList.push_back(pair<string, Float_t>("ghostProb",log(imuon->ghostProbability())));  
  NNinputList.push_back(pair<string, Float_t>("IPPU",   log(imuon->IPPU())));

  double pn = nnet.MLPmTMVA( NNinputList);  
  
  if(msgLevel(MSG::VERBOSE)) verbose() << " Muon pn="<< pn <<endreq;

  //Calibration (w=1-pn) w' = p0 + p1(w-eta)
  pn = 1 - m_P0_Cal_muon - m_P1_Cal_muon * ( (1-pn)-m_Eta_Cal_muon);
  debug() << " Muon pn="<< pn <<" w="<<1-pn<<endreq;
  
  if( pn < 0 || pn > 1 ) cout<<endl<<"===> Something wrong with MUON Training "<<pn;/*return tmu;*/
  if(nnet.m_rnet_mu < -0.5 ) {
    cout<<endl<<"===> Something wrong with MUON nnet "<<nnet.m_rnet_mu ;
    return tmu;
  }
  
  if( pn < m_ProbMin_muon ) return tmu;

  imuon->setIDtag(-13*imuon->charge());
  imuon->setID(-13*imuon->charge());
  float sign=1;
#ifndef TRAINING
  if(pn<0.5){
    pn = 1-pn;
    sign=-1;
   }
#endif


  if(property["checkDV"]==true) {
    
    std::cout<<" TaggerMuon: "<<sign*imuon->charge()<<" omega="<<1-pn<<" NNinputs:";
    
    typedef std::list<std::pair<std::string, Float_t> > ParListType;
    for(ParListType::const_iterator p = NNinputList.begin(); p != NNinputList.end(); p++) {
      std::cout << p->second<<" ";
    }
    std::cout<<" PV=("<<event.RecVertex().x()<<","<<event.RecVertex().y()<<","<<event.RecVertex().z()<<")";
    
    std::cout<<std::endl;
  }
  
  tmu->setOmega( 1-pn );
  tmu->setRawNNetOutput( nnet.m_rnet_mu );
  tmu->setDecision(sign*(imuon->charge()) > 0 ? -1: 1);
  tmu->setType( Tagger::OS_Muon ); 
  tmu->addToTaggerParts(imuon);
  //cout<<" charge "<<imuon->charge()<<" p "<<imuon->p()<<endl;

  return tmu;
}
