#include "configuration.h"
#include "TaggerPionSameTool.h"

TaggerPionSameTool::TaggerPionSameTool() {

  declareProperty( "PionSame_Pt_cut",       m_Pt_cut_pionS  = 600 );
  declareProperty( "PionSame_P_cut",        m_P_cut_pionS   = 4000 );
  declareProperty( "PionSame_IPs_cut",      m_IPs_cut_pionS = 9. );
  declareProperty( "PionSame_IPssign_cut",  m_IPssign_cut_pionS = 9999.0 );
  declareProperty( "PionSame_IPssignPlus_cut",  m_IPssignPlus_cut_pionS = -1.0 );
  declareProperty( "PionSame_IPssignMinus_cut",  m_IPssignMinus_cut_pionS =  1.0 );
  declareProperty( "PionSame_lcs_cut",      m_lcs_cut_pionS       = 5.0 );
  declareProperty( "PionSame_dphi_cut",     m_phi_cut_pionS = 0.75 );
  declareProperty( "PionSame_deta_max_cut", m_eta_max_cut_pionS = 0.35 );
  declareProperty( "PionSame_deta_min_cut", m_eta_min_cut_pionS = -0.5 );
  declareProperty( "PionSame_dR_cut",       m_dR_cut_pionS= 0.75 );
  declareProperty( "PionSame_dQ_cut",       m_dQ_cut_pionS   = 2500);
  declareProperty( "PionSame_dQ_extra_cut", m_dQextra_cut_pionS = 1200);
  declareProperty( "PionSame_ghost_cut",    m_ghost_cut_pionS     = -999.0);
  declareProperty( "PionSame_ipPU_cut",     m_ipPU_cut_pionS      = 6.0 );
  declareProperty( "PionSame_distPhi_cut",  m_distPhi_cut_pionS= 0.005 );
  declareProperty( "PionSame_PIDNoK_cut",   m_PIDNoK_cut_pionS = 4.3);
  declareProperty( "PionSame_PIDNoP_cut",   m_PIDNoP_cut_pionS = 14.0);
  declareProperty( "PionSame_cand",         m_cand_pionS   = 100.);
  declareProperty( "PionSame_ProbMin",      m_ProbMin_pionS   = 0.56);

  declareProperty( "PionSame_ghostProb_cut",m_ghostProb_pionS   = 10. );
  declareProperty( "PionSame_PIDNNpi_cut",  m_PIDNNpi_cut_pionS = -100. );
  declareProperty( "PionSame_PIDNNk_cut",   m_PIDNNk_cut_pionS  = 100. );
  declareProperty( "PionSame_PIDNNp_cut",   m_PIDNNp_cut_pionS  = 100. );

  declareProperty( "PionSame_P0_Cal",       m_P0_Cal_pionS   = 0.407 ); 
  declareProperty( "PionSame_P1_Cal",       m_P1_Cal_pionS   = 0.76 ); 
  declareProperty( "PionSame_P2_Cal",       m_P2_Cal_pionS   = -2.2 ); 
  declareProperty( "PionSame_Eta_Cal",      m_Eta_Cal_pionS  = 0.374 ); 

  NNetTool_MLP nnet;
  tpionS = new Tagger();

  hcut_pS_ippu   = new TH1F("hcut_pS_ippu","hcut_pS_ippu",100, 0, 20);
  hcut_pS_distphi= new TH1F("hcut_pS_distphi","hcut_pS_distphi",100, 0, 0.09);
  hcut_pS_pt     = new TH1F("hcut_pS_pt",  "hcut_pS_pt",  100, 0, 10);
  hcut_pS_p      = new TH1F("hcut_pS_p",   "hcut_pS_p",   100, 0, 100);
  hcut_pS_lcs    = new TH1F("hcut_pS_lcs", "hcut_pS_lcs", 100, 0, 10);
  hcut_pS_IPs    = new TH1F("hcut_pS_IPs", "hcut_pS_IPs", 100, 0, 20);
  hcut_pS_IPssign= new TH1F("hcut_pS_IPssign", "hcut_pS_IPssign", 100, -20, 20);
  hcut_pS_gho    = new TH1F("hcut_pS_gho", "hcut_pS_gho", 100, -50, 0);
  hcut_pS_dphi   = new TH1F("hcut_pS_dphi","hcut_pS_dphi",100, 0, 3);
  hcut_pS_deta   = new TH1F("hcut_pS_deta","hcut_pS_deta",100, 0, 3);
  hcut_pS_dr     = new TH1F("hcut_pS_dr",  "hcut_pS_dr",  100, 0, 5);
  hcut_pS_dq     = new TH1F("hcut_pS_dq",  "hcut_pS_dq",  100, 0, 3);
  hcut_pS_dqe    = new TH1F("hcut_pS_dqe", "hcut_pS_dqe", 100, 0, 3);
  hcut_pS_cand   = new TH1F("hcut_pS_cand", "hcut_pS_cand", 5, 0, 5);

  hcut_pt_1      = new TH1F("hcut_pt_1",   "hcut_pt_1",   100, 0, 10);
  hcut_pt_2      = new TH1F("hcut_pt_2",   "hcut_pt_2",   100, 0, 10);
  hcut_pt_3      = new TH1F("hcut_pt_3",   "hcut_pt_3",   100, 0, 10);
  hcut_pt_4      = new TH1F("hcut_pt_4",   "hcut_pt_4",   100, 0, 10);
  hcut_pt_5      = new TH1F("hcut_pt_5",   "hcut_pt_5",   100, 0, 10);
  hcut_pt_1_r      = new TH1F("hcut_pt_1_r",   "hcut_pt_1_r",   100, 0, 10);
  hcut_pt_2_r      = new TH1F("hcut_pt_2_r",   "hcut_pt_2_r",   100, 0, 10);
  hcut_pt_3_r      = new TH1F("hcut_pt_3_r",   "hcut_pt_3_r",   100, 0, 10);
  hcut_pt_4_r      = new TH1F("hcut_pt_4_r",   "hcut_pt_4_r",   100, 0, 10);
  hcut_pt_5_r      = new TH1F("hcut_pt_5_r",   "hcut_pt_5_r",   100, 0, 10);
  hcut_pt_1_w      = new TH1F("hcut_pt_1_w",   "hcut_pt_1_w",   100, 0, 10);
  hcut_pt_2_w      = new TH1F("hcut_pt_2_w",   "hcut_pt_2_w",   100, 0, 10);
  hcut_pt_3_w     = new TH1F("hcut_pt_3_w",   "hcut_pt_3_w",   100, 0, 10);
  hcut_pt_4_w      = new TH1F("hcut_pt_4_w",   "hcut_pt_4_w",   100, 0, 10);
  hcut_pt_5_w      = new TH1F("hcut_pt_5_w",   "hcut_pt_5_w",   100, 0, 10);


}

///////////////////////////////////////////////
Tagger* TaggerPionSameTool::tag(Event& event) {
  tpionS->reset();

  verbose()<<"--Pion SS Tagger--"<<endreq;

  TLorentzVector ptotB = event.signalB()->momentum();
  double B0mass = ptotB.M();

  Particle* ipionS=NULL;
  double ptmaxpS = -99.0, ncand=0;
  Particles::iterator ipart, jpart;

  Particles vtags = event.particles();
  for( ipart = vtags.begin(); ipart != vtags.end(); ipart++ ) {

    if(!checkPIDhypo(Particle::pion, *ipart)) continue;  // included cuts in the PID, ID, track type
    verbose()<<" Pion PIDk="<< (*ipart)->PIDk() <<endreq;

    double Pt = (*ipart)->pt();
    if( Pt < m_Pt_cut_pionS*0.001 )  continue;//GeV
    double P  = (*ipart)->p();
    if( P  < m_P_cut_pionS*0.001 )  continue;//GeV
    verbose()<<" Pion P="<< P <<" Pt="<< Pt <<endreq;

    
#ifdef NEWCUTS
    if((*ipart)->ghostProbability() >  m_ghostProb_pionS ) continue;    
    if((*ipart)->PIDNNpi() <  m_PIDNNpi_cut_pionS ) continue;    
    if((*ipart)->PIDNNk()  >  m_PIDNNk_cut_pionS ) continue;    
    if((*ipart)->PIDNNp()  >  m_PIDNNp_cut_pionS ) continue; 
    //    cout<<" "<<(*ipart)->ghostProbability()<<" "<<(*ipart)->PIDNNpi()<<" "<<  (*ipart)->PIDNNk()<<" "<<(*ipart)->PIDNNp()<<endl;    
#endif


    double lcs = (*ipart)->LCS();
    if( lcs > m_lcs_cut_pionS ) continue;
    //if( (*ipart)->type() != Particle::Long && 
    //    (*ipart)->type() != Particle::Matched ) continue; //old FTC long and matched tracks are trtyp=7
    if( (*ipart)->type() != Particle::Long ) continue; 

    double tsa = (*ipart)->likelihood();
    if(tsa < m_ghost_cut_pionS) continue;
    verbose() << " Pion lcs="<< lcs <<" tsa="<<tsa<<endmsg;

    double IPsig = (*ipart)->IPs(); // is fabs
    if(IPsig > m_IPs_cut_pionS)  continue;
    
    double IPssig = (*ipart)->IPssign();    
    if(IPssig > m_IPssign_cut_pionS)  continue;
    if ( IPssig>0 && IPssig<m_IPssignPlus_cut_pionS ) continue;
    if ( IPssig<0 && IPssig>m_IPssignMinus_cut_pionS ) continue;
    
    double dQ = (ptotB+(*ipart)->momentum()).M() - B0mass;
    if(dQ > m_dQ_cut_pionS*0.001 ) continue;
    verbose() << " Pion IPs="<< IPsig <<" dQ="<<dQ<<endmsg;

    //ip and phi wrt PU
    double ipPU  = (*ipart)->IPPU();
    if(ipPU < m_ipPU_cut_pionS ) continue;

    double distPhi = (*ipart)->distPhi();
    if (distPhi < m_distPhi_cut_pionS) continue;
  
    double B0the= ptotB.Theta();
    double B0phi= ptotB.Phi();
    double ang = asin(((*ipart)->pt())/((*ipart)->p()));
    double deta= log(tan(B0the/2.))-log(tan(ang/2.));
    double dphi = fabs((*ipart)->momentum().Phi()-B0phi);
    if(dphi>PI) dphi=2.*PI-dphi;
    
    double dR = sqrt(deta*deta+dphi*dphi);    

    if(deta > m_eta_max_cut_pionS) continue;
    if(deta < m_eta_min_cut_pionS) continue;
    if(dphi > m_phi_cut_pionS) continue;
    if(dR > m_dR_cut_pionS) continue; 

    ncand++;
 
    hcut_pS_pt     ->Fill(Pt);
    hcut_pS_p      ->Fill(P);
    hcut_pS_lcs    ->Fill((*ipart)->LCS());
    hcut_pS_IPs    ->Fill(IPsig);
    hcut_pS_IPssign->Fill(IPssig);
    hcut_pS_gho    ->Fill(tsa);
    hcut_pS_deta   ->Fill(deta);
    hcut_pS_dphi   ->Fill(dphi);
    hcut_pS_dr     ->Fill(dR);
    hcut_pS_dq     ->Fill(dQ);

    if( Pt <= ptmaxpS ) continue;//equal sign in "<=" is used to kill duplicates
 
    //accept candidate with the largest pT
    ipionS = (*ipart);
    ptmaxpS = Pt;
    
  } //------- End loop over the tagging particles
  hcut_pS_cand   ->Fill(ncand);

  if (ncand==1) hcut_pt_1   ->Fill(ptmaxpS);
  if (ncand==2) hcut_pt_2   ->Fill(ptmaxpS);
  if (ncand==3) hcut_pt_3   ->Fill(ptmaxpS);
  if (ncand==4) hcut_pt_4   ->Fill(ptmaxpS);
  if (ncand==5) hcut_pt_5   ->Fill(ptmaxpS);
  
  if( !ipionS ) return tpionS;
  if(ncand>m_cand_pionS) return tpionS;

  double extra_dQ = (ptotB+ipionS->momentum()).M() - B0mass;
  if( extra_dQ > m_dQextra_cut_pionS*0.001 ) return tpionS;
  verbose() << " Pion dQExtra="<< extra_dQ <<endmsg;

  hcut_pS_dqe ->Fill(extra_dQ);

  //calculate omega
  double B0the= ptotB.Theta();
  double B0phi= ptotB.Phi();
  double ang = asin((ipionS->pt())/(ipionS->p()));
  double deta= log(tan(B0the/2.))-log(tan(ang/2.));
  double dphi= std::min(fabs(ipionS->momentum().Phi()-B0phi), 
                        2.*PI-fabs(ipionS->momentum().Phi()-B0phi));
  double dQ  = ((ptotB+ ipionS->momentum() ).M() - B0mass);
  double dR = sqrt(deta*deta+dphi*dphi);    
  double partPhi = (ipionS)->Phi();

  list<std::pair<string, Float_t> > NNinputList;


  NNinputList.push_back(pair<string, Float_t>("mult",   event.multiplicity()));
  NNinputList.push_back(pair<string, Float_t>("partP",  log(ipionS->p())));
  // Aggiungere IPs
  NNinputList.push_back(pair<string, Float_t>("partPt", log(ipionS->pt())));
  NNinputList.push_back(pair<string, Float_t>("nnkrec", event.pileup()));
  NNinputList.push_back(pair<string, Float_t>("ptB",    log(event.signalB()->pt())));
  NNinputList.push_back(pair<string, Float_t>("nndeta", deta));
  NNinputList.push_back(pair<string, Float_t>("nndphi", dphi));
  NNinputList.push_back(pair<string, Float_t>("nndq",   dQ/GeV));
  NNinputList.push_back(pair<string, Float_t>("nndr",   dR));

  double pn = nnet.MLPpionSTMVA( NNinputList );
  verbose() << " Pion pn="<< pn <<endmsg;

  //Calibration (w=1-pn) w' = p0 + p1(w-eta)
  pn = 1 - m_P0_Cal_pionS - m_P1_Cal_pionS * ((1-pn)-m_Eta_Cal_pionS) - m_P2_Cal_pionS * ((1-pn)-m_Eta_Cal_pionS) * ((1-pn)-m_Eta_Cal_pionS);
  debug() << " Pion pn="<< pn <<" w="<<1-pn<<endmsg;
  if( pn < 0 || pn > 1) cout<<endl<<"===> Something wrong with PIONS Training "<<pn;/*return tmu;*/
  if(nnet.m_rnet_pS < -0.5 ) {
    cout<<endl<<"===> Something wrong with SSPi nnet "<<nnet.m_rnet_pS ;
    return tpionS;
  }

  if( pn < m_ProbMin_pionS ) return tpionS;

  int tagdecision = ipionS->charge()>0 ? 1: -1;
  if( event.isBu() ) tagdecision = -tagdecision; //is Bu

  float sign=1;
#ifndef TRAINING  
  if(pn<0.5){
    pn = 1-pn;
    sign=-1;
   }
#endif
  double omega = 1-pn;

  tpionS->setDecision( sign*tagdecision );
  tpionS->setRawNNetOutput( nnet.m_rnet_pS );
  tpionS->setOmega( omega );
  tpionS->setType( Tagger::SS_Pion );
  tpionS->addToTaggerParts(ipionS);

  if(event.TrueTag()==tagdecision){
    if (ncand==1) hcut_pt_1_r   ->Fill(ptmaxpS);
    if (ncand==2) hcut_pt_2_r   ->Fill(ptmaxpS);
    if (ncand==3) hcut_pt_3_r   ->Fill(ptmaxpS);
    if (ncand==4) hcut_pt_4_r   ->Fill(ptmaxpS);
    if (ncand==5) hcut_pt_5_r   ->Fill(ptmaxpS);
  }
  if(event.TrueTag()==-tagdecision){
    if (ncand==1) hcut_pt_1_w   ->Fill(ptmaxpS);
    if (ncand==2) hcut_pt_2_w   ->Fill(ptmaxpS);
    if (ncand==3) hcut_pt_3_w   ->Fill(ptmaxpS);
    if (ncand==4) hcut_pt_4_w   ->Fill(ptmaxpS);
    if (ncand==5) hcut_pt_5_w   ->Fill(ptmaxpS);
  }
  
  return tpionS;
}
