#ifndef TaggerPionSameTool_H
#define TaggerPionSameTool_H 1

#include <iostream>
#include <iomanip>
#include <sstream>
#include <ostream>
#include <string>
#include <vector>
#include <list>
#include <TLorentzVector.h>
#include "Tagger.h"
#include "Event.h"
#include "GenericTool.h"
#include "NNetTool_MLP.h"

#include "TH1F.h"
#include "TH2F.h"

using namespace std;

/** @class TaggerPionSameTool
 *
 *  Tool to retrieve a tagger by using pions 
 *   from fragmentation close to signal B meson
 *
 *  @author Marco Musy
 *  @date   2010-07-15
 */

class TaggerPionSameTool  : public GenericTool {
 
 public:
 
  TaggerPionSameTool( );
  ~TaggerPionSameTool( ){ delete tpionS; } ///< Destructor

  Tagger* tag(Event& );

 private:

  double m_Pt_cut_pionS, m_P_cut_pionS, m_IPs_cut_pionS, m_IPssign_cut_pionS, m_IPssignPlus_cut_pionS, m_IPssignMinus_cut_pionS, m_lcs_cut_pionS, m_dQ_cut_pionS,  m_dQextra_cut_pionS, m_ghost_cut_pionS;
  double m_phi_cut_pionS, m_eta_max_cut_pionS, m_eta_min_cut_pionS, m_dR_cut_pionS;
  double m_ipPU_cut_pionS, m_distPhi_cut_pionS;
  double m_cand_pionS;
  double m_ProbMin_pionS;
  double m_P0_Cal_pionS ;
  double m_P1_Cal_pionS ;
  double m_P2_Cal_pionS ;
  double m_Eta_Cal_pionS ;

  NNetTool_MLP nnet;
  Tagger* tpionS;

  TH1F* hcut_pS_pt ;
  TH1F* hcut_pS_p   ;
  TH1F* hcut_pS_lcs ;
  TH1F* hcut_pS_IPs ;
  TH1F* hcut_pS_IPssign ;
  TH1F* hcut_pS_gho ;
  TH1F* hcut_pS_deta  ;
  TH1F* hcut_pS_dphi  ;
  TH1F* hcut_pS_dr  ;
  TH1F* hcut_pS_dq  ;
  TH1F* hcut_pS_dqe ;
  TH1F* hcut_pS_ippu ;
  TH1F* hcut_pS_distphi ;
  TH1F* hcut_pS_cand ;

  TH1F* hcut_pt_1 ;
  TH1F* hcut_pt_2 ;
  TH1F* hcut_pt_3 ;
  TH1F* hcut_pt_4 ;
  TH1F* hcut_pt_5 ;
  TH1F* hcut_pt_0_r ;
  TH1F* hcut_pt_1_r ;
  TH1F* hcut_pt_2_r ;
  TH1F* hcut_pt_3_r ;
  TH1F* hcut_pt_4_r ;
  TH1F* hcut_pt_5_r ;
  TH1F* hcut_pt_0_w ;
  TH1F* hcut_pt_1_w ;
  TH1F* hcut_pt_2_w ;
  TH1F* hcut_pt_3_w ;
  TH1F* hcut_pt_4_w ;
  TH1F* hcut_pt_5_w ;

  TH2F* hcut_corr_pS[10][10];

};

#endif 
