#include "configuration.h"
#include "TaggerVertexChargeTool.h"
#include "NNetTool_MLP.h"

TaggerVertexChargeTool::TaggerVertexChargeTool() {

    //Tune for 2011 reco10 data

    declareProperty( "SecondaryVertexName",
		     m_SecondaryVertexToolName = "SVertexOneSeedTool");

    //"NNet", "Probability"
    declareProperty( "UseObsoleteSV", m_UseObsoleteSV       = false );
    declareProperty( "Vtx_AverageOmega",  m_AverageOmega        = 0.41 );
    
    declareProperty( "Vtx_PowerK",        m_PowerK              = 0.55 ); //0.4
    declareProperty( "Vtx_MinimumVCharge",m_MinimumVCharge      = 0.2 ); //0.275
    declareProperty( "Vtx_Ptsum",         m_Ptsum_vtx           = 1.55);//1.5
    declareProperty( "Vtx_Ptmean",        m_Ptmean_vtx          = 0.);
    declareProperty( "Vtx_IPSsum",        m_IPSsum_vtx          = 0.);//10
    declareProperty( "Vtx_DocaMaxsum",    m_DocaMaxsum_vtx      = 0.5);
    declareProperty( "Vtx_Psum",          m_Psum_vtx            = 8.);//10
    declareProperty( "Vtx_Msum",          m_Msum_vtx            = 0.6);//0.5
    declareProperty( "Vtx_Magsum",        m_Magsum_vtx          = 0.);
    declareProperty( "Vtx_Bdphidir",      m_Bdphidir_vtx        = 50.);
    declareProperty( "Vtx_ProbMin",       m_ProbMin_vtx         = 0.54);//0.54
    declareProperty( "Vtx_P0_Cal",  m_P0_Cal_vtx   = 0.404 ); 
    declareProperty( "Vtx_P1_Cal",  m_P1_Cal_vtx   = 0.84 ); 
    declareProperty( "Vtx_Eta_Cal", m_Eta_Cal_vtx  = 0.362 ); 

    //For Obsolete SV:
    declareProperty( "P0",           m_P0                   =  5.255669e-01 );
    declareProperty( "P1",           m_P1                   = -3.251661e-01 );
    declareProperty( "Gt075",        m_Gt075                = 0.35 );
    declareProperty( "TracksEq2",    m_wSameSign2           = 0.4141 );
    declareProperty( "TracksGt2",    m_wSameSignMoreThan2   = 0.3250 );
    
    NNetTool_MLP nnet;
    tVch = new Tagger();
    if(m_SecondaryVertexToolName=="SVertexOneSeedTool") 
      m_svtool= new SVertexOneSeedTool();
    else err()<<"Unknown SecondaryVertexName: "
	      <<m_SecondaryVertexToolName <<endmsg;
    
    //Histo definition
    hr_vtx_vmaxprobf = new TH1F("hr_vtx_vmaxprobf","hr_vtx_vmaxprobf", 50, 0, 1);
    hw_vtx_vmaxprobf = new TH1F("hw_vtx_vmaxprobf","hw_vtx_vmaxprobf", 50, 0, 1);
    hr_vtx_vptmean = new TH1F("hr_vtx_vptmean","hr_vtx_vptmean", 50, 0, 6);
    hw_vtx_vptmean = new TH1F("hw_vtx_vptmean","hw_vtx_vptmean", 50, 0, 6);
    hr_vtx_vptsum = new TH1F("hr_vtx_vptsum","hr_vtx_vptsum", 50, 0, 10);
    hw_vtx_vptsum = new TH1F("hw_vtx_vptsum","hw_vtx_vptsum", 50, 0, 10);
    hr_vtx_vipsmean = new TH1F("hr_vtx_vipsmean","hr_vtx_vipsmean", 50, 0, 30);
    hw_vtx_vipsmean = new TH1F("hw_vtx_vipsmean","hw_vtx_vipsmean", 50, 0, 30);
    hr_vtx_vdocamax = new TH1F("hr_vtx_vdocamax","hr_vtx_vdocamax", 50, 0, 0.1);
    hw_vtx_vdocamax = new TH1F("hw_vtx_vdocamax","hw_vtx_vdocamax", 50, 0, 0.1);
    hr_vtx_vdphidir = new TH1F("hr_vtx_vdphidir","hr_vtx_vdphidir", 50, 0, 3.2);
    hw_vtx_vdphidir = new TH1F("hw_vtx_vdphidir","hw_vtx_vdphidir", 50, 0, 3.2);
    hr_vtx_vsvm = new TH1F("hr_vtx_vsvm","hr_vtx_vsvm", 50, 0, 7);
    hw_vtx_vsvm = new TH1F("hw_vtx_vsvm","hw_vtx_vsvm", 50, 0, 7);
    hr_vtx_vsvp = new TH1F("hr_vtx_vsvp","hr_vtx_vsvp", 50, 0, 70);
    hw_vtx_vsvp = new TH1F("hw_vtx_vsvp","hw_vtx_vsvp", 50, 0, 70);
    hr_vtx_vsvpt = new TH1F("hr_vtx_vsvpt","hr_vtx_vsvpt", 50, 0, 7);
    hw_vtx_vsvpt = new TH1F("hw_vtx_vsvpt","hw_vtx_vsvpt", 50, 0, 7);
    hr_vtx_vsvtheta = new TH1F("hr_vtx_vsvtheta","hr_vtx_vsvtheta", 50, 0, 0.4);
    hw_vtx_vsvtheta = new TH1F("hw_vtx_vsvtheta","hw_vtx_vsvtheta", 50, 0, 0.4);
    hr_vtx_vsvphi = new TH1F("hr_vtx_vsvphi","hr_vtx_vsvphi", 50, 0, 4);
    hw_vtx_vsvphi = new TH1F("hw_vtx_vsvphi","hw_vtx_vsvphi", 50, 0, 4);
    hr_vtx_vsvdeltar = new TH1F("hr_vtx_vsvdeltar","hr_vtx_vsvdeltar", 50, 0, 10);
    hw_vtx_vsvdeltar = new TH1F("hw_vtx_vsvdeltar","hw_vtx_vsvdeltar", 50, 0, 10);
    hr_vtx_vsvmag = new TH1F("hr_vtx_vsvmag","hr_vtx_vsvmag", 50, 0, 10);
    hw_vtx_vsvmag = new TH1F("hw_vtx_vsvmag","hw_vtx_vsvmag", 50, 0, 10);
    
    hr_tr_pt = new TH1F("hr_tr_pt","hr_tr_pt", 50, 0, 8);
    hw_tr_pt = new TH1F("hw_tr_pt","hw_tr_pt", 50, 0, 8);
    hr_tr_ip = new TH1F("hr_tr_ip","hr_tr_ip", 50, 0, 10);
    hw_tr_ip = new TH1F("hw_tr_ip","hw_tr_ip", 50, 0, 10);
    hr_tr_ips = new TH1F("hr_tr_ips","hr_tr_ips", 50, 0, 15);
    hw_tr_ips = new TH1F("hw_tr_ips","hw_tr_ips", 50, 0, 15);
    hr_tr_ipsv = new TH1F("hr_tr_ipsv","hr_tr_ipsv", 50, 0, 1);
    hw_tr_ipsv = new TH1F("hw_tr_ipsv","hw_tr_ipsv", 50, 0, 1);
    hr_tr_lcs = new TH1F("hr_tr_lcs","hr_tr_lcs", 50, 0, 5);
    hw_tr_lcs = new TH1F("hw_tr_lcs","hw_tr_lcs", 50, 0, 5);
    hr_tr_docamax = new TH1F("hr_tr_docamax","hr_tr_docamax", 50, 0, 0.3);
    hw_tr_docamax = new TH1F("hw_tr_docamax","hw_tr_docamax", 50, 0, 0.3);

}

///////////////////////////////////////////////////
Tagger* TaggerVertexChargeTool::tag(Event& event) {
    tVch->reset();
    
    verbose()<<"--VertexCharge Tagger--"<<endreq;
    
    Particles parts = event.particles();
    Particles::iterator i, ip;
    Particles Pfit; Pfit.clear();
    double maxprobf = 0;
    TVector3 BoppDir;
    
    ///--- Inclusive Secondary Vertex ---
    //look for a secondary Vtx due to opposite B
    Vertex* mysecvtx=NULL;
  
    if(m_UseObsoleteSV) { //use DaVinci processed information
      for (i=parts.begin(); i!=parts.end(); ++i) {
        if((*i)->ComesFromSVtx()) Pfit.push_back(*i);
      }
    } else { //use seed from DaVinci and add tracks custom
      Vertices allvtx = m_svtool->buildVertex( parts, &event );
      if(allvtx.empty()) return tVch;
      mysecvtx = allvtx.at(0);
      if( mysecvtx->type() != Vertex::Secondary ) return tVch;
      Pfit = mysecvtx->outgoingParticles();
      maxprobf= mysecvtx->likelihood();
      BoppDir = mysecvtx->position();//point and svtau
    }
    
    if(msgLevel(MSG::DEBUG)) debug()<<"Pfit.size: "<<Pfit.size()<<endreq;
    
    //if Vertex does not contain any daughters, exit
    if(Pfit.size()<1) return tVch;
    if(msgLevel(MSG::DEBUG)) debug()<<"--- SVTOOL buildVertex returns vertex"
                                    <<" with "<<Pfit.size()<<"tracks"<<endreq;
    if(msgLevel(MSG::DEBUG)) debug()<<" -- likelihood seed "<<maxprobf<<endreq;
    
    //calculate vertex charge and other variables for NN
    double Vch = 0, norm = 0;
    double Vptmean = 0, Vipsmean = 0, Vflaglong = 0, Vdocamax = 0;
    int vflagged = 0;
    TLorentzVector SVmomentum;
    for(ip=Pfit.begin(); ip!=Pfit.end(); ip++) { 
      if(msgLevel(MSG::DEBUG)) 
        debug() <<"SVTOOL  VtxCh, adding track pt= "<<(*ip)->pt()<<
        " docaSV:"<<(*ip)->DOCA()<<endreq;      
      SVmomentum += (*ip)->momentum();//for tau and point
      double a = pow((*ip)->pt()/GeV, m_PowerK);
      Vch += (*ip)->charge() * a;
      norm+= a;
      vflagged++;
      Vptmean  += (*ip)->pt()/GeV;
      Vipsmean += (*ip)->IPs();
      if( (*ip)->type()== Particle::Long ) Vflaglong++; 
      /*
      if( (*ip)->type()== Particle::Long || 
          (*ip)->type()== Particle::Matched) Vflaglong++; //old FTC long and matched have trtyp=7
      */
      Vdocamax += (*ip)->DOCA();

    }
    if(norm) {
      Vch /= norm;
      //    tVch->setcharge( Vch ); //to use only for Vch plot (plot before cut)
      if(fabs(Vch) < m_MinimumVCharge ) Vch = 0;
      Vptmean  /= vflagged;
      Vipsmean /= vflagged;
      Vdocamax/= vflagged;
    }
    if(msgLevel(MSG::DEBUG)) debug()<<"Vch: "<<Vch<<" Vptmean:"<<Vptmean<<" Vipsmean:"<<Vipsmean<<" Vdocamax:"<<Vdocamax<<endreq;
    
    if( Vch==0 ) return tVch;
    
    //Variables of the SV
    double BDphiDir = BoppDir.DeltaPhi(SVmomentum.Vect()); // [-pi,+pi]
    double SVP = SVmomentum.P();
    double SVM = SVmomentum.M();
    double SVMag = SVmomentum.Mag();
    double SVGP = SVP/(0.16*SVM+0.12);
    double SVtau = BoppDir.Mag()*5.28/SVGP/0.299792458;
    //if ( BDphiDir > m_Bdphidir_vtx)           return tVch;  // Stefania
    if ( Vptmean < m_Ptmean_vtx)              return tVch; 
    if ( Vptmean*vflagged  < m_Ptsum_vtx)      return tVch;
    if ( Vipsmean*vflagged < m_IPSsum_vtx)     return tVch;
    if ( Vdocamax*vflagged > m_DocaMaxsum_vtx) return tVch;
    if ( SVP < m_Psum_vtx ) return tVch;
    if ( SVM < m_Msum_vtx ) return tVch;
    if ( SVMag < m_Magsum_vtx ) return tVch;    // no cut at the moment
    
    if(!m_UseObsoleteSV) { //needed by NNtuner
      mysecvtx->setVflagged(vflagged);
      mysecvtx->setVptmean(Vptmean);
      mysecvtx->setVipsmean(Vipsmean);
      mysecvtx->setVdocamax(Vdocamax);
      mysecvtx->setVratio(Vflaglong/(vflagged? vflagged:1));
      mysecvtx->setVCharge(Vch); 
      mysecvtx->setSVtau(SVtau); 
      mysecvtx->setSVm(SVM); 
      mysecvtx->setBDphiDir(BDphiDir);
      mysecvtx->setSVP(SVP);
    }
    
    //&&&&&&&&&&&&&&&&&&&&&&&&&//    
    //plots r/w
    //&&&&&&&&&&&&&&&&&&&&&&&&&//    

    if(event.TrueTag()==-Vch){
      hr_vtx_vmaxprobf -> Fill(maxprobf,event.N_sig_sw());
      hr_vtx_vptmean   -> Fill(Vptmean,event.N_sig_sw());
      hr_vtx_vipsmean  -> Fill(Vipsmean*vflagged,event.N_sig_sw());
      hr_vtx_vdocamax  -> Fill(Vdocamax,event.N_sig_sw());
      hr_vtx_vdphidir  -> Fill(BDphiDir,event.N_sig_sw());
      hr_vtx_vptsum    -> Fill(Vptmean*vflagged,event.N_sig_sw());
      hr_vtx_vsvm      -> Fill(SVM,event.N_sig_sw());
      hr_vtx_vsvp      -> Fill(SVP,event.N_sig_sw());
      hr_vtx_vsvtheta  -> Fill(SVmomentum.Theta(),event.N_sig_sw());
      hr_vtx_vsvphi    -> Fill(SVmomentum.Phi(),event.N_sig_sw());
      hr_vtx_vsvpt     -> Fill(SVmomentum.Pt(),event.N_sig_sw());
      hr_vtx_vsvdeltar -> Fill(sqrt(SVmomentum.Phi()*SVmomentum.Phi()+SVmomentum.Theta()*SVmomentum.Theta()),event.N_sig_sw());
      hr_vtx_vsvmag    -> Fill(SVmomentum.Mag(),event.N_sig_sw());
    } else if(event.TrueTag()==Vch){
      hw_vtx_vmaxprobf -> Fill(maxprobf,event.N_sig_sw());
      hw_vtx_vptmean   -> Fill(Vptmean,event.N_sig_sw());
      hw_vtx_vipsmean  -> Fill(Vipsmean*vflagged,event.N_sig_sw());
      hw_vtx_vdocamax  -> Fill(Vdocamax,event.N_sig_sw());
      hw_vtx_vdphidir  -> Fill(BDphiDir,event.N_sig_sw());
      hw_vtx_vptsum    -> Fill(Vptmean*vflagged,event.N_sig_sw());
      hw_vtx_vsvm      -> Fill(SVM,event.N_sig_sw());
      hw_vtx_vsvp      -> Fill(SVP,event.N_sig_sw());
      hw_vtx_vsvtheta  -> Fill(SVmomentum.Theta(),event.N_sig_sw());
      hw_vtx_vsvphi    -> Fill(SVmomentum.Phi(),event.N_sig_sw());
      hw_vtx_vsvpt     -> Fill(SVmomentum.Pt(),event.N_sig_sw());
      hw_vtx_vsvdeltar -> Fill(sqrt(SVmomentum.Phi()*SVmomentum.Phi()+SVmomentum.Theta()*SVmomentum.Theta()),event.N_sig_sw());
      hw_vtx_vsvmag    -> Fill(SVmomentum.Mag(),event.N_sig_sw());
    }
    
    //plot variables of added tracks to seed
    int countadd=0;
    for(ip=Pfit.begin()+2; ip!=Pfit.end(); ip++) { //only extra part added to the seed
      if (event.TrueTag()==-Vch) {
        hr_tr_pt     -> Fill((*ip)->pt(),event.N_sig_sw());
        hr_tr_ip     -> Fill((*ip)->IP(),event.N_sig_sw());
        hr_tr_ips    -> Fill((*ip)->IPs(),event.N_sig_sw());
        hr_tr_ipsv   -> Fill((*ip)->IPSV(),event.N_sig_sw());
        hr_tr_lcs    -> Fill((*ip)->LCS(),event.N_sig_sw());
        hr_tr_docamax-> Fill((*ip)->DOCA(),event.N_sig_sw());
      }else if (event.TrueTag()==Vch){
        hw_tr_pt     -> Fill((*ip)->pt(),event.N_sig_sw());
        hw_tr_ip     -> Fill((*ip)->IP(),event.N_sig_sw());
        hw_tr_ips    -> Fill((*ip)->IPs(),event.N_sig_sw());
        hw_tr_ipsv   -> Fill((*ip)->IPSV(),event.N_sig_sw());
        hw_tr_lcs    -> Fill((*ip)->LCS(),event.N_sig_sw());
        hw_tr_docamax-> Fill((*ip)->DOCA(),event.N_sig_sw());        
      }
    }
    //&&&&&&&&&&&&&&&&&&&&&&&&&//    
    //calculate omega
    if(msgLevel(MSG::DEBUG)) debug()<<"calculate omega, use obsolete (0=no): "<<m_UseObsoleteSV<<endreq;
    double omega = m_AverageOmega;
    list<std::pair<string, Float_t> > NNinputList;
 
    if(m_UseObsoleteSV) {
      if( fabs(Vch)<0.75 ) omega = m_P0 + m_P1*fabs(Vch) ;
      if( fabs(Vch)>0.75 ) omega = m_Gt075;
      if( fabs(Vch)>0.99 ) { // tracks have all the same charge
        if(Pfit.size()==2) omega = m_wSameSign2;
        if(Pfit.size() >2) omega = m_wSameSignMoreThan2;
      }
      //      nnet.m_rnet_vtx = fabs(Vch);  ?????? Stefania
    } else {      
      //      list<std::pair<string, Float_t> > NNinputList;
      NNinputList.push_back(pair<string, Float_t>("mult",     event.multiplicity()));
      NNinputList.push_back(pair<string, Float_t>("nnkrec",   event.pileup()));
      NNinputList.push_back(pair<string, Float_t>("ptB",      log(event.signalB()->pt())));
      NNinputList.push_back(pair<string, Float_t>("vflag",    vflagged));
      NNinputList.push_back(pair<string, Float_t>("ptmean",   log(Vptmean)));
      NNinputList.push_back(pair<string, Float_t>("ipsmean",  log(Vipsmean)));
      NNinputList.push_back(pair<string, Float_t>("vcharge",  fabs(Vch)));
      NNinputList.push_back(pair<string, Float_t>("svm",      log(SVM)));
      NNinputList.push_back(pair<string, Float_t>("svp",      log(SVP)));
      NNinputList.push_back(pair<string, Float_t>("BDphiDir", BDphiDir));
      NNinputList.push_back(pair<string, Float_t>("svtau",    log(SVtau)));
      NNinputList.push_back(pair<string, Float_t>("docamax",  Vdocamax));
      
      omega = 1 - nnet.MLPvtxTMVA( NNinputList);
       
    }
    
    verbose() <<" VtxCh= "<< Vch <<" with "<< Pfit.size() <<" parts"
              <<", omega= "<< omega <<endmsg;
    
    //Calibration (w=1-pn) w' = p0 + p1(w-eta)
    omega = m_P0_Cal_vtx + m_P1_Cal_vtx * ( omega-m_Eta_Cal_vtx);
    debug()<<" Vtx pn="<<1-omega<<" w="<<omega<<endmsg;
    if( omega < 0 ||  omega > 1 )  cout<<endl<<"===> Something wrong with VTX Training "<<omega;/*return tVch;*/;
    if(nnet.m_rnet_vtx < -0.5 ) {
      cout<<endl<<"===> Something wrong with VTX nnet "<<nnet.m_rnet_vtx ;
      return tVch;
    }

    if( 1-omega < m_ProbMin_vtx ) return tVch;

    float sign=1;
#ifndef TRAINING   
    if(omega>0.5){
      omega = 1-omega;
      sign=-1;
    }
#endif
    if(property["checkDV"]==true) {
      
      std::cout<<" TaggerVertex: "<<sign*Vch<<" omega="<<omega;
      
      typedef std::list<std::pair<std::string, Float_t> > ParListType;
      for(ParListType::const_iterator p = NNinputList.begin(); p != NNinputList.end(); p++) {
        std::cout << p->second<<" ";
      }
      std::cout<<std::endl;
    }
    

    tVch->setcharge( Vch ); //to use only for Vch plot
    tVch->setDecision( (sign*Vch)>0 ? -1 : 1 );
    tVch->setRawNNetOutput( nnet.m_rnet_vtx );
    tVch->setOmega( omega );
    tVch->setType( Tagger::VtxCharge ); 
    for(ip=Pfit.begin(); ip!=Pfit.end(); ip++) {
      tVch->addToTaggerParts(*ip);
    }
    
    return tVch;
}
