// Include files 
#include "NNetTool_MLP.h"

//=============================================================================
// Parametrization for calibration of the NN output to give omega
// NN estimates the prob of being a b quark, the polynomial translate it into omega
//=============================================================================
NNetTool_MLP::NNetTool_MLP( ) {

  declareProperty( "P0_mu_scale", m_P0mu =  -1.73872); //25maig2011 
  declareProperty( "P1_mu_scale", m_P1mu =  12.1473);
  declareProperty( "P2_mu_scale", m_P2mu =  -20.966);
  declareProperty( "P3_mu_scale", m_P3mu =  10.9829);

  declareProperty( "P0_e_scale",  m_P0e  =  1.32902); // 25maig2011
  declareProperty( "P1_e_scale",  m_P1e  =  -2.32922);  
  declareProperty( "P2_e_scale",  m_P2e  =  1.17439);

  declareProperty( "P0_k_scale",  m_P0k  = -3.66218); //25maig2011
  declareProperty( "P1_k_scale",  m_P1k  = 20.5552);
  declareProperty( "P2_k_scale",  m_P2k  = -32.958);
  declareProperty( "P3_k_scale",  m_P3k  = 16.7165);

  declareProperty( "P0_ks_scale", m_P0ks =  1.22418); //dec2011_v2
  declareProperty( "P1_ks_scale", m_P1ks = -1.63297);  
  declareProperty( "P2_ks_scale", m_P2ks =  0.401361);
  
  declareProperty( "P0_ps_scale", m_P0ps =  1.22123); //2_5july2011
  declareProperty( "P1_ps_scale", m_P1ps = -1.76027);
  declareProperty( "P2_ps_scale", m_P2ps =  0.651766);

  declareProperty( "P0_vtx_scale", m_P0vtx =  0.945773 ); //2_5july2011
  declareProperty( "P1_vtx_scale", m_P1vtx = -0.93411 );

  m_rnet_mu= m_rnet_ele= m_rnet_k= m_rnet_kS= m_rnet_pS= m_rnet_vtx=0;
 
}

//=============================================================================
// Normalization to put variables in range [-1,1]
// Must be the same as the one defined in NNTuner.cxx
//=============================================================================
//=============================================================================
void NNetTool_MLP::normaliseOS(std::vector<double>& par) { 

  if(msgLevel(MSG::DEBUG)) debug()<<"par = "<<par.at(0)<<" "<<par.at(2)<<" "<<par.at(3)
         <<" "<<par.at(4)<<" "<<par.at(5)<<" "<<par.at(6)<<" "<<par.at(7)
         <<" "<<par.at(8)<<" "<<par.at(9)<<" "<<par.at(1)<<endreq;

  par.at(0) /= 90.; //multiplicity
  par.at(1) /= 25.; //ptB
  par.at(2) = std::min( sqrt(par.at(2))/15., 1.); //partP
  par.at(3) = std::min( sqrt(par.at(3))/2.5, 1.); //partPt
  double tmp=0;
  if(par.at(4)<0) tmp=-sqrt(-par.at(4)); else tmp=sqrt(par.at(4));//IPPV
  tmp /= 10.; 
  if(tmp> 1) tmp= 1; if(tmp<-1) tmp=-1;
  par.at(4)=tmp;
  par.at(8) = (par.at(8)-1)/4.;//krec
  par.at(9) = (par.at(9)-1)/3;//ncands
}
void NNetTool_MLP::normaliseSS(std::vector<double>& par) { 
  par.at(9) = (par.at(9) * 3) + 1;//undo ncands 
  par.at(9) /= 4.; //nndr
  par.at(5) /= 2.; //nndeta
  par.at(6) /= 3.; //nndphi
  par.at(7) /= 12.;//nndq
  if(par.at(7)>1.) par.at(7) = 1.;
}
void NNetTool_MLP::normalisepSS(std::vector<double>& par) { 
  par.at(5) /= 4.;   //nndr 
  par.at(6) /= 3.2;  //nnpartPhi 
  par.at(7) /= 12.;  //nndq  
  if(par.at(7)>1.) par.at(7) = 1.;
}
void NNetTool_MLP::normaliseVtx(std::vector<double>& par) { 

  debug()<<"par vtx = "<<par.at(0)<<" "<<par.at(1)<<" "<<par.at(2)
         <<" "<<par.at(3)<<" "<<par.at(4)<<" "<<par.at(5)<<" "<<par.at(6)
         <<" "<<par.at(7)<<" "<<par.at(8)<<" "<<par.at(9)<<" "<<par.at(10)<<endreq;

  par.at(0) /= 90.;  //vtag size 
  par.at(1) = (par.at(1)-1)/4.;  //krec 
  par.at(2) /= 25.;  //ptB  
  par.at(3) /= 4.;  //ncand                 
  par.at(4) /= 2.;  //ptmean  
  par.at(5) /= 30.;  //ipsmean
  par.at(6) /= 0.1;  //docamax   
  par.at(8) /= 10.;  //ptsumM 
  par.at(10)/= 6.;  //SVM       
}
/*
void NNetTool_MLP::normaliseOSmu(std::vector<double>& par) { 

  if(DBGLEVEL>2)debug()<<"par = "<<par.at(0)<<" "<<par.at(2)<<" "<<par.at(3)
                       <<" "<<par.at(4)<<" "<<par.at(5)<<" "<<par.at(6)<<" "
                       <<par.at(7)<<" "<<par.at(8)<<" "<<par.at(9)
                       <<" "<<par.at(1)<<endreq;

  par.at(0) /= 90.; //multiplicity
  par.at(1) /= 25.; //ptB
  par.at(2) = std::min( sqrt(par.at(2))/15., 1.); //partP
  par.at(3) = std::min( sqrt(par.at(3))/2.5, 1.); //partPt
  double tmp=0;
  if(par.at(4)<0) tmp=-sqrt(-par.at(4)); else tmp=sqrt(par.at(4)); //IPPV
  tmp /= 10.; 
  if(tmp> 1) tmp= 1; if(tmp<-1) tmp=-1;
  par.at(4)=tmp;
  par.at(8) = (par.at(8)-1)/4.; //krec
  par.at(9) = (par.at(9)-1)/3; //ncands
  
}
//=============================================================================
void NNetTool_MLP::normaliseOSe(std::vector<double>& par) { 

  if(DBGLEVEL>2)debug()<<"par = "<<par.at(0)<<" "<<par.at(2)<<" "<<par.at(3)
                       <<" "<<par.at(4)<<" "<<par.at(5)<<" "<<par.at(6)<<" "
                       <<par.at(7)<<" "<<par.at(8)<<" "<<par.at(9)
                       <<" "<<par.at(1)<<endreq;

  par.at(0) /= 90.; //multiplicity
  par.at(1) /= 25.; //ptB
  par.at(2) = std::min( sqrt(par.at(2))/15., 1.); //partP
  par.at(3) = std::min( sqrt(par.at(3))/2.5, 1.); //partPt
  double tmp=0;
  if(par.at(4)<0) tmp=-sqrt(-par.at(4)); else tmp=sqrt(par.at(4)); //IPPV
  tmp /= 10.; 
  if(tmp> 1) tmp= 1; if(tmp<-1) tmp=-1;
  par.at(4)=tmp;
  par.at(8) = (par.at(8)-1)/4.; //krec
  par.at(9) = (par.at(9)-1)/3; //ncands
  
}
//=============================================================================
void NNetTool_MLP::normaliseOSk(std::vector<double>& par) { 

  if(DBGLEVEL>2)debug()<<"par = "<<par.at(0)<<" "<<par.at(2)<<" "<<par.at(3)
                       <<" "<<par.at(4)<<" "<<par.at(5)<<" "<<par.at(6)<<" "
                       <<par.at(7)<<" "<<par.at(8)<<" "<<par.at(9)
                       <<" "<<par.at(1)<<endreq;

  par.at(0) /= 90.; //multiplicity
  par.at(1) /= 25.; //ptB
  par.at(2) = std::min( sqrt(par.at(2))/15., 1.); //partP
  par.at(3) = std::min( sqrt(par.at(3))/2.5, 1.); //partPt
  double tmp=0;
  if(par.at(4)<0) tmp=-sqrt(-par.at(4)); else tmp=sqrt(par.at(4)); //IPPV
  tmp /= 10.; 
  if(tmp> 1) tmp= 1; if(tmp<-1) tmp=-1;
  par.at(4)=tmp;
  par.at(8) = (par.at(8)-1)/4.; //krec
  par.at(9) = (par.at(9)-1)/3; //ncands
  
}
//=============================================================================
void NNetTool_MLP::normalisekSS(std::vector<double>& par) { 

  if(DBGLEVEL>2)debug()<<"par = "<<par.at(0)<<" "<<par.at(2)<<" "<<par.at(3)
                       <<" "<<par.at(4)<<" "<<par.at(5)<<" "<<par.at(6)<<" "
                       <<par.at(7)<<" "<<par.at(8)<<" "<<par.at(9)
                       <<" "<<par.at(1)<<endreq;

  par.at(0) /= 90.; //multiplicity
  par.at(1) /= 25.; //ptB
  par.at(2) = std::min( sqrt(par.at(2))/15., 1.); //partP
  par.at(3) = std::min( sqrt(par.at(3))/2.5, 1.); //partPt
  double tmp=0;
  if(par.at(4)<0) tmp=-sqrt(-par.at(4)); else tmp=sqrt(par.at(4)); //IPPV
  tmp /= 10.; 
  if(tmp> 1) tmp= 1; if(tmp<-1) tmp=-1;
  par.at(4)=tmp;
  par.at(5) /= 2.; //nndeta
  par.at(6) /= 3.; //nndphi
  par.at(7) /= 12.; //nndq
  if(par.at(7)>1.) par.at(7) = 1.;
  par.at(8) = (par.at(8)-1)/4.; //krec

}
//=============================================================================
void NNetTool_MLP::normalisepSS(std::vector<double>& par) { 
  if(DBGLEVEL>2)debug()<<"par = "<<par.at(0)<<" "<<par.at(2)<<" "<<par.at(3)
                       <<" "<<par.at(4)<<" "<<par.at(5)<<" "<<par.at(6)<<" "
                       <<par.at(7)<<" "<<par.at(8)<<" "<<par.at(9)
                       <<" "<<par.at(1)<<endreq;

  par.at(0) /= 90.; //multiplicity
  par.at(1) /= 25.; //ptB
  par.at(2) = std::min( sqrt(par.at(2))/15., 1.); //partP
  par.at(3) = std::min( sqrt(par.at(3))/2.5, 1.); //partPt
  double tmp=0;
  if(par.at(4)<0) tmp=-sqrt(-par.at(4)); else tmp=sqrt(par.at(4)); //IPPV
  tmp /= 10.; 
  if(tmp> 1) tmp= 1; if(tmp<-1) tmp=-1;
  par.at(4)=tmp;
  par.at(5) /= 4.; //nndr
  par.at(7) /= 12.; //nndq
  if(par.at(7)>1.) par.at(7) = 1.;
  par.at(8) = (par.at(8)-1)/4.; //krec

}
//=============================================================================
void NNetTool_MLP::normaliseVtx(std::vector<double>& par) { 

  if(DBGLEVEL>2)debug()<<"par = "<<par.at(0)<<" "<<par.at(1)<<" "<<par.at(2)<<" "<<par.at(3)
                       <<" "<<par.at(4)<<" "<<par.at(5)<<" "<<par.at(6)
                       <<" "<<par.at(7)<<" "<<par.at(8)<<" "<<par.at(9)
                       <<" "<<par.at(10)<<endreq;

  par.at(0) /= 90.; //vtag size
  par.at(1) = (par.at(1)-1)/4.; //krec
  par.at(2) /= 25.; //ptB
  par.at(3) /= 4.;  //ncand
  par.at(4) /= 2.;  //ptmean
  par.at(5) /= 30.; //ipsmean
  par.at(6) /= 0.1; //docamax
  par.at(10)/= 6.;  //SVM
  par.at(8)/= 10.;  //ptsumM

}
*/
//=============================================================================
// Neural Nets, given the input variables (par), they are normalized
// and NN return a omega estimation
// 1. First the input variables are readed and normalized in the same way done in NNTuner.cxx
// 2. The normalized variables are introduced in the same order as they were trained in NNTuner.cxx
// 3. NN estimates a prob of being a b quark, which is translated into a omega via a polynomials (m_P0XX, m_P1XX)
//=============================================================================
double NNetTool_MLP::MLPm(std::vector<double>& par) { 

  //normaliseOSmu( par );
  normaliseOS( par );
  NNmuon net;
  double rnet = net.value(0, par.at(2),par.at(3),par.at(8),
                          par.at(1),par.at(4)); 
  
  double pn = 1.0-pol(rnet, m_P0mu, m_P1mu, m_P2mu, m_P3mu);

  print(par, rnet, pn);

  m_rnet_mu = rnet;

  return pn;
}; 

//=============================================================================
double NNetTool_MLP::MLPe(std::vector<double>& par) { 

  //normaliseOSe( par );
  normaliseOS( par );
  NNele net;
  double rnet = net.value(0, par.at(2),par.at(3),par.at(8),
                          par.at(1),par.at(4)); 
  
  double pn = 1.0-pol(rnet, m_P0e, m_P1e, m_P2e);

  print(par, rnet, pn);

  m_rnet_ele = rnet;

  return pn;
}; 

//=============================================================================
double NNetTool_MLP::MLPk(std::vector<double>& par ) { 

  //normaliseOSk( par );
  normaliseOS( par );
  NNkaon net;
  double rnet = net.value(0, par.at(0),par.at(2),par.at(3),par.at(8),
                          par.at(1),par.at(4));
  
  double pn = 1.0-pol(rnet, m_P0k, m_P1k, m_P2k, m_P3k);

  print(par, rnet, pn);

  m_rnet_k = rnet;

  return pn;
}; 

//=============================================================================
double NNetTool_MLP::MLPkS(std::vector<double>& par) { 

  //normalisekSS( par );
  normaliseOS( par );
  normaliseSS( par );
  NNkaonS net;
  double rnet = net.value(0, par.at(0),par.at(3),par.at(8),par.at(5),
			  par.at(6),par.at(1),par.at(7));

  double pn = 1.0-pol(rnet, m_P0ks, m_P1ks, m_P2ks);

  print(par, rnet, pn);

  m_rnet_kS = rnet;

  return pn;
}; 

//=============================================================================
double NNetTool_MLP::MLPpS(std::vector<double>& par) { 

  normaliseOS( par );
  normalisepSS( par );
  NNpionS net;
  double rnet = net.value(0,par.at(0),par.at(3),par.at(4),
                          par.at(8),par.at(1),par.at(7),par.at(5));

  double pn = 1.0-pol(rnet, m_P0ps, m_P1ps, m_P2ps);

  print(par, rnet, pn);

  m_rnet_pS = rnet;

  return pn;
};

//=============================================================================
double NNetTool_MLP::MLPvtx(std::vector<double>& par) { 

  normaliseVtx( par );
  NNvtx net;
  double rnet = net.value(0,par.at(0),par.at(1),par.at(2),par.at(3),par.at(4),
                          par.at(5),par.at(9),par.at(10));

  double pn = 1.0-pol(rnet, m_P0vtx, m_P1vtx);

  print(par, rnet, pn);

  m_rnet_vtx = rnet;

  return pn;
};

//=============================================================================
