#ifndef PerformanceMonitor_TAGGING_H 
#define PerformanceMonitor_TAGGING_H 1

// Include files
#include <TROOT.h>
#include <TH2.h>
#include <TF1.h>
#include <TCanvas.h>
#include <TLegend.h>

#include <iostream>
#include <sstream>
#include <iomanip>
#include <stdio.h>
#include <stdlib.h>
#include "FlavourTag.h"
#include "Event.h"
#include "GenericTool.h"

/** @class PerformanceMonitor
 *
 *  General Tagging calculations and printouts class
 *
 *  This class can be used to calculate any tagging efficiency, 
 *  Print out can be done with the printStats() method outside the event loop
 *
 *  @author Marco Musy
 *  @date   2010-06-29
 */
class PerformanceMonitor : public GenericTool {
public:
  /// Standard constructor
  //fname is used in the asymmetry plots as root for the title and names
  PerformanceMonitor();
  ~PerformanceMonitor( ){}

  void increment( Event&, FlavourTag*, double );

  //print statistics assuming the method above has been used
  void printStats();///< print statistics and performances
  int N();///< retrieve the total number of selected events

private:
  bool isBs;
 
  double nsele,nseleOS;
  double nrt[20],nwt[20],nrtag[20],nwtag[20],ntag[6][6],sumD2[20];
  float nlm,nllm,nle,nlle,nlk,nlkS,nlpiS,nllk,nllkS,nllpiS,nidm,nide,nidk,nidkS,nidpiS; 
  float nghost_m,nghost_e,nghost_k, nghost_kS, nghost_piS, ghrate_k,gherror_k; 
  float ghrate_kS,gherror_kS,ghrate_piS,gherror_piS; 
  float ghrate_m,gherror_m,ghrate_e,gherror_e;
  /*int*/double nrTisTis, nrTosTos, nrTisTos, nrTosTis, nrTob; 

  /*int*/double ntruemu, ntrueel, ntruekO, ntruekS, ntruepiS;
  /*int*/double ntruemutag, ntrueeltag, ntruekOtag, ntruekStag,ntruepiStag;
  /*int*/double ndirect_mu,ndirect_el,nbc_mu,nbc_el,nbc_kO,nbc_kS,nbc_piS;
  float den_m, den_e, den_k, den_kS,den_piS;

  float dilution, dilutionOS;

  //int nB511, nB521, nB531, nBother, nBanti;
  //int ntrigL0, ntrigHlt1, ntrigHlt2, ntisL0, ntosL0, ntistosL0;
  double nB511, nB521, nB531, nBother, nBanti;
  double ntrigL0, ntrigHlt1, ntrigHlt2, ntisL0, ntosL0, ntistosL0;

  /*int*/double npartsinSV, nrSV, ninSVtrue, nkrec;

};
#endif 
