#ifndef SVertexOneSeedTool_H
#define SVertexOneSeedTool_H 1

#include "Tagger.h"
#include "Event.h"
#include "GenericTool.h"

#include "TH1F.h"

using namespace std;

/** @class SVertexOneSeedTool
 *
 *  Tool to build an inclusive secondary vertex starting
 *  with a 2-track seed that has been built in DaVinci.
 *  The algorithm search for tracks to be added to the seed,
 *  and calculates a likelihood that can be used in a nnet combination
 *
 *  @author Marco Musy, Marc Grabalosa Gandara
 *  @date   2010-07-15
 */
class SVertexOneSeedTool  : public GenericTool {

 public:

  SVertexOneSeedTool( );
  ~SVertexOneSeedTool( ){} ///< Destructor

  Vertices buildVertex( Particles& parts, Event* evt );

 private:

  //static SVertexOneSeedTool m_svtool;    

  double m_maxprobf, m_ptmin, m_ipsmin, m_dphimin, m_dphimax;

  double combine(double p1, double p2, double p3,
                 double p4, double p5, double p6, double p7);

  TH1F* hvprobf2;
  TH1F* hvprobf0;
  TH1F* hvChi0 ;
  TH1F* hvChi2 ;
  TH1F* hz0    ;
  TH1F* hz2    ;
  TH1F* hvpmin_2 ;
  TH1F* hvpmin_0 ;
  TH1F* hvipmin2 ;
  TH1F* hvipmin0 ;
  TH1F* hvipmax2 ;
  TH1F* hvipmax0 ;
  TH1F* hvipsmin2 ;
  TH1F* hvipsmin0 ;
  TH1F* hvdphi2;
  TH1F* hvdphi0 ;
  TH1F* hvmass2 ;
  TH1F* hvmass0 ;
  TH1F* hvmaxprobf2 ;
  TH1F* hvmaxprobf0 ;
  TH1F* hvreson2 ;
  TH1F* hvreson0 ;
  TH1F* hvsvpointtheta2 ;
  TH1F* hvsvpointtheta0 ;

  TH1F *hr_seed_ipsmin, *hr_seed_ipmax, *hr_seed_ptmin, *hr_seed_chi2, *hr_seed_dphi;
  TH1F *hr_seed_rdist, *hr_seed_probf, *hr_seed_svpointtheta ;
  TH1F *hw_seed_ipsmin, *hw_seed_ipmax, *hw_seed_ptmin, *hw_seed_chi2, *hw_seed_dphi;
  TH1F *hw_seed_rdist, *hw_seed_probf, *hw_seed_svpointtheta ;

  TH1F *htr_pt1, *htr_pt0 ;
  TH1F *htr_ip1, *htr_ip0 ;
  TH1F *htr_ips1, *htr_ips0 ;
  TH1F *htr_ipsv1, *htr_ipsv0 ;
  TH1F *htr_doca1, *htr_doca0 ;
  TH1F *htr_lcs1, *htr_lcs0 ;


};
#endif 
