//#define MCTRUTH          // requires bkg_cat == 0
//#define BU2JPSIK         // to write the output file & apply mass&time cuts
//#define BD2DPI           // to write the output file & apply mass&time cuts
//#define BsMCTD2JPSIKSTAR // to write the output file & apply mass&time cuts
//#define DSTARMUNU        // to write the output file & apply mass&time cuts  
                           // only for DSTARMUNU : applies the correction to the signal momentum according to the K-factor
#define OPTIMIZE         // --- applies mass (and time) cuts
//#define REWEIGHT

#include "tag.h"

int main () {

#include "tag_histo.h" //define here histograms to fill.

  TApplication appKey("appKey",0,0);

  TaggerMuonTool            mutool ; //tagging tools to be used
  TaggerElectronTool        eletool;
  TaggerKaonOppositeTool    ktool  ;
  TaggerKaonSameTool        kStool ;
  TaggerPionSameTool        pStool ;
  TaggerVertexChargeTool    vtxtool;
  TaggerFragmentationOppositeTool    kfragtool;
  TaggerDZeroTool                    dztool;

  CombineTaggersPID         combPIDtool ;
  CombineTaggersNN          combNNtool;
  CombineTaggersProbability combProbtool;

  EventViewer viewer;
  NNTuner nnetune(NNetTrain,usesWeights);
  PerformanceMonitor perfmon, perfmonNN;

#ifdef REWEIGHT
  TRandom3 *m_rnd=new TRandom3();
  m_rnd->SetSeed(4378);
  TFile *file2 = new TFile("Weight.root"); //Rescaling
  TH1F * W_mul = (TH1F*)file2->Get("h6");  //reweighting
#endif

#ifndef OPTIMIZE  // don't want to write ascii files while optimizing
#ifdef BU2JPSIK
  //for B2JpsiK  
  std::ofstream Bufile("output_Bu2JpsiK_Data2012.ascii");
#endif
#ifdef BD2JPSIKSTAR
  //for B2JpsiK*
  std::ofstream Bdfile("output_BdData2011.txt");
  std::ofstream BdOSfile("output_BdOSData2011.txt");
  std::ofstream BdMuonfile("output_BdMuonData2011.txt");
  std::ofstream BdElectronfile("output_BdElectronData2011.txt");
  std::ofstream BdKaonfile("output_BdKaonData2011.txt");
  std::ofstream BdPionfile("output_BdPionData2011.txt");
  std::ofstream BdVtxfile("output_BdVtxData2011.txt");
#endif
#ifdef DSTARMUNU
  //for B2DstarMuNu  
  std::ofstream dstarfile("output_DstarMunu_Data2011.ascii");
#endif
#ifdef BD2DPI
  //for Bd2Dpi
  std::ofstream dpifile("output_Bd2Dpi_Data2011.ascii");
#endif
#endif

  //FILE LOOP ////////////////////////////////////////////////////////////////
  for( std::vector<TString>::iterator ifile=filelist.begin(); ifile!=filelist.end(); ++ifile ) {
    if(Nmax!=-1 && perfmon.N()>=Nmax) break;
    NtpTag tree(*ifile); //interface between ntuple and event class
    int nentries = tree.size();
        
    //EVENT LOOP >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
    for( int i=0; i<nentries; ++i ) {

      Event event; 
      if(! tree.execute(i, event) ) continue; //load event
      if(Nmax!=-1 && perfmon.N()>=Nmax) break;
      PrintAdvance( i, nentries, *ifile );

      // >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      // Select only part of the sample
      // <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
      //if (i%2!=0) continue; //even
      //if (i%2==0) continue; //odd
      //Define weight
      double weight;
      if (usesWeights == 1) weight = event.Nsigsw();
      else weight = 1.;
      
      

#ifdef DSTARMUNU
      //======= Correction for Kinematics in semileptonic decays ===============
      if (UseModBtime>0) {
        //for DstarMuNu channel
        Particle* Dstar = event.Dstar();
        Particle* D0    = event.D0();
        Particle* Mu    = event.Mu();
        TLorentzVector DstarMu = Dstar->momentum() + Mu->momentum();
        double ChiP = DstarMu.P();
        double ChiM = DstarMu.M();
        double newChiP = 0;
        //Marc
        if (UseModBtime==1) newChiP = ChiP/(0.283+ChiM*0.1283);
        //Stefania
        else if (UseModBtime==2){
          double kFactor = (0.9947-0.197*ChiM+0.0375*pow(ChiM,2));
          double BPcor = ChiP/kFactor;
          double BinvPcor = 1./BPcor;
          double kBias = 0.067416 - 0.000564778*BPcor/1000 + 9.55013e-07*pow(BPcor/1000.,2);
          BinvPcor =  (1 - kBias)*BinvPcor;
          newChiP = 1./BinvPcor;
        }
        //Sara (15 - 17 june 2010)
        else if (UseModBtime==3){
          double kFactor= (0.9654-0.1685*ChiM+0.0327*pow(ChiM,2));
          double BPcor = ChiP/kFactor;
          double BinvPcor = 1./BPcor;
          double kBias = 0.07611 - 0.0008899*BPcor + 4.2e-06*pow(BPcor,2)- 1.007e-08*pow(BPcor,3)+7.6e-12*pow(BPcor,4);
          BinvPcor =  (1 - kBias)*BinvPcor;
          newChiP = 1./BinvPcor;
        }
        // Sara (studies on Strip13b data)
        else if(UseModBtime==4){
          Double_t  DmuP = DstarMu.P();
          Double_t Bd_MM = DstarMu.M();
          double kFactor=-1;
          if(ChiP>0 && ChiP<90) kFactor = 0.9658 -0.1844*ChiM+0.03592*pow(ChiM,2);
          else if(ChiP>90 && ChiP<200) kFactor = 1.067 -0.2096*ChiM+0.03706*pow(ChiM,2);
          else if(ChiP>200 && ChiP<300) kFactor = 1.142 -0.2149*ChiM+0.03491*pow(ChiM,2);
          else if(ChiP>300) kFactor = 1.112 -0.1837*ChiM+0.0311*pow(ChiM,2);
          newChiP = ChiP/kFactor;          
        }
        double MassB  = 5.279; 
        
        TVector3 BFlightvec = event.BSVertex()-event.RecVertex();    
        double Bdir = (BFlightvec).Mag();
        TVector3 newBP = (newChiP/Bdir) * BFlightvec ;
        
        double newBE = sqrt(pow(newChiP,2)+pow(MassB,2));  
        double NewBtime = Bdir*MassB/newChiP/0.299792458;
        
        TLorentzVector p( newBP.x(),newBP.y(),newBP.z(), newBE );
        event.signalB()->setMomentum( p );        
        event.setBtau(NewBtime);
        event.setBmass(MassB);
        //event.setBProperTime(NewBtime);
        //cout<<"======corrected Bp="<<newChiP<<" Bpt="<<p.Pt()<<" BE="<<p.E()<<" BM="<<p.M()<<" Btime="<<NewBtime<<endl;        
      }
      /*      
      cout<<" After Bp="<<event.signalB()->p()<<" Bpt="<<event.signalB()->pt()<<" BE="<<event.signalB()->momentum().E()
          <<" BM="<<event.signalB()->M()
          <<" Btime="<<event.Btau()<<" BProperTime="<<event.BProperTime()<<endl;      
      */
//======= End of Correction for Kinematics in semileptonic decays ===============
#endif

#ifdef OPTIMIZE
#ifdef BU2JPSIK
      float bmass = event.signalB()->M();
      if(usesWeights == 0 && abs(bmass - 5.27917)> 0.025 )   continue;      
      if(usesWeights == 0 && event.Btau() < 0.3 )            continue;
#endif
#ifdef BD2DPI
      float bmass = event.signalB()->M();
      if( usesWeights == 0 && abs(bmass - 5.27917 )> 0.025 )   continue;
      float Dmass = event.Dminus()->M();
      if( usesWeights == 0 && abs(Dmass - 1.86960)>0.018)    continue;
      if( usesWeights == 0 && event.Btau() < 0.25) continue;

      //if(event.Btau() > 2.2 ) continue;
      /*
      if(event.Btau() > 2.2 && event.Btau() < 4.08 ) continue;
      if(event.Btau() > 7.98) continue;
      */
      
#endif
#ifdef DSTARMUNU
      Particle* myDstar = event.Dstar();
      Particle* myD0    = event.D0();
      
      if( (myDstar->momentum().M() - myD0->momentum().M()) >  0.1465 || 
          (myDstar->momentum().M() - myD0->momentum().M()) < 0.1445 ) continue;      
      if( fabs(myD0->momentum().M() - 1.86483 )>0.020) continue;
      if( event.Btau() < 0.25) continue;

      //if(event.Btau() > 2.2 ) continue;
      /*
      if(event.Btau() > 2.2 && event.Btau() < 4.08 ) continue;
      if(event.Btau() > 7.98) continue;
      */
      
#endif
#endif

      //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      // CUTS TO SELECT EVENT
      //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      if(!isinTextFile(long(event.runNr()),long(event.eventNr()))) continue;  //read file of selected events
      //      PrintAdvance( i, nentries, *ifile );
      if( requireL0   && event.L0()  ==0 )   continue;
      if( requireHlt1 && event.Hlt1()==0 )   continue;
      if( requireHlt2 && event.Hlt2()==0 )   continue;
      if( requireTis  && event.L0TisTos()&2) continue;
#ifdef MCTRUTH
      if( event.backgroundCat() != 0) continue;  //for MC
#endif
      //if( event.pileup() > 1 )   continue;
      //if( event.multiplicity()<30) continue;
      //if( event.multiplicity()<60) continue;
      //if( event.TrueTag()== -1 ) continue;
      //if( event.signalB()->pt()<5000 ) continue;
      //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

      Particles parts = event.particles();//candidate particles
      Particle* BSpart = event.signalB();
      Particle* BOpart = event.oppositeB();

      //if (BSpart->ID()==-531) continue; //only B+/B-
      //if (BOpart->ID()<0) continue; //only B+/B- Bs

#ifdef REWEIGHT
      double x_value;
      x_value = event.multiplicity();
      double xmax= W_mul->GetXaxis()->GetXmax();
      double xmin= W_mul->GetXaxis()->GetXmin();
      int nbin = W_mul->GetNbinsX();
      double xbin = (xmax - xmin)/double(nbin);
      int ibin = int (float(x_value-xmin)/xbin) + 1;
      double weight = W_mul->GetBinContent(ibin);
      double rndm = m_rnd->Rndm(0);
      if (rndm<weight*1) continue;
#endif

      //Individual tagger's response:
      Tagger* tmuon = mutool .tag(event);
      Tagger* tele  = eletool.tag(event);
      Tagger* tkaon = ktool  .tag(event);
      Tagger* tsamek = kStool.tag(event);      
      Tagger* tsamepi = pStool.tag(event);
      Tagger* tvtx  = vtxtool.tag(event);

      Tagger *tsame;      
      if (event.isBs()) tsame=tsamek;
      else tsame=tsamepi;
      

      //extra taggers, put to 0 in the tagger
      Tagger* tdz    = dztool .tag(event);
      Tagger* tkfrag = kfragtool .tag(event);

      //Combination of tagger's decisions:
      Taggers taggers, OStaggers; 
      taggers.push_back(tmuon); 
      taggers.push_back(tele); 
      taggers.push_back(tkaon); 
      taggers.push_back(tsamek); 
      taggers.push_back(tsamepi); 
      taggers.push_back(tvtx); 
      taggers.push_back(tkfrag); 
      taggers.push_back(tdz); 
      OStaggers.push_back(tmuon); 
      OStaggers.push_back(tele); 
      OStaggers.push_back(tkaon); 
      OStaggers.push_back(tvtx); 
      //Tagger combination technique
      int signalType =0;
      if (event.isBs()) signalType=2;
      else signalType=1;

      FlavourTag* theTag = combProbtool.combineTaggers( taggers , signalType);
      //FlavourTag* theTag = combNNtool.combineTaggers( taggers );
      //FlavourTag* theTag = combPIDtool.combineTaggers( taggers );
      debug()<<"omega: "<<theTag->omega()<<endreq;      
      debug()<<"omegaOS: "<<theTag->omegaOS()<<endreq;      

      int TrueTag = event.TrueTag();
#ifdef OPTIMIZE
      //if (event.Btau()> 4.08 && event.Btau()  < 7.98) TrueTag = -1*TrueTag; //split sign for high propertime
#endif
      theTag->setTrueTag(TrueTag);

      nnetune.Fill(event, theTag);//fill info for retuning, set by NNetTrain
      perfmon.increment(event, theTag, weight);//update counters for this event
 

      //************************************************************************
      //************************************************************************
      //  FROM THIS POINT JUST HISTOGRAM FILLING AND ASCI GENERATION!          *
      //************************************************************************
      //************************************************************************

      //shortcuts
      double     Btime = event.BProperTime(); //in ps
      bool        isBs = event.isBs();
      bool       isTis = event.L0TisTos()&2;
      bool       isTos = event.L0TisTos()&1;
      int     category = theTag->category();
      int  tagdecision = theTag->decision();
      double     omega = theTag->omega();
      int  tagdecisionOS = theTag->decisionOS();
      double     omegaOS = theTag->omegaOS();
      int       nparts = parts.size();
      int         krec = event.pileup();
      //get the particle used by tagger:
      Particle*  imuon = tmuon->decision()? tmuon->taggerparts().at(0) : NULL;
      Particle*   iele = tele ->decision()? tele ->taggerparts().at(0) : NULL;
      Particle*  ikaon = tkaon->decision()? tkaon->taggerparts().at(0) : NULL;
      Particle*  isamek = tsamek->decision()? tsamek->taggerparts().at(0) : NULL;
      Particle*  isamepi = tsamepi->decision()? tsamepi->taggerparts().at(0) : NULL;

      Particle* isame;
      if (event.isBs()) isame=isamek;
      else isame=isamepi;
      

      //store first events to visualize
      if(tagdecision && i<10) viewer.storeClone( event );


      //************************************************************************
      //  ASCI CREATION                                                        *
      //************************************************************************

#ifndef OPTIMIZE
#ifdef BU2JPSIK      
      Int_t sign=TrueTag;
      sign = BSpart->ID()/abs(BSpart->ID());
      //write ascii files for B2JpsiK
      Bufile<<BSpart->M()*1000<<" "<<Btime<<" "<<0.02<<" "<<
        -(tagdecision*sign)<<" "<<omega<<" "<<
        -(tagdecisionOS*sign)<<" "<<omegaOS<<" "<<
        -tmuon->decision()*sign<<" "<<tmuon->omega()<<" "<<
        -tele->decision()*sign<<" "<<tele->omega()<<" "<<
        -tkaon->decision()*sign<<" "<<tkaon->omega()<<" "<<
        -tsame->decision()*sign<<" "<<tsame->omega()<<" "<<
        -tvtx->decision()*sign<<" "<<tvtx->omega()<<" "<<
        sign<<" "<<event.runNr() <<" "<<event.pileup()<< endl;
      //  sign<<" "<<int(my_nPV)<<" "<<my_nTracks<<" "<<int(my_Trig)<<" "<<endl;
#endif      
#ifdef BD2JPSIKSTAR
      // write ascii files for BdJpsiK*
      double Bid=BSpart->ID();
      double mixState = 0, mixStateOS = 0; 
      double tmuonmixState = 0, telemixState = 0, tkaonmixState = 0, tsamemixState = 0, tvtxmixState = 0;
      if(tagdecision!=0)       mixState      = tagdecision*Bid/fabs(tagdecision*Bid);
      if(tagdecisionOS!=0)     mixStateOS    = tagdecisionOS*Bid/fabs(tagdecisionOS*Bid);
      if(tmuon->decision()!=0) tmuonmixState = tmuon->decision()*Bid/fabs(tmuon->decision()*Bid);
      if(tele->decision()!=0)  telemixState  = tele->decision()*Bid/fabs(tele->decision()*Bid);
      if(tkaon->decision()!=0) tkaonmixState = tkaon->decision()*Bid/fabs(tkaon->decision()*Bid);
      if(tsame->decision()!=0) tsamemixState = tsame->decision()*Bid/fabs(tsame->decision()*Bid);
      if(tvtx->decision()!=0)  tvtxmixState  = tvtx->decision()*Bid/fabs(tvtx->decision()*Bid);
      /*      
      if((tagdecision== +1 && Bid == 511) || (tagdecision== -1 && Bid == -511 ))                  mixState = 1;
      if((tagdecision== +1 && Bid == -511) || (tagdecision== -1 && Bid == 511 ))                  mixState = -1;
      if((tagdecisionOS== +1 && Bid == 511) || (tagdecisionOS== -1 && Bid == -511 ))              mixStateOS = 1;
      if((tagdecisionOS== +1 && Bid == -511) || (tagdecisionOS== -1 && Bid == 511 ))              mixStateOS = -1;
      if((tmuon->decision()== +1 && Bid == 511) || (tmuon->decision()== -1 && Bid == -511 ))      tmuonmixState = 1;
      if((tmuon->decision()== +1 && Bid == -511) || (tmuon->decision()== -1 && Bid == 511 ))      tmuonmixState = -1;
      if((tele->decision()== +1 && Bid == 511) || (tele->decision()== -1 && Bid == -511 ))        telemixState = 1;
      if((tele->decision()== +1 && Bid == -511) || (tele->decision()== -1 && Bid == 511 ))        telemixState = -1;
      if((tkaon->decision()== +1 && Bid == 511) || (tkaon->decision()== -1 && Bid == -511 ))      tkaonmixState = 1;
      if((tkaon->decision()== +1 && Bid == -511) || (tkaon->decision()== -1 && Bid == 511 ))      tkaonmixState = -1;
      if((tsame->decision()== +1 && Bid == 511) || (tsame->decision()== -1 && Bid == -511 ))      tsamemixState = 1;
      if((tsame->decision()== +1 && Bid == -511) || (tsame->decision()== -1 && Bid == 511 ))      tsamemixState = -1;
      if((tvtx->decision()== +1 && Bid == 511) || (tvtx->decision()== -1 && Bid == -511 ))        tvtxmixState = 1;
      if((tvtx->decision()== +1 && Bid == -511) || (tvtx->decision()== -1 && Bid == 511 ))        tvtxmixState = -1;
      */
      double     Btestmass = BSpart->M()*1000.; //in ps
      Bdfile<<Btestmass<<" "<<Btime<<" "<< tagdecision<<" "<<mixState <<" "<<category <<" "<<omega <<endl;
      BdOSfile<<Btestmass<<" "<<Btime<<" "<< tagdecisionOS<<" "<<mixStateOS <<endl;
      BdMuonfile<<Btestmass<<" "<<Btime<<" "<<tmuon->decision()<<" "<<tmuonmixState <<" "<<endl;
      BdElectronfile<<Btestmass<<" "<<Btime<<" "<<tele->decision()<<" "<<telemixState <<" "<< endl;
      BdKaonfile<<Btestmass<<" "<<Btime<<" "<<tkaon->decision()<<" "<<tkaonmixState <<" "<< endl;
      BdPionfile<<Btestmass<<" "<<Btime<<" "<<tsame->decision()<<" "<<tsamemixState <<" "<< endl;
      BdVtxfile<<Btestmass<<" "<<Btime<<" "<<tvtx->decision()<<" "<<tvtxmixState <<" "<< endl;
#endif
#ifdef BD2DPI
      // write ascii files for BdJpsiK*
      double Bid=BSpart->ID();
      double mix = 0;
      int decision = theTag->decisionOS();
      double eta = theTag->omegaOS();
      /* 
         decision = theTag->decision();
         eta = theTag->omega();
         decision = tmuon->decision();
         eta = tmuon->omega();
         decision = tele->decision();
         eta = tele->omega();
         decision = tkaon->decision();
         eta = tkaon->omega();
         decision = tvtx->decision();
         eta = tvtx->omega();
         decision = tsame->decision();
         eta = tsame->omega();
      */
      if(decision!=0) mix = decision*Bid/fabs(decision*Bid);      
      dpifile<<Btime<<"  "<<event.signalB()->M()*1000<<"    "<<mix<<" "<<eta << endl;    
#endif
#ifdef DSTARMUNU
        Particle* Dstar = event.Dstar();
        Particle* D0    = event.D0();
        Particle* Mu    = event.Mu();
        TLorentzVector DstarMu = Dstar->momentum() + Mu->momentum();
        //write ascii file for B2DstarMuNu
        double DeltaM  = Dstar->M()-D0->M();
        //Float_t mix = -(tvtx->decision()*Mu->ID())/13;         //one tagger
        Float_t mix = -(theTag->decision()*Mu->ID())/13;         // SS+OS combined
        dstarfile<<Btime<<"  "<<DeltaM*1000<<"    "<<mix<<" "<<(D0->M())*1000 << endl;    
#endif

#endif // end #ifndef OPTIMIZE
    
      //plots omega
        if(theTag->decision()) homega->Fill(theTag->omega(),weight);
      //if(tvtx->decision())
      //homega->Fill(tvtx->omega(),weight);
      //homega->Fill(tele->omega(),weight);
      //homega->Fill(tmuon->omega(),weight);
      //homega->Fill(tkaon->omega(),weight);
      //if(tsame->decision()) homega->Fill(tsame->omega(),weight);

      Int_t FlTag = BOpart->ID()/511;
      if((theTag->decision()*FlTag)>0) homega2->Fill(theTag->omega(),weight);
      //if((tvtx->decision()*FlTag)>0)	
      //homega2->Fill(tvtx->omega(),weight);
      //homega2->Fill(tele->omega(),weight);
      //homega2->Fill(tmuon->omega(),weight);
      //homega2->Fill(tkaon->omega(),weight);
      //if((tsame->decision()*FlTag)>0) homega2->Fill(tsame->omega(),weight);

      //****************************************
      //event histos
      //****************************************
      h1->Fill(BSpart->M(),weight);//do not remove this.
      h2->Fill(krec,weight);
      h3->Fill(BOpart->pt(),weight);
      h4->Fill(event.multiplicity(),weight);

      //**********************//
      //******SV studies******//
      //**********************//
      Vertices secvtx = event.getSecondaryVertices();
      Double_t   sz=0;
      bool containskaon=false; 
      if( !secvtx.empty()) {
        int nsecpart = secvtx.at(0)->outgoingParticles().size();
        sz = secvtx.at(0)->position().z();
        TVector3 BoppDir = secvtx.at(0)->position();
        Particles secparts = secvtx.at(0)->outgoingParticles();
        TVector3 BoppMom;
        TLorentzVector SVmomentum;
        //look for kaons or leptons in SV
        int kaoncounter = 0;
        int leptoncounter = 0;
        int kaonleptoncounter = 0;
        int tagpartcounter = 0;
        int frombcounter = 0;
        for (int ip=0; ip<nsecpart; ip++) {
          BoppMom += secparts.at(ip)->momentum().Vect();
          SVmomentum += secparts.at(ip)->momentum();
          if (secparts.at(ip)->absID()==321) { containskaon = true; kaoncounter+=1; }
          if (secparts.at(ip)->absID()==13 || secparts.at(ip)->absID()==11) leptoncounter+=1;
          if (secparts.at(ip)->absID()==321 || secparts.at(ip)->absID()==13 || secparts.at(ip)->absID()==11) kaonleptoncounter+=1;
          tagpartcounter+=1;
          if (secparts.at(ip)->fromB()==1) frombcounter+=1;
        }
        hsv100->Fill(kaoncounter);
        hsv101->Fill(leptoncounter);
        hsv102->Fill(kaonleptoncounter);
        hsv103->Fill(tagpartcounter);
        hsv1031->Fill(tagpartcounter-2);
        hsv104->Fill(double(frombcounter)/double(tagpartcounter));
        //pointing variables
        double BDphiDir = BoppDir.DeltaPhi(SVmomentum.Vect());
        double BDTheDir = BoppDir.Theta()-SVmomentum.Vect().Theta();	
        hsv200->Fill(BDphiDir);
        hsv201->Fill(BDTheDir);
        //reconstruction of BO tau
        double SVP = SVmomentum.P();
        double SVM = SVmomentum.M();
        double SVGP = SVP/(0.16*SVM+0.12);
        double SVtau = BoppDir.Mag()*5.28/SVGP/0.299792458;
        hsv300->Fill(SVtau);
        //Check MC SV
        double BMCP = BOpart->p();
        double BMCM = BOpart->M();
        TVector3 truePV = event.MCVertex();
        double BMCsz = BOpart->endVertexPosition().z()-event.MCVertex().z();
        double BMCtau = (BOpart->endVertexPosition()-truePV).Mag()*5.28/BMCP/0.299792458;
        double BMCDphiDir = BoppDir.DeltaPhi(BOpart->momentum().Vect());
        double BMCDTheDir = BoppDir.Theta()-BOpart->momentum().Vect().Theta();
        double BMCDphiVec = BoppMom.DeltaPhi(BOpart->momentum().Vect());
        double distfromtrue = ((BOpart->endVertexPosition() - truePV)- secvtx.at(0)->position() ).Mag();
        hsv9->Fill(distfromtrue);
        //resolution
        hsv301->Fill(BMCP-SVGP);
        hsv302->Fill(BMCtau-SVtau);
        hsv303->Fill(BMCsz-sz);
        if(distfromtrue<1.0) { //consider a good SV if SV closer to BO in less than 1 mm
          //pointing
          hsv200_g->Fill(BMCDphiDir);
          hsv201_g->Fill(BMCDTheDir);
          hsv202_g->Fill(BMCDphiVec);
          //for tau
          hsv301_g->Fill(BMCP-SVGP);
          hsv302_g->Fill(BMCtau-SVtau);
          hsv303_g->Fill(BMCsz-sz);
          hsv310_g->Fill(SVP, BMCP);
          hsv311_g->Fill(SVP/BMCP);
          hsv312_g->Fill(SVM, SVP/BMCP);
          hsv313_g->Fill(SVM/BMCM);
        } else {
          //pointing
          hsv200_b->Fill(BMCDphiDir);
          hsv201_b->Fill(BMCDTheDir);
          hsv202_b->Fill(BMCDphiVec);
          //for tau
          hsv310_b->Fill(SVP, BMCP);
          hsv311_b->Fill(SVP/BMCP);
          hsv312_b->Fill(SVM, SVP/BMCP);
          hsv313_b->Fill(SVM/BMCM);
        }//distfromtrue
        if (tvtx->decision()){
          if(tvtx->decision()==TrueTag) {
            hsv100_r->Fill(kaoncounter);
            hsv101_r->Fill(leptoncounter);
            hsv102_r->Fill(kaonleptoncounter);
            hsv103_r->Fill(tagpartcounter);
            hsv200_r->Fill(BDphiDir);
            hsv201_r->Fill(BDTheDir);
            hsv300_r->Fill(SVtau);
            if (!containskaon) hsv320_r->Fill(SVtau);
            if (fabs(BOpart->ID())==531) hsv321_r->Fill(SVtau);
            if (fabs(BOpart->ID())==511) hsv322_r->Fill(SVtau);
            if (fabs(BOpart->ID())==521) hsv323_r->Fill(SVtau);
          } else {
            hsv100_w->Fill(kaoncounter);
            hsv101_w->Fill(leptoncounter);
            hsv102_w->Fill(kaonleptoncounter);
            hsv103_w->Fill(tagpartcounter);
            hsv200_w->Fill(BDphiDir);
            hsv201_w->Fill(BDTheDir);
            hsv300_w->Fill(SVtau);
            if (!containskaon) hsv320_w->Fill(SVtau);
            if (fabs(BOpart->ID())==531) hsv321_w->Fill(SVtau);
            if (fabs(BOpart->ID())==511) hsv322_w->Fill(SVtau);
            if (fabs(BOpart->ID())==521) hsv323_w->Fill(SVtau);
          }//truetag
        }
        //** SV vs PU **//
        if (krec>=1 && tvtx->decision()){
          //PileUp
          double zposPU1 = secparts.at(0)->zpos();
          double zposPU2 = secparts.at(0)->zpos2();
          double zposPU3 = secparts.at(0)->zpos3();
          hsv502->Fill(zposPU1); hsv502->Fill(zposPU2); hsv502->Fill(zposPU3);
          int PUcounter=0;
          if (zposPU1!=100000) PUcounter+=1;
          if (zposPU2!=100000) PUcounter+=1;
          if (zposPU3!=100000) PUcounter+=1;
          hsv500->Fill(PUcounter);
          int tracksPU1 = secparts.at(0)->tracks();
          int tracksPU2 = secparts.at(0)->tracks2();
          int tracksPU3 = secparts.at(0)->tracks3();
          int totaltracks = tracksPU1+tracksPU2+tracksPU3;
          //info()<<"tracksPU: "<<tracksPU1<<" "<<tracksPU2<<endreq;
          if (totaltracks == 0) totaltracks = -1;
          hsv501->Fill(tracksPU1); hsv501->Fill(tracksPU2); hsv501->Fill(tracksPU3);
          double PUMag = sqrt(zposPU1*zposPU1+
                              secparts.at(0)->xpos()*secparts.at(0)->xpos()+
                              secparts.at(0)->ypos()*secparts.at(0)->ypos());
          hsv522->Fill(PUMag);
          if (zposPU1!=100000) hsv503->Fill(tracksPU1/totaltracks);//PU according to activity
          if (zposPU2!=100000) hsv503->Fill(tracksPU2/totaltracks);
          if (zposPU3!=100000) hsv503->Fill(tracksPU3/totaltracks);
          //distance SV versus PV and PU
          TVector3 BoppDirnocor = BoppDir+event.RecVertex();//SV z pos wrt (0,0)
          hsv401->Fill(BoppDir.z());//SV z pos wrt PV
          hsv421->Fill(BoppDir.Mag());//SV mag wrt PV 
          TVector3 SVUnit = SVmomentum.Vect().Unit();
          TVector3 PUpos;
          PUpos.SetX(secparts.at(0)->xpos());
          PUpos.SetY(secparts.at(0)->ypos());
          PUpos.SetZ(secparts.at(0)->zpos());
          TVector3 k;//distance SV - PU
          k.SetX(BoppDirnocor.x()-PUpos.x());
          k.SetY(BoppDirnocor.y()-PUpos.y());
          k.SetZ(BoppDirnocor.z()-PUpos.z());
          double cte = SVUnit.Dot(k)/k.Mag();
          double distance = k.Mag() * sqrt(1-cte*cte);//calculate distance SVmomentum and PU
          hsv405->Fill(distance);
          double difzposSVPU = BoppDirnocor.z()-zposPU1;//only first PU
          double difSVPU = (BoppDirnocor-PUpos).Mag();
          hsv402->Fill(difzposSVPU);
          hsv422->Fill(difSVPU);
          double PUPV = fabs(difzposSVPU)-fabs(BoppDir.z());//SV closer to PV or to PU
          double PUPVmag = fabs(difSVPU)-fabs(BoppDir.Mag());
          hsv403->Fill(PUPV);
          hsv423->Fill(PUPVmag);
          if(tvtx->decision()==TrueTag) {
            hsv401_r->Fill(BoppDir.z());
            hsv402_r->Fill(difzposSVPU);
            hsv403_r->Fill(PUPV);
            hsv404_r->Fill(BoppDir.x(),BoppDir.y());//xy plane for SV
            hsv405_r->Fill(distance);
            hsv500_r->Fill(PUcounter);
            hsv501_r->Fill(tracksPU1); hsv501_r->Fill(tracksPU2); hsv501_r->Fill(tracksPU3);
            hsv502_r->Fill(zposPU1); hsv502_r->Fill(zposPU2); hsv502_r->Fill(zposPU3);
            if (zposPU1!=100000) hsv503_r->Fill(tracksPU1/totaltracks);
            if (zposPU2!=100000) hsv503_r->Fill(tracksPU2/totaltracks);
            if (zposPU3!=100000) hsv503_r->Fill(tracksPU3/totaltracks);
          } else {
            hsv401_w->Fill(BoppDir.z());
            hsv402_w->Fill(difzposSVPU);
            hsv403_w->Fill(PUPV);
            hsv404_w->Fill(BoppDir.x(),BoppDir.y());//xy plane for SV
            hsv405_w->Fill(distance);
            hsv500_w->Fill(PUcounter);
            hsv501_w->Fill(tracksPU1); hsv501_w->Fill(tracksPU2); hsv501_w->Fill(tracksPU3);
            hsv502_w->Fill(zposPU1); hsv502_w->Fill(zposPU2); hsv502_w->Fill(zposPU3);
            if (zposPU1!=100000) hsv503_w->Fill(tracksPU1/totaltracks);
            if (zposPU2!=100000) hsv503_w->Fill(tracksPU2/totaltracks);
            if (zposPU3!=100000) hsv503_w->Fill(tracksPU3/totaltracks);
          }//truetag
        }//krec>=1
      }//sec not empty
      //**********************//
      //**********************//

      //***********************//
      //**Multiple PU studies**//
      //***********************//
      Particle*  iparticle = imuon;
      Tagger* tparticle = tmuon;
      //      Particle*  iparticle = iele;
      //      Tagger* tparticle = tele;
      if ((iparticle) and (krec>1)) {
        double gippu=sqrt(iparticle->nippuerr());
        double ipmeandif= (iparticle->IP()-iparticle->ipmean());
        double ipdif=(iparticle->IP()-iparticle->nippu());
        double zposdif=(iparticle->trackzfirst()-iparticle->zpos());
        double zposabsdif=fabs(iparticle->trackzfirst()-iparticle->zpos());
        if (tparticle->decision()==TrueTag) {
          //if (iparticle->fromB()==1) {
          hpv_gippu_r->Fill(gippu);
          hpv_difip_r->Fill(ipdif);
          hpv_ipmeandif_r->Fill(ipmeandif);
          hpv_zposdif_r->Fill(zposdif);
          hpv_zposabsdif_r->Fill(zposabsdif);
        } else {
          hpv_gippu_w->Fill(gippu);
          hpv_difip_w->Fill(ipdif);
          hpv_ipmeandif_w->Fill(ipmeandif);
          hpv_zposdif_w->Fill(zposdif);
          hpv_zposabsdif_w->Fill(zposabsdif);
        }
      }
      //**********************//
      //**********************//

      //plots
      double var = BSpart->pt();
      h6->Fill(var, weight);
      if(tagdecision && tagdecision==TrueTag) h6r->Fill(var, weight); else h6w->Fill(var, weight);
      if(isame){
        if(tsame->decision()==TrueTag) h6rSS->Fill(var, weight); else h6wSS->Fill(var, weight);
      }
      if(tagdecision && isTis){
        if(tagdecision==TrueTag) h6rtis->Fill(var, weight); else h6wtis->Fill(var, weight);
      }
      if(tagdecision && isTos){
        if(tagdecision==TrueTag) h6rtos->Fill(var, weight); else h6wtos->Fill(var, weight);
      }
      if(tagdecision && isTis && isTos){
        if(tagdecision==TrueTag) h6rtistos->Fill(var, weight); else h6wtistos->Fill(var, weight);
      }

      //tagger histos
      if(iele) {
        h1010->Fill( iele->PIDe() );
        if( iele->absMCID() == 11 ) h1000->Fill(iele->PIDe(), weight);
        if( iele->MCID() == 0  )    h1020->Fill(iele->PIDe(), weight);
      }
      if(imuon) {
        h1011->Fill(imuon->PIDm());
        if( imuon->absMCID()== 13 ) h1001->Fill(imuon->PIDm(), weight);
        if( imuon->MCID()== 0  )    h1021->Fill(imuon->MCID(), weight);
      }
      if(ikaon) {
        h1012->Fill(ikaon->PIDk()); h1015->Fill(ikaon->PIDk()-ikaon->PIDp(), weight);
        if( ikaon->absMCID()== 321) h1002->Fill(ikaon->PIDk(), weight);
        if( ikaon->absMCID()== 321) h1005->Fill(ikaon->PIDk()-ikaon->PIDp(), weight);
        if( ikaon->MCID()== 0)      h1022->Fill(ikaon->PIDk(), weight);
        if( ikaon->MCID()== 0)      h1025->Fill(ikaon->PIDk()-ikaon->PIDp(), weight);

        bool okk = (tkaon->decision() == TrueTag);
        bool k3  = (krec==3);
        bool k1  = (krec==1);
        h70->Fill(ikaon->pt(), weight);if(okk&&k1)   h80->Fill(ikaon->pt(), weight);
        if(okk&&k3) h80_3->Fill(ikaon->pt(), weight); 
        h71->Fill(ikaon->p(), weight); if(okk&&k1)   h81->Fill(ikaon->p(), weight); 
        if(okk&&k3) h81_3->Fill(ikaon->p(), weight); h72->Fill(ikaon->IPs(), weight); 
        if(okk&&k1) h82->Fill(ikaon->IPs(), weight);
        if(okk&&k3) h82_3->Fill(ikaon->IPs(), weight);
        h73->Fill(ikaon->LCS(), weight);  if(okk&&k1) h83->Fill(ikaon->LCS(), weight); 
        if(okk&&k3) h83_3->Fill(ikaon->LCS(), weight); 
        h74->Fill(ikaon->likelihood(), weight);if(okk&&k1) h84->Fill(ikaon->likelihood(), weight);
        if(okk&&k3) h84_3->Fill(ikaon->likelihood(), weight); 
        h75->Fill(ikaon->PIDk(), weight);  if(okk&&k1) h85->Fill(ikaon->PIDk(), weight); 
        if(okk&&k3) h85_3->Fill(ikaon->PIDk(), weight); 
        h76->Fill(ikaon->PIDk()-ikaon->PIDp(), weight);  
        if(okk&&k1) h86->Fill(ikaon->PIDk()-ikaon->PIDp(), weight); 
        if(okk&&k3) h86_3->Fill(ikaon->PIDk()-ikaon->PIDp(), weight); 
        h77->Fill(ikaon->distPhi(), weight);  
        if(okk&&k1) h87->Fill(ikaon->distPhi(), weight); 
        if(okk&&k3) h87_3->Fill(ikaon->distPhi(), weight); 
      }
      if(isBs && isame) {
        h1013->Fill(isame->PIDk(), weight);  h1014->Fill(isame->PIDk()-isame->PIDp(), weight);
        if( isame->absMCID()== 321 ) 
        { h1003->Fill(isame->PIDk(), weight); h1004->Fill(isame->PIDk()-isame->PIDp(), weight); }
        if( isame->absMCID()== 0   ) 
        { h1023->Fill(isame->PIDk(), weight); h1024->Fill(isame->PIDk()-isame->PIDp(), weight); }
      }
      
      /////////////////////////
      if(imuon) { 
        if (tagdecisionOS==TrueTag) hr_mu_probComb->Fill(1-tmuon->omega(),weight);
        if (tagdecisionOS==-TrueTag) hw_mu_probComb->Fill(1-tmuon->omega(),weight);
        if(tmuon->decision()==TrueTag) { 
          hr_mu_p->Fill(imuon->p(), weight); 
          hr_mu_pt->Fill(imuon->pt(), weight); 
          hr_mu_ips->Fill(imuon->IPs(), weight); 
          hr_mu_ipssign->Fill(imuon->IPssign(), weight); 
          if(krec>1) hr_mu_ippu->Fill(imuon->IPPU(), weight);
          hr_mu_lcs->Fill(imuon->LCS(), weight); 
          hr_mu_pid->Fill(imuon->PIDm(), weight); 
          hr_mu_pidNN->Fill(imuon->PIDNNm(), weight); 
          hr_mu_tsal->Fill(imuon->likelihood(), weight); 
          hr_mu_mult->Fill(nparts, weight);         
          homphi_mu_r->Fill(imuon->distPhi(), weight);
          hr_mu_theta->Fill(acos(imuon->pt()/imuon->p()), weight); 
          hr_mu_prob->Fill(1-tmuon->omega(), weight);
        } else {
          hw_mu_p->Fill(imuon->p(), weight); 
          hw_mu_pt->Fill(imuon->pt(), weight);
          hw_mu_ips->Fill(imuon->IPs(), weight); 
          hw_mu_ipssign->Fill(imuon->IPssign(), weight); 
          if(krec>1) hw_mu_ippu->Fill(imuon->IPPU(), weight);
          hw_mu_lcs->Fill(imuon->LCS(), weight); 
          hw_mu_pid->Fill(imuon->PIDm(), weight); 
          hw_mu_pidNN->Fill(imuon->PIDNNm(), weight); 
          hw_mu_tsal->Fill(imuon->likelihood(), weight); 
          hw_mu_mult->Fill(nparts, weight); 
          homphi_mu_w->Fill(imuon->distPhi(), weight);
          hw_mu_theta->Fill(acos(imuon->pt()/imuon->p()), weight); 
          hw_mu_prob->Fill(1-tmuon->omega(), weight);
         }
        if(iele)  eta_corr[0][1]->Fill(tmuon->omega(),tele->omega(),weight);        
        if(ikaon) eta_corr[0][2]->Fill(tmuon->omega(),tkaon->omega(),weight);        
        if(tvtx) eta_corr[0][3]->Fill(tmuon->omega(),tvtx->omega(),weight);        
        if(isame) eta_corr[0][4]->Fill(tmuon->omega(),tsame->omega(),weight);        
      }
      if(iele) { 
        if (tagdecisionOS==TrueTag) hr_ele_probComb->Fill(1-tele->omega(),weight);
        if (tagdecisionOS==-TrueTag) hw_ele_probComb->Fill(1-tele->omega(),weight);
        if(tele->decision()==TrueTag) { 
          hr_ele_p->Fill(iele->p(), weight); 
          hr_ele_pt->Fill(iele->pt(), weight); 
          hr_ele_ips->Fill(iele->IPs(), weight); 
          if (iele->IPssign()>0) hr_ele_ipssign->Fill(iele->IPssign(), weight); else hr_ele_ipssign1->Fill(iele->IPssign(), weight); 
          if(krec>1) hr_ele_ippu->Fill(iele->IPPU(), weight);
          hr_ele_lcs->Fill(iele->LCS(), weight); 
          hr_ele_pid->Fill(iele->PIDe(), weight); 
          hr_ele_pidNN->Fill(iele->PIDNNe(), weight); 
          hr_ele_tsal->Fill(iele->likelihood(), weight); 
          hr_ele_mult->Fill(nparts, weight); 
          hr_ele_veloch->Fill(iele->VeloCharge(), weight); 
          hr_ele_eOverP->Fill(iele->eOverP(), weight); 
          homphi_ele_r->Fill(iele->distPhi(), weight);
          hr_ele_theta->Fill(acos(iele->pt()/iele->p()), weight); 
          hr_ele_prob->Fill(1-tele->omega(), weight);
        } else {
          hw_ele_p->Fill(iele->p(), weight); 
          hw_ele_pt->Fill(iele->pt(), weight);
          hw_ele_ips->Fill(iele->IPs(), weight); 
          if (iele->IPssign()>0) hw_ele_ipssign->Fill(iele->IPssign(), weight); else hw_ele_ipssign1->Fill(iele->IPssign(), weight); 
          if(krec>1) hw_ele_ippu->Fill(iele->IPPU(), weight);
          hw_ele_lcs->Fill(iele->LCS(), weight); 
          hw_ele_pid->Fill(iele->PIDe(), weight); 
          hw_ele_pidNN->Fill(iele->PIDNNe(), weight); 
          hw_ele_tsal->Fill(iele->likelihood(), weight); 
          hw_ele_mult->Fill(nparts, weight); 
          hw_ele_veloch->Fill(iele->VeloCharge(), weight); 
          hw_ele_eOverP->Fill(iele->eOverP(), weight); 
          homphi_ele_w->Fill(iele->distPhi(), weight);
          hw_ele_theta->Fill(acos(iele->pt()/iele->p()), weight); 
          hw_ele_prob->Fill(1-tele->omega(), weight);
        }
        if(ikaon) eta_corr[1][2]->Fill(tele->omega(),tkaon->omega(),weight);        
        if(tvtx) eta_corr[1][3]->Fill(tele->omega(),tvtx->omega(),weight);        
        if(isame) eta_corr[1][4]->Fill(tele->omega(),tsame->omega(),weight);        
      }
      if(ikaon) { 
        if (tagdecisionOS==TrueTag) hr_k_probComb->Fill(1-tkaon->omega(),weight);
        if (tagdecisionOS==-TrueTag) hw_k_probComb->Fill(1-tkaon->omega(),weight);
        if(tkaon->decision()==TrueTag) { 
          hr_k_p->Fill(ikaon->p(), weight); 
          hr_k_pt->Fill(ikaon->pt(), weight); 
          hr_k_ips->Fill(ikaon->IPs(), weight); 
          hr_k_ip->Fill(ikaon->IP(), weight); 
          hr_k_ipssign->Fill(ikaon->IPssign(), weight); 
          if (ikaon->IPssign()>0) hr_k_ipssign->Fill(ikaon->IPssign(), weight); else hr_k_ipssign1->Fill(ikaon->IPssign(), weight); 
          if(krec>1) hr_k_ippu->Fill(ikaon->IPPU(), weight);
          hr_k_lcs->Fill(ikaon->LCS(), weight); 
          hr_k_pid->Fill(ikaon->PIDk(), weight); 
          hr_k_pidNNk->Fill(ikaon->PIDNNk(), weight); 
          hr_k_pidNNp->Fill(ikaon->PIDNNp(), weight); 
          hr_k_pidNNpi->Fill(ikaon->PIDNNpi(), weight); 
          hr_k_pidkp->Fill(ikaon->PIDk()-ikaon->PIDp(), weight); 
          hr_k_tsal->Fill(ikaon->likelihood(), weight); 
          hr_k_mult->Fill(nparts, weight); 
          homphi_kO_r->Fill(ikaon->distPhi(), weight);
          hr_k_theta->Fill(acos(ikaon->pt()/ikaon->p()), weight);        
          hr_k_prob->Fill(1-tkaon->omega(), weight);
        } else {
          hw_k_p->Fill(ikaon->p(), weight); 
          hw_k_pt->Fill(ikaon->pt(), weight);
          hw_k_ips->Fill(ikaon->IPs(), weight);
          hw_k_ip->Fill(ikaon->IP(), weight); 
          hw_k_ipssign->Fill(ikaon->IPssign(), weight); 
          if (ikaon->IPssign()>0) hw_k_ipssign->Fill(ikaon->IPssign(), weight); else hw_k_ipssign1->Fill(ikaon->IPssign(), weight);
          if(krec>1) hw_k_ippu->Fill(ikaon->IPPU(), weight);
          hw_k_lcs->Fill(ikaon->LCS(), weight); 
          hw_k_pid->Fill(ikaon->PIDk(), weight); 
          hw_k_pidNNk->Fill(ikaon->PIDNNk(), weight); 
          hw_k_pidNNp->Fill(ikaon->PIDNNp(), weight); 
          hw_k_pidNNpi->Fill(ikaon->PIDNNpi(), weight); 
          hw_k_pidkp->Fill(ikaon->PIDk()-ikaon->PIDp(), weight); 
          hw_k_tsal->Fill(ikaon->likelihood(), weight); 
          hw_k_mult->Fill(nparts, weight); 
          homphi_kO_w->Fill(ikaon->distPhi(), weight);
          hw_k_theta->Fill(acos(ikaon->pt()/ikaon->p()), weight);        
          hw_k_prob->Fill(1-tkaon->omega(), weight);
        }
        if(tvtx) eta_corr[2][3]->Fill(tkaon->omega(),tvtx->omega(),weight);        
        if(isame) eta_corr[2][4]->Fill(tkaon->omega(),tsame->omega(),weight);        
       }
      if(isame) { 
        if (tagdecisionOS==TrueTag) hr_kS_probComb->Fill(1-tsame->omega(),weight);
        if (tagdecisionOS==-TrueTag) hw_kS_probComb->Fill(1-tsame->omega(),weight);
#ifdef MCARLOTEST
        TLorentzVector ptotB = BSpart->momentumMC();
        cout<<" BSpart "<< BSpart->momentumMC().P()<<endl;
        double B0the= ptotB.Theta();
        double B0phi= ptotB.Phi();
        double ang = asin((isame->pt())/(isame->p()));
        double deta= log(tan(B0the/2.))-log(tan(ang/2.));
        double dphi= std::min(fabs(isame->momentum().Phi()-B0phi),
                              6.283-fabs(isame->momentum().Phi()-B0phi));
        double dR = sqrt(deta*deta+dphi*dphi);
        double B0mass = ptotB.M();
        double dQ  = ((ptotB + isame->momentum() ).M() - B0mass);
        /*
        // with the true momentum
        if(tree.enableMC){
          
        TLorentzVector T_ptotB = BSpart->momentumMC();
        //cout<<" T_ptotB "<<T_ptotB.P()<<" "<<T_ptotB.M()<<endl;
        
        double T_B0the= T_ptotB.Theta();
        double T_B0phi= T_ptotB.Phi();
        double T_deta = log(tan(T_B0the/2.))-log(tan(ang/2.));
        double T_dphi = std::min(fabs(isame->momentum().Phi()-T_B0phi),
                              6.283-fabs(isame->momentum().Phi()-T_B0phi));
        double T_dR = sqrt(T_deta*T_deta+T_dphi*T_dphi);
        double T_B0mass = T_ptotB.M();
        double T_dQ  = ((T_ptotB + isame->momentum() ).M() - T_B0mass);

        h_kFactor_dQ->Fill(dQ,T_dQ/dQ,weight);
        k_kFactor_dR->Fill(dR,T_dR/dR,weight);
        h_kFactor_deta->Fill(deta,T_deta/deta,weight);
        h_kFactor_dphi->Fill(dphi,T_dphi/dphi,weight);
        
        h_T_dQ->Fill(T_dQ,weight);
        h_dQ->Fill(dQ,weight);

        h_T_dR->Fill(T_dR,weight);
        h_dR->Fill(dR,weight);
        
        h_T_deta->Fill(T_deta,weight);
        h_deta->Fill(deta,weight);

        h_T_dphi->Fill(T_dphi,weight);
        h_dphi->Fill(dphi,weight);
        
        }
         */
#else
        TLorentzVector ptotB = BSpart->momentum();
        double B0the= ptotB.Theta();
        double B0phi= ptotB.Phi();
        double ang = asin((isame->pt())/(isame->p()));
        double deta= log(tan(B0the/2.))-log(tan(ang/2.));
        double dphi= std::min(fabs(isame->momentum().Phi()-B0phi),
                              6.283-fabs(isame->momentum().Phi()-B0phi));
        double dR = sqrt(deta*deta+dphi*dphi);
        double B0mass = ptotB.M();
        double dQ  = ((ptotB + isame->momentum() ).M() - B0mass);
#endif
        
        
        if(tsame->decision()==TrueTag) { 
          hr_kS_p->Fill(isame->p(), weight); 
          hr_kS_pt->Fill(isame->pt(), weight); 
          hr_kS_ips->Fill(isame->IPs(), weight); 
          hr_kS_ipssign->Fill(isame->IPssign(), weight);        
          hr_kS_ipsign->Fill(isame->IPsign(), weight);        
          if (isame->IPssign()>0) hr_kS_ipssign->Fill(isame->IPssign(), weight); else hr_kS_ipssign1->Fill(isame->IPssign(), weight);
          if(krec>1) hr_kS_ippu->Fill(isame->IPPU(), weight);
          hr_kS_lcs->Fill(isame->LCS(), weight); 
          hr_kS_pid->Fill(isame->PIDk(), weight); 
          hr_kS_pidNNk->Fill(isame->PIDNNk(), weight); 
          hr_kS_pidNNp->Fill(isame->PIDNNp(), weight); 
          hr_kS_pidNNpi->Fill(isame->PIDNNpi(), weight); 
          hr_kS_pidp->Fill(isame->PIDp(), weight); 
          hr_kS_dq->Fill(dQ, weight); 
          hr_kS_deta->Fill(deta, weight); 
          hr_kS_fabsdeta->Fill(fabs(deta), weight); 
          hr_kS_dphi->Fill(dphi, weight); 
          hr_kS_dr->Fill(dR, weight); 
          hr_kS_tsal->Fill(isame->likelihood(), weight); 
          hr_kS_mult->Fill(nparts, weight); 
      	  homphi_kS_r->Fill(isame->distPhi(), weight);
          hr_kS_theta->Fill(acos(isame->pt()/isame->p()), weight);        
          hr_kS_prob->Fill(1-tsame->omega(), weight);
        }
        if(tsame->decision()==-TrueTag) { 
          hw_kS_p->Fill(isame->p(), weight); 
          hw_kS_pt->Fill(isame->pt(), weight);
          hw_kS_ips->Fill(isame->IPs(), weight); 
          hw_kS_ipssign->Fill(isame->IPssign(), weight);        
          hw_kS_ipsign->Fill(isame->IPsign(), weight);        
          if (isame->IPssign()>0) hw_kS_ipssign->Fill(isame->IPssign(), weight); else hw_kS_ipssign1->Fill(isame->IPssign(), weight);
          if(krec>1) hw_kS_ippu->Fill(isame->IPPU(), weight);
          hw_kS_lcs->Fill(isame->LCS(), weight); 
          hw_kS_pid->Fill(isame->PIDk(), weight); 
          hw_kS_pidp->Fill(isame->PIDp(), weight); 
          hw_kS_pidNNk->Fill(isame->PIDNNk(), weight); 
          hw_kS_pidNNp->Fill(isame->PIDNNp(), weight); 
          hw_kS_pidNNpi->Fill(isame->PIDNNpi(), weight);          
          hw_kS_dq->Fill(dQ, weight); 
          hw_kS_deta->Fill(deta, weight); 
          hw_kS_fabsdeta->Fill(fabs(deta), weight); 
          hw_kS_dphi->Fill(dphi, weight); 
          hw_kS_dr->Fill(dR, weight); 
          hw_kS_tsal->Fill(isame->likelihood(), weight); 
          hw_kS_mult->Fill(nparts, weight); 
          homphi_kS_w->Fill(isame->distPhi(), weight);
          hw_kS_theta->Fill(acos(isame->pt()/isame->p()), weight);        
          hw_kS_prob->Fill(1-tsame->omega(), weight);
        }
      }
      if(tvtx){
        double VtxSumpt=0, VtxSumips=0, VtxSumdoca=0, VtxM=0, VtxP=0, VtxLike=0, VtxDphi=0, VtxMag=0;
        if( !secvtx.empty()) {
          if (tagdecisionOS==TrueTag) hr_vtx_probComb->Fill(1-tvtx->omega(),weight);
          if (tagdecisionOS==-TrueTag) hw_vtx_probComb->Fill(1-tvtx->omega(),weight);
          TVector3 VtxPos = secvtx.at(0)->position();
          int nsecpart = secvtx.at(0)->outgoingParticles().size();
          VtxLike = secvtx.at(0)->likelihood();
          Particles secparts = secvtx.at(0)->outgoingParticles();
          TLorentzVector Vtxmomentum;
          for (int ip=0; ip<nsecpart; ip++) {
            Vtxmomentum += secparts.at(ip)->momentum();
            VtxSumpt += secparts.at(ip)->pt();
            VtxSumips += secparts.at(ip)->IPs();
            VtxSumdoca += secparts.at(ip)->DOCA();
          }
          VtxM = Vtxmomentum.M();
          VtxP = Vtxmomentum.P();
          VtxMag = Vtxmomentum.Mag();
          VtxDphi = VtxPos.DeltaPhi(Vtxmomentum.Vect());
        }
        debug()<<"Vdec: "<<tvtx->decision()<<"Vch: "<<tvtx->charge()<<endreq;
        if(tvtx->decision()==TrueTag) {
          hr_vtx_ch->Fill(fabs(tvtx->charge()), weight); //abs(Vch)
          hr_vtx_prob->Fill(1-tvtx->omega(), weight);
          hr_vtx_like->Fill(VtxLike, weight);
          hr_vtx_dphidir->Fill(VtxDphi, weight);
          hr_vtx_sumpt->Fill(VtxSumpt, weight);
          hr_vtx_sumips->Fill(VtxSumips, weight);
          hr_vtx_sumdoca->Fill(VtxSumdoca, weight);
          hr_vtx_M->Fill(VtxM, weight);
          hr_vtx_P->Fill(VtxP, weight);
          hr_vtx_Mag->Fill(VtxMag, weight);
        }
        if(tvtx->decision()==-1*TrueTag) {
          hw_vtx_ch->Fill(fabs(tvtx->charge()), weight); 
          hw_vtx_prob->Fill(1-tvtx->omega(), weight);
          hw_vtx_like->Fill(VtxLike, weight);
          hw_vtx_dphidir->Fill(VtxDphi, weight);
          hw_vtx_sumpt->Fill(VtxSumpt, weight);
          hw_vtx_sumips->Fill(VtxSumips, weight);
          hw_vtx_sumdoca->Fill(VtxSumdoca, weight);
          hw_vtx_M->Fill(VtxM, weight);
          hw_vtx_P->Fill(VtxP, weight);
          hw_vtx_Mag->Fill(VtxMag, weight);
        }
        if(tvtx->decision()==0) hu_vtx_ch->Fill(fabs(tvtx->charge()), weight); 
        if(isame) eta_corr[3][4]->Fill(tvtx->omega(),tsame->omega(),weight);        
      }

      //more histos:
      if(TrueTag== tagdecision) {h100->Fill(1-omega,weight); h103->Fill(omega,weight);}
      if(TrueTag==-tagdecision) {h102->Fill(1-omega,weight); h104->Fill(omega,weight);}
      
      if(imuon){
        if(tmuon->decision()==TrueTag) h613->Fill(tmuon->RawNNetOutput(),weight); 
        else h513->Fill(tmuon->RawNNetOutput(),weight);
        if(tmuon->decision()==TrueTag) h613p->Fill(tmuon->omega(),weight); 
        else h513p->Fill(tmuon->omega(),weight);
      }
      if(iele){
        if(tele->decision()==TrueTag) h611->Fill(tele->RawNNetOutput(),weight);
        else h511->Fill(tele->RawNNetOutput(),weight);
        if(tele->decision()==TrueTag) h611p->Fill(tele->omega(),weight);
        else h511p->Fill(tele->omega(),weight);
      }
      if(ikaon){
        if(tkaon->decision()==TrueTag) h6321->Fill(tkaon->RawNNetOutput(),weight); 
        else h5321->Fill(tkaon->RawNNetOutput(),weight);
        if(tkaon->decision()==TrueTag) h6321p->Fill(tkaon->omega(),weight); 
        else h5321p->Fill(tkaon->omega(),weight);
      }
      if(isame){
        if(tsame->decision()==TrueTag) h535->Fill(tsame->RawNNetOutput(),weight); 
        else h635->Fill(tsame->RawNNetOutput(),weight);
        if(tsame->decision()==TrueTag) h535p->Fill(tsame->omega(),weight); 
        else h635p->Fill(tsame->omega(),weight);	
      }
      if(tvtx->decision()){
        if(tvtx->decision()==TrueTag) h540->Fill(tvtx->RawNNetOutput(),weight); 
        else h640->Fill(tvtx->RawNNetOutput(),weight);
        if(tvtx->decision()==TrueTag) h540p->Fill(tvtx->omega(),weight); 
        else h640p->Fill(tvtx->omega(),weight);	
      }

      //omega plot for taggers
      if (tagdecisionOS==TrueTag)  hr_probComb_OS->Fill(1-omegaOS,weight);
      if (tagdecisionOS==-TrueTag) hw_probComb_OS->Fill(1-omegaOS,weight);
      if (tagdecision==TrueTag)  hr_probComb->Fill(1-omega,weight);
      if (tagdecision==-TrueTag) hw_probComb->Fill(1-omega,weight);
      if (imuon) hom_mu->Fill(tmuon->omega(),weight);
      if (iele) hom_e->Fill(tele->omega(),weight);
      if (ikaon) hom_k->Fill(tkaon->omega(),weight);
      if (isame) hom_ss->Fill(tsame->omega(),weight);
      if (tvtx->decision()) hom_vtx->Fill(tvtx->omega(),weight);
      if (theTag->decisionOS()) hom_combOS->Fill(theTag->omegaOS(),weight);
      if (theTag->decision()) hom_combOSSS->Fill(theTag->omega(),weight);

      //oscillation plots:
      int Btag = TrueTag;
      if(imuon) { if(Btag*tmuon->decision()>0)
          h2011->Fill(Btime, weight); else h2012->Fill(Btime, weight); }
      if(iele) { if(Btag*tele->decision()>0) 
          h2021->Fill(Btime, weight); else h2022->Fill(Btime, weight); }
      if(ikaon) { if(Btag*tkaon->decision()>0) 
          h2031->Fill(Btime, weight); else h2032->Fill(Btime, weight); }
      if(isame) { if(Btag*tsame->decision()>0) 
          h2041->Fill(Btime, weight); else h2042->Fill(Btime, weight); }
      if(tvtx->decision())  { if(Btag*tvtx->decision()>0) 
          h2051->Fill(Btime, weight); else h2052->Fill(Btime, weight); }

      int iosc = TrueTag*tagdecision;
      if     (category==1) {if(iosc>0) h3011->Fill(Btime, weight);else h3012->Fill(Btime, weight);}
      else if(category==2) {if(iosc>0) h3021->Fill(Btime, weight);else h3022->Fill(Btime, weight);}
      else if(category==3) {if(iosc>0) h3031->Fill(Btime, weight);else h3032->Fill(Btime, weight);}
      else if(category==4) {if(iosc>0) h3041->Fill(Btime, weight);else h3042->Fill(Btime, weight);}
      else if(category==5) {if(iosc>0) h3051->Fill(Btime, weight);else h3052->Fill(Btime, weight);}
      if     (category!=0) {if(iosc>0) h3061->Fill(Btime, weight);else h3062->Fill(Btime, weight);}
      
      if(tree.enableMC) {
        if( BSpart->hasOscillated() ) iosc = -iosc;
        if( BSpart->hasOscillated() ) iosc = -1*TrueTag; else iosc = 1*TrueTag;
        if(iosc>0) h4061->Fill(Btime, weight); else h4062->Fill(Btime, weight);
      }

      //dump some info if in debug mode////////////////////////
      if(msgLevel(MSG::DEBUG)  //&& tagdecision && event.multiplicity()<15
         ) {
        event.Print(); theTag->Print();
        Particles::iterator ipa;
        int npcount=1;
        for(ipa=parts.begin(); ipa!=parts.end(); ipa++) {
          debug()<< npcount++; (*ipa)->Print();
        }
      }
    } //end of loop on event

  }//end of loop on files

  //////////////////////////////////////////////////////////////////////////

  //dump full list of declared properties:
  propertyMap::const_iterator ipro;
  for( ipro=property.begin(); ipro!=property.end(); ++ipro ) {
    if(msgLevel(MSG::DEBUG)) 
      debug()<<"Property "<<(*ipro).first<<" = "<<(*ipro).second<<endreq;
  }

  if(NNetTrain!="none") {
    nnetune.Write();
    nnetune.TrainNNet(usesWeights);
  } else {
    perfmon.printStats();
  }
 
  fhisto.Write(); fhisto.Close();

  //viewer.DrawLHCb(); //draw openGL view of detector
  if(property["ViewOptionLHCb"]>=0)
    viewer.DrawStoredEvents(); //also draw tracks on top of it

  timesum += (clock()-timestart)/(double)CLOCKS_PER_SEC;
  info()<<timesum<<" seconds elapsed time ("
        <<timesum*1000./((double)perfmon.N()+1)<<" ms/evt)."<<endmsg;

  return 0;
} 
